@@ -1,17 +1,18 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __AFSD_H_ENV__
-#define __AFSD_H_ENV__ 1
+#ifndef OPENAFS_WINNT_AFSD_AFSD_H
+#define OPENAFS_WINNT_AFSD_AFSD_H 1
 
 #define USE_BPLUS 1
 
+#include <afsconfig.h>
 #include <afs/param.h>
 
 BOOL InitClass(HANDLE);
@@ -59,6 +60,7 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include "cm_memmap.h"
 #include "cm_freelance.h"
 #include "cm_performance.h"
+#include "cm_rdr.h"
 #include "afsd_init.h"
 #include "afsd_eventlog.h"
 
@@ -110,13 +112,18 @@ extern int cm_fakeGettingCallback;			// 1 if currently updating the fake root.af
 #endif /* AFS_FREELANCE_CLIENT */
 
 extern int cm_dnsEnabled;
-extern int cm_freelanceEnabled;
 extern int cm_readonlyVolumeVersioning;
 
 extern long rx_mtu;
 
 extern HANDLE WaitToTerminate;
 
+extern int RDR_Initialized;
+
+extern afs_uint32 smb_Enabled;
+
+extern int cm_virtualCache;
+
 #define DFS_SUPPORT 1
 #define LOG_PACKET 1
 #undef  NOTSERVICE
@@ -141,4 +148,4 @@ typedef BOOL ( APIENTRY * AfsdStoppingHook )(void);
 typedef BOOL ( APIENTRY * AfsdStoppedHook )(void);
 
 #define SERVICE_CONTROL_CUSTOM_DUMP 128
-#endif /* AFSD_H_ENV */
+#endif /* OPENAFS_WINNT_AFSD_AFSD_H */
