@@ -7,7 +7,10 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
+#include <roken.h>
+
 #include <afs/stds.h>
 
 #include <windows.h>
@@ -27,7 +30,6 @@ DWORD RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
 unsigned short ConnDeadtimeout = CM_CONN_CONNDEADTIME;
 unsigned short HardDeadtimeout = CM_CONN_HARDDEADTIME;
 unsigned short IdleDeadtimeout = CM_CONN_IDLEDEADTIME;
-unsigned short ReplicaIdleDeadtimeout = CM_CONN_IDLEDEADTIME_REP;
 unsigned short NatPingInterval = CM_CONN_NATPINGINTERVAL;
 
 #define LANMAN_WKS_PARAM_KEY "SYSTEM\\CurrentControlSet\\Services\\lanmanworkstation\\parameters"
@@ -125,13 +127,6 @@ void cm_InitConn(void)
                 afsi_log("IdleDeadTimeout is %d", IdleDeadtimeout);
             }
 	    dummyLen = sizeof(DWORD);
-	    code = RegQueryValueEx(parmKey, "ReplicaIdleDeadTimeout", NULL, NULL,
-				    (BYTE *) &dwValue, &dummyLen);
-	    if (code == ERROR_SUCCESS) {
-                ReplicaIdleDeadtimeout = (unsigned short)dwValue;
-                afsi_log("ReplicaIdleDeadTimeout is %d", ReplicaIdleDeadtimeout);
-            }
-	    dummyLen = sizeof(DWORD);
 	    code = RegQueryValueEx(parmKey, "NatPingInterval", NULL, NULL,
 				    (BYTE *) &dwValue, &dummyLen);
 	    if (code == ERROR_SUCCESS) {
@@ -147,33 +142,38 @@ void cm_InitConn(void)
          * the smb redirector session timeout (RDRtimeout).
          *
          * The UNIX cache manager uses 120 seconds for the hard dead
-         * timeout and 1200 seconds for the connection and idle timeouts.
+         * timeout and 50 seconds for the connection and idle timeouts.
          *
          * We base our values on those while making sure we leave
          * enough time for overhead.
-         *
-         * To further complicate matters we need to take into account
-         * file server hard dead timeouts as they affect the length
-         * of time it takes the file server to give up when attempting
-         * to break callbacks to unresponsive clients.  The file
-         * server hard dead timeout is 120 seconds.
          */
-        afsi_log("lanmanworkstation : SessTimeout %u", RDRtimeout);
-        if (ConnDeadtimeout == 0) {
-            ConnDeadtimeout = (unsigned short) ((RDRtimeout / 2) < 50 ? (RDRtimeout / 2) : 50);
-            afsi_log("ConnDeadTimeout is %d", ConnDeadtimeout);
-        }
-        if (HardDeadtimeout == 0) {
-            HardDeadtimeout = (unsigned short) (RDRtimeout > 125 ? 120 : (RDRtimeout - 5));
-            afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
-        }
-        if (IdleDeadtimeout == 0) {
-            IdleDeadtimeout = 10 * (unsigned short) HardDeadtimeout;
-            afsi_log("IdleDeadTimeout is %d", IdleDeadtimeout);
-        }
-        if (ReplicaIdleDeadtimeout == 0) {
-            ReplicaIdleDeadtimeout = (unsigned short) HardDeadtimeout;
-            afsi_log("ReplicaIdleDeadTimeout is %d", ReplicaIdleDeadtimeout);
+        if (smb_Enabled) {
+            afsi_log("lanmanworkstation : SessTimeout %u", RDRtimeout);
+            if (ConnDeadtimeout == 0) {
+                ConnDeadtimeout = (unsigned short) ((RDRtimeout / 2) < 50 ? (RDRtimeout / 2) : 50);
+                afsi_log("ConnDeadTimeout is %d", ConnDeadtimeout);
+            }
+            if (HardDeadtimeout == 0) {
+                HardDeadtimeout = (unsigned short) (RDRtimeout > 125 ? 120 : (RDRtimeout - 5));
+                afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
+            }
+            if (IdleDeadtimeout == 0) {
+                IdleDeadtimeout = (unsigned short) ConnDeadtimeout;
+                afsi_log("IdleDeadTimeout is %d", IdleDeadtimeout);
+            }
+        } else {
+            if (ConnDeadtimeout == 0) {
+                ConnDeadtimeout = CM_CONN_IFS_CONNDEADTIME;
+                afsi_log("ConnDeadTimeout is %d", ConnDeadtimeout);
+            }
+            if (HardDeadtimeout == 0) {
+                HardDeadtimeout = CM_CONN_IFS_HARDDEADTIME;
+                afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
+            }
+            if (IdleDeadtimeout == 0) {
+                IdleDeadtimeout = CM_CONN_IFS_IDLEDEADTIME;
+                afsi_log("IdleDeadTimeout is %d", IdleDeadtimeout);
+            }
         }
 	osi_EndOnce(&once);
     }
@@ -186,11 +186,10 @@ void cm_InitReq(cm_req_t *reqp)
 }
 
 static long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
-	struct cm_req *reqp, afs_uint32 *replicated, cm_serverRef_t ***serversppp)
+	struct cm_req *reqp, cm_serverRef_t ***serversppp)
 {
     long code;
     cm_volume_t *volp = NULL;
-    cm_vol_state_t *volstatep = NULL;
     cm_cell_t *cellp = NULL;
 
     if (!fidp) {
@@ -206,8 +205,6 @@ static long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
     if (code)
         return code;
 
-    volstatep = cm_VolumeStateByID(volp, fidp->volume);
-    *replicated = (volstatep->flags & CM_VOL_STATE_FLAG_REPLICATED);
     *serversppp = cm_GetVolServers(volp, fidp->volume, userp, reqp);
 
     lock_ObtainRead(&cm_volumeLock);
@@ -216,39 +213,6 @@ static long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
     return (*serversppp ? 0 : CM_ERROR_NOSUCHVOLUME);
 }
 
-void
-cm_SetServerBusyStatus(cm_serverRef_t *serversp, cm_server_t *serverp)
-{
-    cm_serverRef_t *tsrp;
-
-    lock_ObtainWrite(&cm_serverLock);
-    for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
-        if (tsrp->status == srv_deleted)
-            continue;
-        if (cm_ServerEqual(tsrp->server, serverp) && tsrp->status == srv_not_busy) {
-            tsrp->status = srv_busy;
-            break;
-        }
-    }
-    lock_ReleaseWrite(&cm_serverLock);
-}
-
-void
-cm_ResetServerBusyStatus(cm_serverRef_t *serversp)
-{
-    cm_serverRef_t *tsrp;
-
-    lock_ObtainWrite(&cm_serverLock);
-    for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
-        if (tsrp->status == srv_deleted)
-            continue;
-        if (tsrp->status == srv_busy) {
-            tsrp->status = srv_not_busy;
-        }
-    }
-    lock_ReleaseWrite(&cm_serverLock);
-}
-
 /*
  * Analyze the error return from an RPC.  Determine whether or not to retry,
  * and if we're going to retry, determine whether failover is appropriate,
@@ -266,15 +230,11 @@ cm_ResetServerBusyStatus(cm_serverRef_t *serversp)
  * volSyncp and/or cbrp may also be NULL.
  */
 int
-cm_Analyze(cm_conn_t *connp,
-           cm_user_t *userp,
-           cm_req_t *reqp,
+cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
            struct cm_fid *fidp,
-           afs_uint32 storeOp,
            AFSVolSync *volSyncp,
            cm_serverRef_t * serversp,
-           cm_callbackRequest_t *cbrp,
-           long errorCode)
+           cm_callbackRequest_t *cbrp, long errorCode)
 {
     cm_server_t *serverp = NULL;
     cm_serverRef_t **serverspp = NULL;
@@ -283,8 +243,6 @@ cm_Analyze(cm_conn_t *connp,
     cm_ucell_t *ucellp;
     cm_volume_t * volp = NULL;
     cm_vol_state_t *statep = NULL;
-    cm_scache_t * scp = NULL;
-    afs_uint32 replicated;
     int retry = 0;
     int free_svr_list = 0;
     int dead_session;
@@ -327,7 +285,10 @@ cm_Analyze(cm_conn_t *connp,
 
     /* timeleft - get it from reqp the same way as cm_ConnByMServers does */
     timeUsed = (GetTickCount() - reqp->startTime) / 1000;
-    timeLeft = HardDeadtimeout - timeUsed;
+    if ( reqp->flags & CM_REQ_SOURCE_SMB )
+        timeLeft = HardDeadtimeout - timeUsed;
+    else
+        timeLeft = 0x0FFFFFFF;
 
     /* get a pointer to the cell */
     if (errorCode) {
@@ -405,20 +366,6 @@ cm_Analyze(cm_conn_t *connp,
             format = "All servers are offline when accessing cell %s volume %d.";
 	    LogEvent(EVENTLOG_WARNING_TYPE, msgID, cellp->name, fidp->volume);
 
-            if (!serversp) {
-                code = cm_GetServerList(fidp, userp, reqp, &replicated, &serverspp);
-                if (code == 0) {
-                    serversp = *serverspp;
-                    free_svr_list = 1;
-                }
-            }
-            cm_ResetServerBusyStatus(serversp);
-            if (free_svr_list) {
-                cm_FreeServerList(serverspp, 0);
-                free_svr_list = 0;
-                serversp = NULL;
-            }
-
             code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp,
                                       CM_GETVOL_FLAG_NO_LRU_UPDATE,
                                       &volp);
@@ -456,30 +403,47 @@ cm_Analyze(cm_conn_t *connp,
             format = "All servers are busy when accessing cell %s volume %d.";
 	    LogEvent(EVENTLOG_WARNING_TYPE, msgID, cellp->name, fidp->volume);
 
-            if (!serversp) {
-                code = cm_GetServerList(fidp, userp, reqp, &replicated, &serverspp);
-                if (code == 0) {
-                    serversp = *serverspp;
-                    free_svr_list = 1;
-                }
-            }
-            cm_ResetServerBusyStatus(serversp);
-            if (free_svr_list) {
-                cm_FreeServerList(serverspp, 0);
-                free_svr_list = 0;
-                serversp = NULL;
-            }
-
             code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp,
                                      CM_GETVOL_FLAG_NO_LRU_UPDATE,
                                      &volp);
             if (code == 0) {
                 if (timeLeft > 7) {
                     thrd_Sleep(5000);
+
                     statep = cm_VolumeStateByID(volp, fidp->volume);
-                    retry = 1;
+                    if (statep->state != vl_offline &&
+                         statep->state != vl_busy &&
+                         statep->state != vl_unknown) {
+                        retry = 1;
+                    } else {
+                        if (!serversp) {
+                            code = cm_GetServerList(fidp, userp, reqp, &serverspp);
+                            if (code == 0) {
+                                serversp = *serverspp;
+                                free_svr_list = 1;
+                            }
+                        }
+                        lock_ObtainWrite(&cm_serverLock);
+                        for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
+                            if (tsrp->status == srv_deleted)
+                                continue;
+                            if (tsrp->status == srv_busy) {
+                                tsrp->status = srv_not_busy;
+                            }
+                        }
+                        lock_ReleaseWrite(&cm_serverLock);
+                        if (free_svr_list) {
+                            cm_FreeServerList(serverspp, 0);
+                            serversp = NULL;
+                            free_svr_list = 0;
+                        }
+
+                        cm_UpdateVolumeStatus(volp, fidp->volume);
+                        retry = 1;
+                    }
+                } else {
+                    cm_UpdateVolumeStatus(volp, fidp->volume);
                 }
-                cm_UpdateVolumeStatus(volp, fidp->volume);
 
                 lock_ObtainRead(&cm_volumeLock);
                 cm_PutVolume(volp);
@@ -493,7 +457,15 @@ cm_Analyze(cm_conn_t *connp,
                 thrd_Sleep(5000);
 
                 if (serversp) {
-                    cm_ResetServerBusyStatus(serversp);
+                    lock_ObtainWrite(&cm_serverLock);
+                    for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
+                        if (tsrp->status == srv_deleted)
+                            continue;
+                        if (tsrp->status == srv_busy) {
+                            tsrp->status = srv_not_busy;
+                        }
+                    }
+                    lock_ReleaseWrite(&cm_serverLock);
                     retry = 1;
                 }
             }
@@ -503,7 +475,7 @@ cm_Analyze(cm_conn_t *connp,
     /* special codes:  VBUSY and VRESTARTING */
     else if (errorCode == VBUSY || errorCode == VRESTARTING) {
         if (!serversp && fidp) {
-            code = cm_GetServerList(fidp, userp, reqp, &replicated, &serverspp);
+            code = cm_GetServerList(fidp, userp, reqp, &serverspp);
             if (code == 0) {
                 serversp = *serverspp;
                 free_svr_list = 1;
@@ -533,24 +505,33 @@ cm_Analyze(cm_conn_t *connp,
 	    LogEvent(EVENTLOG_WARNING_TYPE, msgID, addr, fidp->volume, cellp->name);
         }
 
-        cm_SetServerBusyStatus(serversp, serverp);
-
-        if (fidp) { /* File Server query */
-            code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp,
-                                      CM_GETVOL_FLAG_NO_LRU_UPDATE,
-                                      &volp);
-            if (code == 0) {
-                statep = cm_VolumeStateByID(volp, fidp->volume);
-
-                if (statep)
-                    cm_UpdateVolumeStatus(volp, statep->ID);
-
-                lock_ObtainRead(&cm_volumeLock);
-                cm_PutVolume(volp);
-                lock_ReleaseRead(&cm_volumeLock);
-                volp = NULL;
+        lock_ObtainWrite(&cm_serverLock);
+        for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
+            if (tsrp->status == srv_deleted)
+                continue;
+            if (tsrp->server == serverp && tsrp->status == srv_not_busy) {
+                tsrp->status = srv_busy;
+                if (fidp) { /* File Server query */
+                    lock_ReleaseWrite(&cm_serverLock);
+                    code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp,
+                                             CM_GETVOL_FLAG_NO_LRU_UPDATE,
+                                             &volp);
+                    if (code == 0)
+                        statep = cm_VolumeStateByID(volp, fidp->volume);
+                    lock_ObtainWrite(&cm_serverLock);
+                }
+                break;
             }
         }
+        lock_ReleaseWrite(&cm_serverLock);
+
+        if (statep) {
+            cm_UpdateVolumeStatus(volp, statep->ID);
+            lock_ObtainRead(&cm_volumeLock);
+            cm_PutVolume(volp);
+            lock_ReleaseRead(&cm_volumeLock);
+            volp = NULL;
+        }
 
         if (free_svr_list) {
             cm_FreeServerList(serverspp, 0);
@@ -562,7 +543,7 @@ cm_Analyze(cm_conn_t *connp,
 
     /* special codes:  missing volumes */
     else if (errorCode == VNOVOL || errorCode == VMOVED || errorCode == VOFFLINE ||
-             errorCode == VSALVAGE || errorCode == VIO)
+             errorCode == VSALVAGE || errorCode == VNOSERVICE || errorCode == VIO)
     {
         /* In case of timeout */
         reqp->volumeError = errorCode;
@@ -584,6 +565,10 @@ cm_Analyze(cm_conn_t *connp,
 	    msgID = MSG_SERVER_REPORTS_VSALVAGE;
             format = "Server %s reported volume %d in cell %s as needs salvage.";
             break;
+        case VNOSERVICE:
+	    msgID = MSG_SERVER_REPORTS_VNOSERVICE;
+            format = "Server %s reported volume %d in cell %s as not in service.";
+            break;
         case VIO:
 	    msgID = MSG_SERVER_REPORTS_VIO;
             format = "Server %s reported volume %d in cell %s as temporarily unaccessible.";
@@ -641,7 +626,7 @@ cm_Analyze(cm_conn_t *connp,
              * from the server list if it was moved or is not present.
              */
             if (!serversp || location_updated) {
-                code = cm_GetServerList(fidp, userp, reqp, &replicated, &serverspp);
+                code = cm_GetServerList(fidp, userp, reqp, &serverspp);
                 if (code == 0) {
                     serversp = *serverspp;
                     free_svr_list = 1;
@@ -662,58 +647,25 @@ cm_Analyze(cm_conn_t *connp,
 
             if (cm_ServerEqual(tsrp->server, serverp)) {
                 /* REDIRECT */
-                switch (errorCode) {
-                case VMOVED:
-                    osi_Log2(afsd_logp, "volume %u moved from server %s",
+                if (errorCode == VMOVED || errorCode == VNOVOL) {
+                    osi_Log2(afsd_logp, "volume %d not present on server %s",
                              fidp->volume, osi_LogSaveString(afsd_logp,addr));
                     tsrp->status = srv_deleted;
                     if (fidp)
                         cm_RemoveVolumeFromServer(serverp, fidp->volume);
-                    break;
-                case VNOVOL:
-                    /*
-                     * The 1.6.0 and 1.6.1 file servers send transient VNOVOL errors which
-                     * are no indicative of the volume not being present.  For example,
-                     * VNOVOL can be sent during a transition to a VBUSY state prior to
-                     * salvaging or when cloning a .backup volume instance.  As a result
-                     * the cache manager must attempt at least one retry when a VNOVOL is
-                     * receive but there are no changes to the volume location information.
-                     */
-                    if (reqp->vnovolError > 0 && cm_ServerEqual(reqp->errorServp, serverp)) {
-                        osi_Log2(afsd_logp, "volume %u not present on server %s",
-                                  fidp->volume, osi_LogSaveString(afsd_logp,addr));
-                        tsrp->status = srv_deleted;
-                        if (fidp)
-                            cm_RemoveVolumeFromServer(serverp, fidp->volume);
-                    } else {
-                        osi_Log2(afsd_logp, "VNOVOL received for volume %u from server %s",
-                                 fidp->volume, osi_LogSaveString(afsd_logp,addr));
-                        if (replicated) {
-                            cm_SetServerBusyStatus(serversp, serverp);
-                        } else {
-                            Sleep(2000);
-                        }
-                    }
-                    break;
-                case VOFFLINE:
-                    osi_Log2(afsd_logp, "VOFFLINE received for volume %u from server %s",
-                              fidp->volume, osi_LogSaveString(afsd_logp,addr));
+                } else {
+                    osi_Log2(afsd_logp, "volume %d instance on server %s marked offline",
+                             fidp->volume, osi_LogSaveString(afsd_logp,addr));
                     tsrp->status = srv_offline;
-                    break;
-                default:
-                    osi_Log3(afsd_logp, "volume %u exists on server %s with status %u",
-                             fidp->volume, osi_LogSaveString(afsd_logp,addr), tsrp->status);
                 }
+                /* break; */
+            } else {
+                osi_Log3(afsd_logp, "volume %d exists on server %s with status %u",
+                         fidp->volume, osi_LogSaveString(afsd_logp,addr), tsrp->status);
             }
         }
         lock_ReleaseWrite(&cm_serverLock);
 
-        /* Remember that the VNOVOL error occurred */
-        if (errorCode == VNOVOL) {
-            reqp->errorServp = serverp;
-            reqp->vnovolError++;
-        }
-
         /* Free the server list before cm_ForceUpdateVolume is called */
         if (free_svr_list) {
             cm_FreeServerList(serverspp, 0);
@@ -725,6 +677,7 @@ cm_Analyze(cm_conn_t *connp,
             retry = 1;
     } else if ( errorCode == VNOVNODE ) {
 	if ( fidp ) {
+	    cm_scache_t * scp;
 	    osi_Log4(afsd_logp, "cm_Analyze passed VNOVNODE cell %u vol %u vn %u uniq %u.",
 		      fidp->cell, fidp->volume, fidp->vnode, fidp->unique);
 
@@ -743,15 +696,23 @@ cm_Analyze(cm_conn_t *connp,
 		lock_ReleaseWrite(&cm_scacheLock);
                 cm_LockMarkSCacheLost(scp);
 		lock_ReleaseWrite(&scp->rw);
+                if (RDR_Initialized)
+                    RDR_InvalidateObject(scp->fid.cell, scp->fid.volume, scp->fid.vnode, scp->fid.unique,
+                                          scp->fid.hash, scp->fileType, AFS_INVALIDATE_DELETED);
 		cm_ReleaseSCache(scp);
 
  		if (pscp) {
 		    if (cm_HaveCallback(pscp)) {
  			lock_ObtainWrite(&pscp->rw);
  			cm_DiscardSCache(pscp);
- 			lock_ReleaseWrite(&pscp->rw);
- 		    }
- 		    cm_ReleaseSCache(pscp);
+			lock_ReleaseWrite(&pscp->rw);
+
+                        if (RDR_Initialized)
+                            RDR_InvalidateObject(pscp->fid.cell, pscp->fid.volume, pscp->fid.vnode, pscp->fid.unique,
+                                                 pscp->fid.hash, pscp->fileType, AFS_INVALIDATE_EXPIRED);
+
+		    }
+		    cm_ReleaseSCache(pscp);
  		}
 	    }
 	} else {
@@ -778,28 +739,10 @@ cm_Analyze(cm_conn_t *connp,
             LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_HARD_DEAD_TIME_EXCEEDED, addr);
             osi_Log1(afsd_logp, "cm_Analyze: hardDeadTime or idleDeadtime exceeded addr[%s]",
                      osi_LogSaveString(afsd_logp,addr));
-            reqp->errorServp = serverp;
+            reqp->tokenIdleErrorServp = serverp;
             reqp->idleError++;
         }
 
-        if (fidp && storeOp)
-            scp = cm_FindSCache(fidp);
-        if (scp) {
-            if (cm_HaveCallback(scp)) {
-                lock_ObtainWrite(&scp->rw);
-                cm_DiscardSCache(scp);
-                lock_ReleaseWrite(&scp->rw);
-
-                /*
-                * We really should notify the redirector that we discarded
-                * the status information but doing so in this case is not
-                * safe as it can result in a deadlock with extent release
-                * processing.
-                */
-            }
-            cm_ReleaseSCache(scp);
-        }
-
         if (timeLeft > 2) {
             if (!fidp) { /* vldb */
                 retry = 1;
@@ -834,7 +777,7 @@ cm_Analyze(cm_conn_t *connp,
 
         retry = 1;
     }
-    else if (errorCode == RX_CALL_BUSY) {
+    else if (errorCode == CM_RX_RETRY_BUSY_CALL) {
         /*
          * RPC failed because the selected call channel
          * is currently busy on the server.  Unconditionally
@@ -852,160 +795,6 @@ cm_Analyze(cm_conn_t *connp,
                  osi_LogSaveString(afsd_logp,addr));
         retry = 1;
     }
-    else if (errorCode == VNOSERVICE) {
-        /*
-         * The server did not service the RPC.
-         * If this was a file server RPC it means that for at
-         * least the file server's idle dead timeout period the
-         * file server did not receive any new data packets from
-         * client.
-         *
-         * The RPC was not serviced so it can be retried and any
-         * existing status information is still valid.
-         */
-        if (fidp) {
-            if (serverp)
-                sprintf(addr, "%d.%d.%d.%d",
-                        ((serverp->addr.sin_addr.s_addr & 0xff)),
-                        ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                        ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                        ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24));
-
-            LogEvent(EVENTLOG_WARNING_TYPE, MSG_SERVER_REPORTS_VNOSERVICE,
-                     addr, fidp->volume, cellp->name);
-            osi_Log3(afsd_logp, "Server %s reported rpc to volume %d in cell %s as not serviced.",
-                     osi_LogSaveString(afsd_logp,addr), fidp->volume, cellp->name);
-        }
-
-        if (timeLeft > 2)
-            retry = 1;
-    }
-    else if (errorCode == RX_CALL_IDLE) {
-        /*
-         * RPC failed because the server failed to respond with data
-         * within the idle dead timeout period.  This could be for a variety
-         * of reasons:
-         *  1. The server could have a bad partition such as a failed
-         *     disk or iSCSI target and all I/O to that partition is
-         *     blocking on the server and will never complete.
-         *
-         *  2. The server vnode may be locked by another client request
-         *     that is taking a very long time.
-         *
-         *  3. The server may have a very long queue of requests
-         *     pending and is unable to process this request.
-         *
-         *  4. The server could be malicious and is performing a denial
-         *     of service attack against the client.
-         *
-         * If this is a request against a .readonly with alternate sites
-         * the server should be marked down for this request and the
-         * client should fail over to another server.  If this is a
-         * request against a single source, the client may retry once.
-         */
-        if (serverp)
-            sprintf(addr, "%d.%d.%d.%d",
-                    ((serverp->addr.sin_addr.s_addr & 0xff)),
-                    ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                    ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                    ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24));
-
-        if (fidp) {
-            code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp,
-                                     CM_GETVOL_FLAG_NO_LRU_UPDATE,
-                                     &volp);
-            if (code == 0) {
-                statep = cm_VolumeStateByID(volp, fidp->volume);
-
-                if (statep)
-                    replicated = (statep->flags & CM_VOL_STATE_FLAG_REPLICATED);
-
-                lock_ObtainRead(&cm_volumeLock);
-                cm_PutVolume(volp);
-                lock_ReleaseRead(&cm_volumeLock);
-                volp = NULL;
-            }
-
-            if (storeOp)
-                scp = cm_FindSCache(fidp);
-	    if (scp) {
-                if (cm_HaveCallback(scp)) {
-                    lock_ObtainWrite(&scp->rw);
-                    cm_DiscardSCache(scp);
-                    lock_ReleaseWrite(&scp->rw);
-
-                    /*
-                     * We really should notify the redirector that we discarded
-                     * the status information but doing so in this case is not
-                     * safe as it can result in a deadlock with extent release
-                     * processing.
-                     */
-                }
-                cm_ReleaseSCache(scp);
-            }
-        }
-
-        if (replicated && serverp) {
-            reqp->errorServp = serverp;
-            reqp->tokenError = errorCode;
-
-            if (timeLeft > 2)
-                retry = 1;
-        }
-
-        LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_IDLE_DEAD_TIMEOUT, addr, retry);
-        osi_Log2(afsd_logp, "cm_Analyze: RPC failed due to idle dead timeout addr[%s] retry=%u",
-                 osi_LogSaveString(afsd_logp,addr), retry);
-    }
-    else if (errorCode == RX_CALL_DEAD) {
-        /* mark server as down */
-        if (serverp)
-            sprintf(addr, "%d.%d.%d.%d",
-                    ((serverp->addr.sin_addr.s_addr & 0xff)),
-                    ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                    ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                    ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24));
-
-        osi_Log2(afsd_logp, "cm_Analyze: Rx Call Dead addr[%s] forcedNew[%s]",
-                 osi_LogSaveString(afsd_logp,addr),
-                 (reqp->flags & CM_REQ_NEW_CONN_FORCED ? "yes" : "no"));
-
-        if (serverp) {
-            if ((reqp->flags & CM_REQ_NEW_CONN_FORCED)) {
-                lock_ObtainMutex(&serverp->mx);
-                if (!(serverp->flags & CM_SERVERFLAG_DOWN)) {
-                    _InterlockedOr(&serverp->flags, CM_SERVERFLAG_DOWN);
-                    serverp->downTime = time(NULL);
-                }
-                lock_ReleaseMutex(&serverp->mx);
-            } else {
-                reqp->flags |= CM_REQ_NEW_CONN_FORCED;
-                forcing_new = 1;
-                cm_ForceNewConnections(serverp);
-            }
-        }
-
-        if (fidp && storeOp)
-            scp = cm_FindSCache(fidp);
-        if (scp) {
-            if (cm_HaveCallback(scp)) {
-                lock_ObtainWrite(&scp->rw);
-                cm_DiscardSCache(scp);
-                lock_ReleaseWrite(&scp->rw);
-
-                /*
-                * We really should notify the redirector that we discarded
-                * the status information but doing so in this case is not
-                * safe as it can result in a deadlock with extent release
-                * processing.
-                */
-            }
-            cm_ReleaseSCache(scp);
-        }
-
-        if ( timeLeft > 2 )
-            retry = 1;
-    }
     else if (errorCode >= -64 && errorCode < 0) {
         /* mark server as down */
         if (serverp)
@@ -1015,20 +804,35 @@ cm_Analyze(cm_conn_t *connp,
                     ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
                     ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24));
 
-        osi_Log3(afsd_logp, "cm_Analyze: Rx Misc Error[%d] addr[%s] forcedNew[%s]",
-                 errorCode,
-                 osi_LogSaveString(afsd_logp,addr),
-                 (reqp->flags & CM_REQ_NEW_CONN_FORCED ? "yes" : "no"));
+        if (errorCode == RX_CALL_DEAD)
+            osi_Log2(afsd_logp, "cm_Analyze: Rx Call Dead addr[%s] forcedNew[%s]",
+                     osi_LogSaveString(afsd_logp,addr),
+                     (reqp->flags & CM_REQ_NEW_CONN_FORCED ? "yes" : "no"));
+        else
+            osi_Log3(afsd_logp, "cm_Analyze: Rx Misc Error[%d] addr[%s] forcedNew[%s]",
+                     errorCode,
+                     osi_LogSaveString(afsd_logp,addr),
+                     (reqp->flags & CM_REQ_NEW_CONN_FORCED ? "yes" : "no"));
 
         if (serverp) {
-            if (reqp->flags & CM_REQ_NEW_CONN_FORCED) {
-                reqp->errorServp = serverp;
-                reqp->tokenError = errorCode;
+            lock_ObtainMutex(&serverp->mx);
+            if (errorCode == RX_CALL_DEAD &&
+                (reqp->flags & CM_REQ_NEW_CONN_FORCED)) {
+                if (!(serverp->flags & CM_SERVERFLAG_DOWN)) {
+                    _InterlockedOr(&serverp->flags, CM_SERVERFLAG_DOWN);
+                    serverp->downTime = time(NULL);
+                }
             } else {
-                reqp->flags |= CM_REQ_NEW_CONN_FORCED;
-                forcing_new = 1;
-                cm_ForceNewConnections(serverp);
+                if (reqp->flags & CM_REQ_NEW_CONN_FORCED) {
+                    reqp->tokenIdleErrorServp = serverp;
+                    reqp->tokenError = errorCode;
+                } else {
+                    reqp->flags |= CM_REQ_NEW_CONN_FORCED;
+                    forcing_new = 1;
+                }
             }
+            lock_ReleaseMutex(&serverp->mx);
+            cm_ForceNewConnections(serverp);
         }
         if ( timeLeft > 2 )
             retry = 1;
@@ -1070,7 +874,7 @@ cm_Analyze(cm_conn_t *connp,
                   errorCode, s);
 
         if (serverp) {
-            reqp->errorServp = serverp;
+            reqp->tokenIdleErrorServp = serverp;
             reqp->tokenError = errorCode;
             retry = 1;
         }
@@ -1083,7 +887,7 @@ cm_Analyze(cm_conn_t *connp,
          * and force the use of another server.
          */
         if (serverp) {
-            reqp->errorServp = serverp;
+            reqp->tokenIdleErrorServp = serverp;
             reqp->tokenError = errorCode;
             retry = 1;
         }
@@ -1216,7 +1020,7 @@ cm_Analyze(cm_conn_t *connp,
 
     /* If not allowed to retry, don't */
     if (!forcing_new && (reqp->flags & CM_REQ_NORETRY) &&
-        (errorCode != RX_MSGSIZE && errorCode != RX_CALL_BUSY))
+        (errorCode != RX_MSGSIZE && errorCode != CM_RX_RETRY_BUSY_CALL))
 	retry = 0;
     else if (retry && dead_session)
         retry = 0;
@@ -1235,20 +1039,12 @@ cm_Analyze(cm_conn_t *connp,
         reqp->flags &= ~CM_REQ_VOLUME_UPDATED;
     }
 
-    if ( serversp &&
-         errorCode != VBUSY &&
-         errorCode != VRESTARTING &&
-         errorCode != CM_ERROR_ALLBUSY)
-    {
-        cm_ResetServerBusyStatus(serversp);
-    }
-
     /* retry until we fail to find a connection */
     return retry;
 }
 
-long cm_ConnByMServers(cm_serverRef_t *serversp, afs_uint32 replicated, cm_user_t *usersp,
-                       cm_req_t *reqp, cm_conn_t **connpp)
+long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
+	cm_req_t *reqp, cm_conn_t **connpp)
 {
     long code;
     cm_serverRef_t *tsrp;
@@ -1279,15 +1075,15 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, afs_uint32 replicated, cm_user_
             continue;
 
         tsp = tsrp->server;
-        if (reqp->errorServp) {
+        if (reqp->tokenIdleErrorServp) {
             /*
              * search the list until we find the server
              * that failed last time.  When we find it
-             * clear the error, skip it and try the one
+             * clear the error, skip it and try the next one
              * in the list.
              */
-            if (tsp == reqp->errorServp)
-                reqp->errorServp = NULL;
+            if (tsp == reqp->tokenIdleErrorServp)
+                reqp->tokenIdleErrorServp = NULL;
             continue;
         }
         if (tsp) {
@@ -1304,7 +1100,7 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, afs_uint32 replicated, cm_user_
                 } else {
                     allOffline = 0;
                     allBusy = 0;
-                    code = cm_ConnByServer(tsp, usersp, replicated, connpp);
+                    code = cm_ConnByServer(tsp, usersp, connpp);
                     if (code == 0) {        /* cm_CBS only returns 0 */
                         cm_PutServer(tsp);
 #ifdef SET_RX_TIMEOUTS_TO_TIMELEFT
@@ -1389,7 +1185,7 @@ void cm_GCConnections(cm_server_t *serverp)
 }
 
 static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
-                               cm_server_t *serverp, afs_uint32 replicated)
+                               cm_server_t *serverp)
 {
     unsigned short port;
     int serviceID;
@@ -1441,16 +1237,12 @@ static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
                                     secIndex);
     rx_SetConnDeadTime(tcp->rxconnp, ConnDeadtimeout);
     rx_SetConnHardDeadTime(tcp->rxconnp, HardDeadtimeout);
+    rx_SetConnIdleDeadTime(tcp->rxconnp, IdleDeadtimeout);
 
     /*
-     * Setting idle dead timeout to a non-zero value activates RX_CALL_IDLE errors
+     * Register the error to be returned on an idle dead timeout
      */
-    if (replicated) {
-        tcp->flags &= CM_CONN_FLAG_REPLICATION;
-        rx_SetConnIdleDeadTime(tcp->rxconnp, ReplicaIdleDeadtimeout);
-    } else {
-        rx_SetConnIdleDeadTime(tcp->rxconnp, IdleDeadtimeout);
-    }
+    rx_SetServerConnIdleDeadErr(tcp->rxconnp, RX_CALL_DEAD);
 
     /*
      * Let the Rx library know that we can auto-retry if an
@@ -1464,17 +1256,15 @@ static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
      * Only file servers implement client callbacks and we only need one ping
      * to be sent to each server.
      */
-    if (NatPingInterval && serverp->type == CM_SERVER_FILE &&
-         (ucellp->flags & CM_UCELLFLAG_ROOTUSER)) {
+    if (NatPingInterval && serverp->type == CM_SERVER_FILE && secIndex == 0)
         rx_SetConnSecondsUntilNatPing(tcp->rxconnp, NatPingInterval);
-    }
 
     tcp->ucgen = ucellp->gen;
     if (secObjp)
         rxs_Release(secObjp);   /* Decrement the initial refCount */
 }
 
-long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, afs_uint32 replicated, cm_conn_t **connpp)
+long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, cm_conn_t **connpp)
 {
     cm_conn_t *tcp;
     cm_ucell_t *ucellp;
@@ -1487,9 +1277,7 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, afs_uint32 replicat
     lock_ObtainMutex(&userp->mx);
     lock_ObtainRead(&cm_connLock);
     for (tcp = serverp->connsp; tcp; tcp=tcp->nextp) {
-        if (tcp->userp == userp &&
-            (replicated && (tcp->flags & CM_CONN_FLAG_REPLICATION) ||
-             !replicated && !(tcp->flags & CM_CONN_FLAG_REPLICATION)))
+        if (tcp->userp == userp)
             break;
     }
 
@@ -1508,21 +1296,21 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, afs_uint32 replicat
         cm_GetServer(serverp);
         tcp = malloc(sizeof(*tcp));
         memset(tcp, 0, sizeof(*tcp));
+        tcp->nextp = serverp->connsp;
+        serverp->connsp = tcp;
         cm_HoldUser(userp);
         tcp->userp = userp;
         lock_InitializeMutex(&tcp->mx, "cm_conn_t mutex", LOCK_HIERARCHY_CONN);
+        lock_ObtainMutex(&tcp->mx);
         tcp->serverp = serverp;
         tcp->cryptlevel = rxkad_clear;
-        cm_NewRXConnection(tcp, ucellp, serverp, replicated);
+        cm_NewRXConnection(tcp, ucellp, serverp);
         tcp->refCount = 1;
-        tcp->nextp = serverp->connsp;
-        serverp->connsp = tcp;
+        lock_ReleaseMutex(&tcp->mx);
         lock_ReleaseWrite(&cm_connLock);
-        lock_ReleaseMutex(&userp->mx);
     } else {
         lock_ReleaseRead(&cm_connLock);
       haveconn:
-        lock_ReleaseMutex(&userp->mx);
         InterlockedIncrement(&tcp->refCount);
 
         lock_ObtainMutex(&tcp->mx);
@@ -1534,13 +1322,14 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, afs_uint32 replicat
                 osi_Log0(afsd_logp, "cm_ConnByServer replace connection due to token update");
             else
                 osi_Log0(afsd_logp, "cm_ConnByServer replace connection due to crypt change");
-            tcp->flags &= ~CM_CONN_FLAG_FORCE_NEW;
+	    tcp->flags &= ~CM_CONN_FLAG_FORCE_NEW;
             rx_SetConnSecondsUntilNatPing(tcp->rxconnp, 0);
             rx_DestroyConnection(tcp->rxconnp);
-            cm_NewRXConnection(tcp, ucellp, serverp, replicated);
+            cm_NewRXConnection(tcp, ucellp, serverp);
         }
         lock_ReleaseMutex(&tcp->mx);
     }
+    lock_ReleaseMutex(&userp->mx);
 
     /* return this pointer to our caller */
     osi_Log1(afsd_logp, "cm_ConnByServer returning conn 0x%p", tcp);
@@ -1557,11 +1346,10 @@ long cm_ServerAvailable(struct cm_fid *fidp, struct cm_user *userp)
     cm_serverRef_t *tsrp;
     cm_server_t *tsp;
     int someBusy = 0, someOffline = 0, allOffline = 1, allBusy = 1, allDown = 1;
-    afs_uint32 replicated;
 
     cm_InitReq(&req);
 
-    code = cm_GetServerList(fidp, userp, &req, &replicated, &serverspp);
+    code = cm_GetServerList(fidp, userp, &req, &serverspp);
     if (code)
         return 0;
 
@@ -1606,15 +1394,15 @@ long cm_ConnFromFID(struct cm_fid *fidp, struct cm_user *userp, cm_req_t *reqp,
 {
     long code;
     cm_serverRef_t **serverspp;
-    afs_uint32 replicated;
 
     *connpp = NULL;
 
-    code = cm_GetServerList(fidp, userp, reqp, &replicated, &serverspp);
-    if (code)
+    code = cm_GetServerList(fidp, userp, reqp, &serverspp);
+    if (code) {
         return code;
+    }
 
-    code = cm_ConnByMServers(*serverspp, replicated, userp, reqp, connpp);
+    code = cm_ConnByMServers(*serverspp, userp, reqp, connpp);
     cm_FreeServerList(serverspp, 0);
     return code;
 }
@@ -1625,16 +1413,12 @@ long cm_ConnFromVolume(struct cm_volume *volp, unsigned long volid, struct cm_us
 {
     long code;
     cm_serverRef_t **serverspp;
-    afs_uint32 replicated;
-    cm_vol_state_t * volstatep;
 
     *connpp = NULL;
 
-    volstatep = cm_VolumeStateByID(volp, volid);
-    replicated = (volstatep->flags & CM_VOL_STATE_FLAG_REPLICATED);
     serverspp = cm_GetVolServers(volp, volid, userp, reqp);
 
-    code = cm_ConnByMServers(*serverspp, replicated, userp, reqp, connpp);
+    code = cm_ConnByMServers(*serverspp, userp, reqp, connpp);
     cm_FreeServerList(serverspp, 0);
     return code;
 }
