@@ -36,16 +36,13 @@
 #include "afs/afs_util.h"
 #include "afs/unified_afs.h"
 
-#if	defined(AFS_SUN56_ENV)
+#if	defined(AFS_SUN5_ENV)
 #include <inet/led.h>
 #include <inet/common.h>
-#if     defined(AFS_SUN58_ENV)
 #include <netinet/ip6.h>
-#endif
 #include <inet/ip.h>
 #endif
 
-
 /* shouldn't do it this way, but for now will do */
 #ifndef ERROR_TABLE_BASE_U
 #define ERROR_TABLE_BASE_U	(5376L)
@@ -118,14 +115,14 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
 	VSleep(2);		/* Better safe than sorry. */
 	tconn =
 	    afs_ConnByMHosts(tcell->cellHosts, tcell->vlport, tcell->cellNum,
-			     &treq, SHARED_LOCK, 0, &rxconn);
+			     &treq, SHARED_LOCK, &rxconn);
 	if (tconn) {
-	    if (tconn->srvr->server->flags & SNO_LHOSTS) {
+	    if ( tconn->parent->srvr->server->flags & SNO_LHOSTS) {
 		type = 0;
 		RX_AFS_GUNLOCK();
 		i = VL_GetEntryByNameO(rxconn, bp, &v->tve);
 		RX_AFS_GLOCK();
-	    } else if (tconn->srvr->server->flags & SYES_LHOSTS) {
+	    } else if (tconn->parent->srvr->server->flags & SYES_LHOSTS) {
 		type = 1;
 		RX_AFS_GUNLOCK();
 		i = VL_GetEntryByNameN(rxconn, bp, &v->ntve);
@@ -135,7 +132,7 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
 		RX_AFS_GUNLOCK();
 		i = VL_GetEntryByNameU(rxconn, bp, &v->utve);
 		RX_AFS_GLOCK();
-		if (!(tconn->srvr->server->flags & SVLSRV_UUID)) {
+		if (!(tconn->parent->srvr->server->flags & SVLSRV_UUID)) {
 		    if (i == RXGEN_OPCODE) {
 			type = 1;
 			RX_AFS_GUNLOCK();
@@ -143,14 +140,14 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
 			RX_AFS_GLOCK();
 			if (i == RXGEN_OPCODE) {
 			    type = 0;
-			    tconn->srvr->server->flags |= SNO_LHOSTS;
+			    tconn->parent->srvr->server->flags |= SNO_LHOSTS;
 			    RX_AFS_GUNLOCK();
 			    i = VL_GetEntryByNameO(rxconn, bp, &v->tve);
 			    RX_AFS_GLOCK();
 			} else if (!i)
-			    tconn->srvr->server->flags |= SYES_LHOSTS;
+			    tconn->parent->srvr->server->flags |= SYES_LHOSTS;
 		    } else if (!i)
-			tconn->srvr->server->flags |= SVLSRV_UUID;
+			tconn->parent->srvr->server->flags |= SVLSRV_UUID;
 		}
 		lastcode = i;
 	    }
@@ -261,126 +258,19 @@ afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
 		    areq->skipserver[i] = 1;
 		}
 	    }
+	    afs_PutVolume(tvp, READ_LOCK);
 	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 	        if (tvp->serverHost[i] && areq->skipserver[i] == 0) {
 		    serversleft = 1;
 		    break;
 		}
 	    }
-	    afs_PutVolume(tvp, READ_LOCK);
 	    return serversleft;
 	}
     }
     return serversleft;
 }
 
-/*------------------------------------------------------------------------
- * afs_ClearStatus
- *
- * Description:
- *	Analyze the outcome of an RPC operation, taking whatever support
- *	actions are necessary.
- *
- * Arguments:
- *	afid  : The FID of the file involved in the action.  This argument
- *		may be null if none was involved.
- *      op    : which RPC we are analyzing.
- *      avp   : A pointer to the struct volume, if we already have one.
- *
- * Returns:
- *	Non-zero value if the related RPC operation can be retried,
- *	zero otherwise.
- *
- * Environment:
- *	This routine is called when we got a network error,
- *      and discards state if the operation was a data-mutating
- *      operation.
- *------------------------------------------------------------------------*/
-static int
-afs_ClearStatus(struct VenusFid *afid, int op, struct volume *avp)
-{
-    struct volume *tvp = NULL;
-
-    /* if it's not a write op, we have nothing to veto and shouldn't clear. */
-    if (!AFS_STATS_FS_RPCIDXES_ISWRITE(op)) {
-	return 1;
-    }
-
-    if (avp)
-	tvp = avp;
-    else if (afid)
-	tvp = afs_FindVolume(afid, READ_LOCK);
-
-    /* don't assume just discarding will fix if no cached volume */
-    if (tvp) {
-	struct vcache *tvc;
-	ObtainReadLock(&afs_xvcache);
-	if ((tvc = afs_FindVCache(afid, 0, 0))) {
-	    ReleaseReadLock(&afs_xvcache);
-	    tvc->f.states &= ~(CStatd | CUnique);
-	    afs_PutVCache(tvc);
-	} else {
-	    ReleaseReadLock(&afs_xvcache);
-	}
-	if (!avp)
-	    afs_PutVolume(tvp, READ_LOCK);
-    }
-
-    if (AFS_STATS_FS_RPCIDXES_WRITE_RETRIABLE(op))
-	return 1;
-
-    /* not retriable: we may have raced ourselves */
-    return 0;
-}
-
-/*!
- * \brief
- *      Print the last errors from the servers for the volume on
- *      this request.
- *
- * \param[in] areq   The request record associated with this operation.
- * \param[in] afid   The FID of the file involved in the action.  This argument
- *		     may be null if none was involved.
- *
- * \return
- *      None
- *
- * \note
- *      This routine is called before a hard-mount retry, to display
- *      the servers by primary address and the errors encountered.
- */
-static void
-afs_PrintServerErrors(struct vrequest *areq, struct VenusFid *afid)
-{
-    int i;
-    struct volume *tvp;
-    struct srvAddr *sa;
-    afs_uint32 address;
-    char *sep = " (";
-    char *term = "";
-
-    if (afid) {
-	tvp = afs_FindVolume(afid, READ_LOCK);
-	if (tvp) {
-	    for (i = 0; i < AFS_MAXHOSTS; i++) {
-		if (tvp->serverHost[i]) {
-		    sa = tvp->serverHost[i]->addr;
-		    if (sa) {
-			address = ntohl(sa->sa_ip);
-			afs_warnuser("%s%d.%d.%d.%d code=%d", sep,
-				     (address >> 24), (address >> 16) & 0xff,
-				     (address >> 8) & 0xff, (address) & 0xff,
-				     areq->lasterror[i]);
-			sep = ", ";
-			term = ")";
-		    }
-		}
-	    }
-	    afs_PutVolume(tvp, READ_LOCK);
-	}
-    }
-    afs_warnuser("%s\n", term);
-}
 
 /*------------------------------------------------------------------------
  * EXPORTED afs_Analyze
@@ -416,9 +306,8 @@ afs_PrintServerErrors(struct vrequest *areq, struct VenusFid *afid)
  *------------------------------------------------------------------------*/
 int
 afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
-	    afs_int32 acode,
-	    struct VenusFid *afid, struct vrequest *areq, int op,
-	    afs_int32 locktype, struct cell *cellp)
+            afs_int32 acode, struct VenusFid *afid, struct vrequest *areq,
+            int op, afs_int32 locktype, struct cell *cellp)
 {
     afs_int32 i;
     struct srvAddr *sa;
@@ -492,7 +381,7 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 	return shouldRetry;	/* should retry */
     }
 
-    if (!aconn || !aconn->srvr) {
+    if (!aconn || !aconn->parent->srvr) {
 	if (!areq->volumeError) {
 	    if (aerrP)
 		(aerrP->err_Network)++;
@@ -550,15 +439,12 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 		    if (shouldRetry) {
 			if (warn) {
 			    afs_warnuser
-			        ("afs: hard-mount waiting for volume %u",
+			        ("afs: hard-mount waiting for volume %u\n",
 			         afid->Fid.Volume);
-			    afs_PrintServerErrors(areq, afid);
 			}
 
 			VSleep(hm_retry_int);
 			afs_CheckServers(1, cellp);
-			/* clear the black listed servers on this request. */
-			memset(areq->skipserver, 0, sizeof(areq->skipserver));
 
 			if (vp_vhm) {
 			    tvp = afs_FindVolume(afid, READ_LOCK);
@@ -575,21 +461,15 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 	    else {
 		if (acode == RX_MSGSIZE)
 		    shouldRetry = 1;
-		else {
+		else
 		    areq->networkError = 1;
-		    /* do not promote to shouldRetry if not already */
-		    if (afs_ClearStatus(afid, op, NULL) == 0)
-			shouldRetry = 0;
-		}
 	    }
 	}
-	if (aconn) /* simply lacking aconn->server doesn't absolve this */
-	    afs_PutConn(aconn, rxconn, locktype);
 	return shouldRetry;
     }
 
     /* Find server associated with this connection. */
-    sa = aconn->srvr;
+    sa = aconn->parent->srvr;
     tsp = sa->server;
     address = ntohl(sa->sa_ip);
 
@@ -616,65 +496,44 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 	return 0;
     }
 
-    /* Save the last code of this server on this request. */
-    tvp = afs_FindVolume(afid, READ_LOCK);
-    if (tvp) {
-	for (i = 0; i < AFS_MAXHOSTS; i++) {
-	    if (tvp->serverHost[i] == tsp) {
-		areq->lasterror[i] = acode;
-	    }
-	}
-	afs_PutVolume(tvp, READ_LOCK);
-    }
-
+    /* If network troubles, mark server as having bogued out again. */
+    /* VRESTARTING is < 0 because of backward compatibility issues
+     * with 3.4 file servers and older cache managers */
 #ifdef AFS_64BIT_CLIENT
     if (acode == -455)
 	acode = 455;
 #endif /* AFS_64BIT_CLIENT */
-    if (acode == RX_MSGSIZE || acode == RX_CALL_BUSY) {
-	shouldRetry = 1;
-	goto out;
-    }
-    if (acode == RX_CALL_TIMEOUT || acode == RX_CALL_IDLE || acode == VNOSERVICE) {
-	serversleft = afs_BlackListOnce(areq, afid, tsp);
-	if (afid)
-	    tvp = afs_FindVolume(afid, READ_LOCK);
-	if ((serversleft == 0) && tvp &&
-	    ((tvp->states & VRO) || (tvp->states & VBackup))) {
-	    shouldRetry = 0;
-	} else {
-	    shouldRetry = 1;
-	}
-	if (!afid || !tvp || (tvp->states & VRO))
-	    areq->idleError++;
-	else if (afs_ClearStatus(afid, op, tvp) == 0)
-	    shouldRetry = 0;
-
-	if (tvp)
-	    afs_PutVolume(tvp, READ_LOCK);
-	/* By doing this, we avoid ever marking a server down
-	 * in an idle timeout case. That's because the server is
-	 * still responding and may only be letting a single vnode
-	 * time out. We otherwise risk having the server continually
-	 * be marked down, then up, then down again...
-	 */
-	goto out;
-    }
-    /* If network troubles, mark server as having bogued out again. */
-    /* VRESTARTING is < 0 because of backward compatibility issues
-     * with 3.4 file servers and older cache managers */
     if ((acode < 0) && (acode != VRESTARTING)) {
-	afs_ServerDown(sa, acode);
+	if (acode == RX_MSGSIZE) {
+	    shouldRetry = 1;
+	    goto out;
+	}
+	if (acode == RX_CALL_TIMEOUT) {
+	    serversleft = afs_BlackListOnce(areq, afid, tsp);
+	    if (afid)
+		tvp = afs_FindVolume(afid, READ_LOCK);
+	    if (!afid || !tvp || (tvp->states & VRO))
+		areq->idleError++;
+	    if ((serversleft == 0) && tvp &&
+		((tvp->states & VRO) || (tvp->states & VBackup))) {
+		shouldRetry = 0;
+	    } else {
+		shouldRetry = 1;
+	    }
+	    if (tvp)
+		afs_PutVolume(tvp, READ_LOCK);
+	    /* By doing this, we avoid ever marking a server down
+	     * in an idle timeout case. That's because the server is
+	     * still responding and may only be letting a single vnode
+	     * time out. We otherwise risk having the server continually
+	     * be marked down, then up, then down again...
+	     */
+	    goto out;
+	}
+	afs_ServerDown(sa);
 	ForceNewConnections(sa); /**multi homed clients lock:afs_xsrvAddr? */
 	if (aerrP)
 	    (aerrP->err_Server)++;
-#if 0
-	/* retry *once* when the server is timed out in case of NAT */
-	if (markeddown && acode == RX_CALL_DEAD) {
-	    aconn->forceConnectFS = 1;
-	    shouldRetry = 1;
-	}
-#endif
     }
 
     if (acode == VBUSY || acode == VRESTARTING) {
@@ -720,11 +579,11 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 		aconn->forceConnectFS = 1;
 	    } else if (acode == RXKADEXPIRED) {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
-		aconn->user->states |= UTokensBad;
+		aconn->parent->user->states |= UTokensBad;
 		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
 		    ("afs: Tokens for user of AFS id %d for cell %s have expired (server %d.%d.%d.%d)\n",
-		     tu->vid, aconn->srvr->server->cell->cellName,
+		     tu->viceId, aconn->parent->srvr->server->cell->cellName,
 		     (address >> 24), (address >> 16) & 0xff,
 		     (address >> 8) & 0xff, (address) & 0xff);
 	    } else {
@@ -734,18 +593,18 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 		if (serversleft) {
 		    afs_warnuser
 			("afs: Tokens for user of AFS id %d for cell %s: rxkad error=%d (server %d.%d.%d.%d)\n",
-			 tu->vid, aconn->srvr->server->cell->cellName, acode,
+			 tu->viceId, aconn->parent->srvr->server->cell->cellName, acode,
 			 (address >> 24), (address >> 16) & 0xff,
 			 (address >> 8) & 0xff, (address) & 0xff);
 		    shouldRetry = 1;
 		} else {
 		    areq->tokenError = 0;
 		    aconn->forceConnectFS = 0;	/* don't check until new tokens set */
-		    aconn->user->states |= UTokensBad;
+		    aconn->parent->user->states |= UTokensBad;
 		    afs_NotifyUser(tu, UTokensDropped);
 		    afs_warnuser
 			("afs: Tokens for user of AFS id %d for cell %s are discarded (rxkad error=%d, server %d.%d.%d.%d)\n",
-			 tu->vid, aconn->srvr->server->cell->cellName, acode,
+			 tu->viceId, aconn->parent->srvr->server->cell->cellName, acode,
 			 (address >> 24), (address >> 16) & 0xff,
 			 (address >> 8) & 0xff, (address) & 0xff);
 		}
@@ -757,20 +616,21 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 		aconn->forceConnectFS = 1;
 	    } else if (acode == RXKADEXPIRED) {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
-		aconn->user->states |= UTokensBad;
+		aconn->parent->user->states |= UTokensBad;
 		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
 		    ("afs: Tokens for user %d for cell %s have expired (server %d.%d.%d.%d)\n",
-		     areq->uid, aconn->srvr->server->cell->cellName,
+		     areq->uid, aconn->parent->srvr->server->cell->cellName,
 		     (address >> 24), (address >> 16) & 0xff,
 		     (address >> 8) & 0xff, (address) & 0xff);
 	    } else {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
-		aconn->user->states |= UTokensBad;
+		aconn->parent->user->states |= UTokensBad;
 		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
 		    ("afs: Tokens for user %d for cell %s are discarded (rxkad error = %d, server %d.%d.%d.%d)\n",
-		     areq->uid, aconn->srvr->server->cell->cellName, acode,
+		     areq->uid, aconn->parent->srvr->server->cell->cellName,
+                     acode,
 		     (address >> 24), (address >> 16) & 0xff,
 		     (address >> 8) & 0xff, (address) & 0xff);
 
@@ -790,7 +650,7 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
     }
     /* check for ubik errors; treat them like crashed servers */
     else if (acode >= ERROR_TABLE_BASE_U && acode < ERROR_TABLE_BASE_U + 255) {
-	afs_ServerDown(sa, acode);
+	afs_ServerDown(sa);
 	if (aerrP)
 	    (aerrP->err_Server)++;
 	shouldRetry = 1;	/* retryable (maybe one is working) */
@@ -798,7 +658,7 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
     }
     /* Check for bad volume data base / missing volume. */
     else if (acode == VSALVAGE || acode == VOFFLINE || acode == VNOVOL
-	     || acode == VMOVED) {
+	     || acode == VNOSERVICE || acode == VMOVED) {
 	struct cell *tcell;
 	int same;
 
@@ -850,7 +710,7 @@ afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
 	 * retry in case there is another server.  However, if we find
 	 * no connection (aconn == 0) we set the networkError flag.
 	 */
-	afs_ServerDown(sa, acode);
+	afs_MarkServerUpOrDown(sa, SRVR_ISDOWN);
 	if (aerrP)
 	    (aerrP->err_Server)++;
 	VSleep(1);		/* Just a hack for desperate times. */
