@@ -9,7 +9,7 @@
 
 /*
  * Implements:
- * genpagval
+ * genpag
  * getpag
  * afs_setpag
  * AddPag
@@ -32,15 +32,22 @@
 
 
 /* Imported variables */
-extern enum afs_shutdown_state afs_shuttingdown;
+extern int afs_shuttingdown;
 
 /* Exported variables */
 afs_uint32 pag_epoch;
-#if defined(UKERNEL)
+#if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
 afs_uint32 pagCounter = 1;
 #else
 afs_uint32 pagCounter = 0;
-#endif /* UKERNEL */
+#endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
+
+#ifdef AFS_LINUX26_ONEGROUP_ENV
+#define NUMPAGGROUPS 1
+#else
+#define NUMPAGGROUPS 2
+#endif
+/* Local variables */
 
 /*
  * Pags are implemented as follows: the set of groups whose long
@@ -66,9 +73,9 @@ afs_uint32 pagCounter = 0;
  * anyway, so the pag is an alternative handle which is somewhat more
  * secure (although of course not absolutely secure).
 */
-#if !defined(UKERNEL)
-static afs_uint32
-genpagval(void)
+#if !defined(UKERNEL) || !defined(AFS_WEB_ENHANCEMENTS)
+afs_uint32
+genpag(void)
 {
     AFS_STATCNT(genpag);
 #ifdef AFS_LINUX20_ENV
@@ -96,8 +103,8 @@ getpag(void)
 /* Web enhancement: we don't need to restrict pags to 41XXXXXX since
  * we are not sharing the space with anyone.  So we use the full 32 bits. */
 
-static afs_uint32
-genpagval(void)
+afs_uint32
+genpag(void)
 {
     AFS_STATCNT(genpag);
 #ifdef AFS_LINUX20_ENV
@@ -118,7 +125,7 @@ getpag(void)
     return (pagCounter);
 #endif
 }
-#endif /* UKERNEL */
+#endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
 
 /* used to require 10 seconds between each setpag to guarantee that
  * PAGs never wrap - which would be a security hole.  If we presume
@@ -181,24 +188,13 @@ afs_pag_wait(afs_ucred_t **acred)
     return 0;
 }
 
-/*
- * Note that 'acred' is actually an 'afs_ucred_t*', not an 'afs_ucred_t**'. But
- * various callers and callees expect an 'afs_ucred_t*' due to historical
- * reasons.
- */
-afs_int32
-afs_genpag(afs_ucred_t **acred, afs_uint32 *apag)
-{
-    afs_pag_wait(acred);
-    *apag = genpagval();
-    return 0;
-}
-
 int
 #if	defined(AFS_SUN5_ENV)
 afs_setpag(afs_ucred_t **credpp)
 #elif	defined(AFS_FBSD_ENV)
 afs_setpag(struct thread *td, void *args)
+#elif	defined(AFS_NBSD_ENV)
+afs_setpag(afs_proc_t *p, const void *args, register_t *retval)
 #elif  defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 afs_setpag(afs_proc_t *p, void *args, int *retval)
 #else
@@ -215,7 +211,6 @@ afs_setpag(void)
 #endif
 
     int code = 0;
-    afs_uint32 pag;
 
 #if defined(AFS_SGI53_ENV) && defined(MP)
     /* This is our first chance to get the global lock. */
@@ -224,17 +219,17 @@ afs_setpag(void)
 
     AFS_STATCNT(afs_setpag);
 
-    afs_genpag(acred, &pag);
+    afs_pag_wait(acred);
+
 
 #if	defined(AFS_SUN5_ENV)
-    code = AddPag(pag, credpp);
+    code = AddPag(genpag(), credpp);
 #elif	defined(AFS_FBSD_ENV)
-    code = AddPag(td, pag, &td->td_ucred);
+    code = AddPag(td, genpag(), &td->td_ucred);
 #elif   defined(AFS_NBSD40_ENV)
-    /* XXXX won't work */
-    code = AddPag(p, pag, (afs_ucred_t **) osi_curcred());
+    code = AddPag(p, genpag(), &p->l_proc->p_cred);
 #elif	defined(AFS_XBSD_ENV)
-    code = AddPag(p, pag, &p->p_rcred);
+    code = AddPag(p, genpag(), &p->p_rcred);
 #elif	defined(AFS_AIX41_ENV)
     {
 	struct ucred *credp;
@@ -242,7 +237,7 @@ afs_setpag(void)
 
 	credp = crref();
 	credp0 = credp;
-	code = AddPag(pag, &credp);
+	code = AddPag(genpag(), &credp);
 	/* If AddPag() didn't make a new cred, then free our cred ref */
 	if (credp == credp0) {
 	    crfree(credp);
@@ -251,36 +246,36 @@ afs_setpag(void)
 #elif	defined(AFS_HPUX110_ENV)
     {
 	struct ucred *credp = p_cred(u.u_procp);
-	code = AddPag(pag, &credp);
+	code = AddPag(genpag(), &credp);
     }
 #elif	defined(AFS_SGI_ENV)
     {
 	cred_t *credp;
 	credp = OSI_GET_CURRENT_CRED();
-	code = AddPag(pag, &credp);
+	code = AddPag(genpag(), &credp);
     }
 #elif	defined(AFS_LINUX20_ENV)
     {
 	afs_ucred_t *credp = crref();
-	code = AddPag(pag, &credp);
+	code = AddPag(genpag(), &credp);
 	crfree(credp);
     }
 #elif defined(AFS_DARWIN80_ENV)
     {
 	afs_ucred_t *credp = kauth_cred_proc_ref(p);
-	code = AddPag(p, pag, &credp);
+	code = AddPag(p, genpag(), &credp);
 	crfree(credp);
     }
 #elif defined(AFS_DARWIN_ENV)
     {
 	afs_ucred_t *credp = crdup(p->p_cred->pc_ucred);
-	code = AddPag(p, pag, &credp);
+	code = AddPag(p, genpag(), &credp);
 	crfree(credp);
     }
 #elif defined(UKERNEL)
-    code = AddPag(pag, &(get_user_struct()->u_cred));
+    code = AddPag(genpag(), &(get_user_struct()->u_cred));
 #else
-    code = AddPag(pag, &u.u_cred);
+    code = AddPag(genpag(), &u.u_cred);
 #endif
 
     afs_Trace1(afs_iclSetp, CM_TRACE_SETPAG, ICL_TYPE_INT32, code);
@@ -297,11 +292,11 @@ afs_setpag(void)
     return (code);
 }
 
-#if defined(UKERNEL)
+#if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
 /*
  * afs_setpag_val
  * This function is like setpag but sets the current thread's pag id to a
- * caller-provided value instead of calling afs_genpag().  This implements a
+ * caller-provided value instead of calling genpag().  This implements a
  * form of token caching since the caller can recall a particular pag value
  * for the thread to restore tokens, rather than reauthenticating.
  */
@@ -395,7 +390,7 @@ afs_setpag_val(int pagval)
     return (code);
 }
 
-#ifndef AFS_PAG_ONEGROUP_ENV
+#ifndef AFS_LINUX26_ONEGROUP_ENV
 int
 afs_getpag_val(void)
 {
@@ -420,7 +415,7 @@ afs_getpag_val(void)
     return pagvalue;
 }
 #endif
-#endif /* UKERNEL */
+#endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
 
 
 /* Note - needs to be available on AIX, others can be static - rework this */
@@ -439,8 +434,6 @@ AddPag(afs_int32 aval, afs_ucred_t **credpp)
     AFS_STATCNT(AddPag);
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     if ((code = setpag(p, credpp, aval, &newpag, 0)))
-#elif defined(AFS_NBSD40_ENV)
-    if ((code = setpag(p, (void *) credpp, aval, &newpag, 0)))
 #else
     if ((code = setpag(credpp, aval, &newpag, 0)))
 #endif
@@ -462,7 +455,7 @@ afs_InitReq(struct vrequest *av, afs_ucred_t *acred)
 
     AFS_STATCNT(afs_InitReq);
     memset(av, 0, sizeof(*av));
-    if (afs_shuttingdown == AFS_SHUTDOWN)
+    if (afs_shuttingdown)
 	return EIO;
 
 #ifdef AFS_LINUX26_ENV
@@ -487,67 +480,13 @@ afs_InitReq(struct vrequest *av, afs_ucred_t *acred)
 #elif defined(AFS_SUN510_ENV)
         av->uid = crgetruid(acred);
 #else
-	av->uid = afs_cr_ruid(acred);	/* default when no pag is set */
+	av->uid = afs_cr_uid(acred);	/* default when no pag is set */
 #endif
     }
     return 0;
 }
 
-/*!
- * Allocate and setup a vrequest.
- *
- * \note The caller must free the allocated vrequest with
- *       afs_DestroyReq() if this function returns successfully (zero).
- *
- * \note The GLOCK must be held on platforms which require the GLOCK
- *       for osi_AllocSmallSpace() and osi_FreeSmallSpace().
- *
- * \param[out] avpp   address of the vrequest pointer
- * \param[in]  acred  user credentials to setup the vrequest
- *                    afs_osi_credp should be used for anonymous connections
- * \return     0 on success
- */
-int
-afs_CreateReq(struct vrequest **avpp, afs_ucred_t *acred)
-{
-    int code;
-    struct vrequest *treq = NULL;
-
-    if (afs_shuttingdown == AFS_SHUTDOWN) {
-	return EIO;
-    }
-    if (!avpp || !acred) {
-	return EINVAL;
-    }
-    treq = osi_AllocSmallSpace(sizeof(struct vrequest));
-    if (!treq) {
-	return ENOMEM;
-    }
-    code = afs_InitReq(treq, acred);
-    if (code != 0) {
-	osi_FreeSmallSpace(treq);
-	return code;
-    }
-    *avpp = treq;
-    return 0;
-}
-
-/*!
- * Deallocate a vrequest.
- *
- * \note The GLOCK must be held on platforms which require the GLOCK
- *       for osi_FreeSmallSpace().
- *
- * \param[in]  av  pointer to the vrequest to free; may be NULL
- */
-void
-afs_DestroyReq(struct vrequest *av)
-{
-    if (av) {
-	osi_FreeSmallSpace(av);
-    }
-}
-
+#ifndef AFS_LINUX26_ONEGROUP_ENV
 afs_uint32
 afs_get_pag_from_groups(gid_t g0a, gid_t g1a)
 {
@@ -564,18 +503,17 @@ afs_get_pag_from_groups(gid_t g0a, gid_t g1a)
 	h = (g0 >> 14);
 	h = (g1 >> 14) + h + h + h;
 	ret = ((h << 28) | l);
-# if defined(UKERNEL)
+# if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
 	return ret;
 # else
 	/* Additional testing */
 	if (((ret >> 24) & 0xff) == 'A')
 	    return ret;
-# endif /* UKERNEL */
+# endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
     }
     return NOPAG;
 }
 
-#ifndef AFS_PAG_ONEGROUP_ENV
 void
 afs_get_groups_from_pag(afs_uint32 pag, gid_t * g0p, gid_t * g1p)
 {
@@ -584,9 +522,9 @@ afs_get_groups_from_pag(afs_uint32 pag, gid_t * g0p, gid_t * g1p)
     AFS_STATCNT(afs_get_groups_from_pag);
     *g0p = pag;
     *g1p = 0;
-# if !defined(UKERNEL)
+# if !defined(UKERNEL) || !defined(AFS_WEB_ENHANCEMENTS)
     pag &= 0x7fffffff;
-# endif /* UKERNEL */
+# endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
     g0 = 0x3fff & (pag >> 14);
     g1 = 0x3fff & pag;
     g0 |= ((pag >> 28) / 3) << 14;
@@ -595,8 +533,7 @@ afs_get_groups_from_pag(afs_uint32 pag, gid_t * g0p, gid_t * g1p)
     *g1p = g1 + 0x3f00;
 }
 #else
-void
-afs_get_groups_from_pag(afs_uint32 pag, gid_t *g0p, gid_t *g1p)
+void afs_get_groups_from_pag(afs_uint32 pag, gid_t *g0p, gid_t *g1p)
 {
     AFS_STATCNT(afs_get_groups_from_pag);
     *g0p = pag;
@@ -604,13 +541,7 @@ afs_get_groups_from_pag(afs_uint32 pag, gid_t *g0p, gid_t *g1p)
 }
 #endif
 
-#ifdef AFS_LINUX26_ENV
-/* osi_get_group_pag is defined in <ARCH>/osi_groups.c */
-#elif defined(AFS_PAG_ONEGROUP_ENV)
-/* osi_get_group_pag is defined in <ARCH>/osi_groups.c */
-#elif defined(AFS_DARWIN110_ENV)
-/* We don't have pags, so we do not define an osi_get_group_pag */
-#else
+#if !defined(AFS_LINUX26_ENV) && !defined(AFS_DARWIN110_ENV)
 static afs_int32
 osi_get_group_pag(afs_ucred_t *cred)
 {
@@ -626,13 +557,12 @@ osi_get_group_pag(afs_ucred_t *cred)
     ngroups = crgetngroups(cred);
 #endif
 #if defined(AFS_NBSD40_ENV)
-#warning com afs_ucred_t w/magic will not work
     if (cred == NOCRED || cred == FSCRED)
       return NOPAG;
     if (osi_crngroups(cred) < 3)
       return NOPAG;
-    g0 = osi_crgroupbyid(cred, 0);
-    g1 = osi_crgroupbyid(cred, 1);
+    g0 = osi_crgroupbyid(cred, 1);
+    g1 = osi_crgroupbyid(cred, 2);
 #elif defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     if (cred == NOCRED || cred == FSCRED)
 	return NOPAG;
@@ -646,7 +576,7 @@ osi_get_group_pag(afs_ucred_t *cred)
     if (cred->cr_ngrps < 2)
 	return NOPAG;
 # elif defined(AFS_LINUX26_ENV)
-    if (afs_cr_group_info(cred)->ngroups < AFS_NUMPAGGROUPS)
+    if (afs_cr_group_info(cred)->ngroups < NUMPAGGROUPS)
 	return NOPAG;
 # elif defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_LINUX20_ENV) || defined(AFS_XBSD_ENV)
 #  if defined(AFS_SUN510_ENV)
