@@ -31,7 +31,7 @@
 
 afs_int32 afs_termState = 0;
 afs_int32 afs_gcpags = AFS_GCPAGS;
-enum afs_shutdown_state afs_shuttingdown = AFS_RUNNING;
+int afs_shuttingdown = 0;
 int afs_cold_shutdown = 0;
 int afs_resourceinit_flag = 0;
 afs_int32 afs_nfs_server_addr;
@@ -98,7 +98,7 @@ afspag_Init(afs_int32 nfs_server_addr)
     AFS_GLOCK();
 
     afs_InitStats();
-    rx_SetBusyChannelError(1);  /* turn on busy call error reporting */
+    rx_SetBusyChannelError(RX_CALL_TIMEOUT);
     rx_Init(htons(7001));
 
     AFS_STATCNT(afs_ResourceInit);
@@ -106,10 +106,6 @@ afspag_Init(afs_int32 nfs_server_addr)
     AFS_RWLOCK_INIT(&afs_xpagcell, "afs_xpagcell");
     AFS_RWLOCK_INIT(&afs_xpagsys, "afs_xpagsys");
     AFS_RWLOCK_INIT(&afs_icl_lock, "afs_icl_lock");
-#ifndef AFS_FBSD_ENV
-    LOCK_INIT(&osi_fsplock, "osi_fsplock");
-    LOCK_INIT(&osi_flplock, "osi_flplock");
-#endif
 
     afs_resourceinit_flag = 1;
     afs_nfs_server_addr = nfs_server_addr;
@@ -163,9 +159,9 @@ afspag_Init(afs_int32 nfs_server_addr)
 void
 afspag_Shutdown(void)
 {
-    if (afs_shuttingdown != AFS_RUNNING)
+    if (afs_shuttingdown)
 	return;
-    afs_shuttingdown = AFS_SHUTDOWN;
+    afs_shuttingdown = 1;
     afs_termState = AFSOP_STOP_RXCALLBACK;
     rx_WakeupServerProcs();
     while (afs_termState == AFSOP_STOP_RXCALLBACK)
@@ -493,8 +489,6 @@ afs_syscall_pioctl(path, com, cmarg, follow)
 	goto out_idata;
     }
 
-    memset(odata.rmtbulk_val, 0, out_size);
-
     AFS_GUNLOCK();
     code = RMTSYS_Pioctl(rmtsys_conn, &ccred, abspath, com, follow,
 			 &idata, &odata, &err);
@@ -548,7 +542,7 @@ int
 afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
      long parm, parm2, parm3, parm4, parm5, parm6;
 {
-    /* Superusers may shut us down, such as with afsd -shutdown. */
+    /* superusers may shut us down, as with afsd --shutdown */
 #ifdef AFS_SUN5_ENV
     if (parm == AFSOP_SHUTDOWN && afs_suser(CRED()))
 #else
