@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,35 +14,30 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <afs/debug.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
 #include <ctype.h>
-#include <string.h>
 
-    /* These two needed for rxgen output to work */
-#include <sys/types.h>
+#include <hcrypto/des.h>
+#include <hcrypto/ui.h>
+
 #include <rx/xdr.h>
-
-#include <stdio.h>
 #include <rx/rx.h>
 #include <rx/rxkad.h>
 #include <lock.h>
 #define UBIK_LEGACY_CALLITER 1
 #include <ubik.h>
-#ifndef AFS_NT40_ENV
-#include <pwd.h>
-#endif
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <afs/cmd.h>
 #include <afs/com_err.h>
 #include <afs/afsutil.h>
-#include <des.h>
-#include <des_prototypes.h>
 
 #include "kauth.h"
+#include "kauth_internal.h"
 #include "kautils.h"
 #include "kaport.h"
 #include "kkids.h"
@@ -98,7 +93,7 @@ DumpUser(char *user, char *arock, int showadmin, int showkey, char *inst)
     if (!inst)
 	inst = instance;
     code =
-	ubik_Call(KAM_GetEntry, conn, 0, name, inst, KAMAJORVERSION, &tentry);
+	ubik_KAM_GetEntry(conn, 0, name, inst, KAMAJORVERSION, &tentry);
     if (code) {
 	afs_com_err(whoami, code, "getting information for %s.%s", name, inst);
 	return code;
@@ -254,7 +249,7 @@ ListUsers(struct cmd_syndesc *as, void *arock)
     }
     for (index = 0; 1; index = next_index) {
 	code =
-	    ubik_Call(KAM_ListEntry, conn, 0, index, &next_index, &count,
+	    ubik_KAM_ListEntry(conn, 0, index, &next_index, &count,
 		      &name);
 	if (code) {
 	    afs_com_err(whoami, code, "calling KAM_ListEntry");
@@ -341,7 +336,8 @@ CreateUser(struct cmd_syndesc *as, void *arock)
     ka_StringToKey(as->parms[1].items->data, cell, &key);
 
     do {
-	code = ubik_Call(KAM_CreateUser, conn, 0, name, instance, key);
+	code = ubik_KAM_CreateUser(conn, 0, name, instance,
+				   *ktc_to_EncryptionKey(&key));
 	if (!code)
 	    return 0;
 	ka_PrintUserID("Creating user ", name, instance, " ");
@@ -368,7 +364,7 @@ DeleteUser(struct cmd_syndesc *as, void *arock)
     }
 
     do {
-	code = ubik_Call(KAM_DeleteUser, conn, 0, name, instance);
+	code = ubik_KAM_DeleteUser(conn, 0, name, instance);
 	if (!code)
 	    return 0;
 	ka_PrintUserID("Deleting user ", name, instance, " ");
@@ -431,7 +427,7 @@ parse_flags(char *name, char *inst, char *str, afs_int32 * flags)
 	    } else
 		addop = 1;
 	    code =
-		ubik_Call(KAM_GetEntry, conn, 0, name, inst, KAMAJORVERSION,
+		ubik_KAM_GetEntry(conn, 0, name, inst, KAMAJORVERSION,
 			  &tentry);
 	    if (code) {
 		afs_com_err(whoami, code,
@@ -520,7 +516,7 @@ ka_islocked(char *name, char *instance, afs_uint32 * when)
 			  0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 	if (code) {
 	    if (seriouserror(code))
-		afs_com_err(whoami, code, "");
+		afs_com_err(whoami, code, NULL);
 	} else if (tempwhen) {	/* user is locked */
 	    if (!*when || tempwhen < *when) {
 		*when = tempwhen;
@@ -581,6 +577,7 @@ SetFields(struct cmd_syndesc *as, void *arock)
     int code;
     char name[MAXKTCNAMELEN];
     char instance[MAXKTCNAMELEN];
+    char *end;
     afs_int32 flags = 0;
     Date expiration = 0;
     afs_int32 lifetime = 0;
@@ -637,9 +634,8 @@ SetFields(struct cmd_syndesc *as, void *arock)
     for (i = 0; i < 4; misc_auth_bytes[i++] = 0);
 
     if (as->parms[4].items) {
-	if (util_isint(as->parms[4].items->data))
-	    pwexpiry = atoi(as->parms[4].items->data);
-	else {
+	pwexpiry = strtol(as->parms[4].items->data, &end, 10);
+	if (*end != '\0') {
 	    fprintf(stderr,
 		    "Password lifetime specified must be a non-negative decimal integer.\n");
 	    pwexpiry = -1;
@@ -649,9 +645,9 @@ SetFields(struct cmd_syndesc *as, void *arock)
 		    "Password lifetime range must be [0..254] days.\n");
 	    fprintf(stderr, "Zero represents an unlimited lifetime.\n");
 	    return KABADCMD;
-	} else {
-	    misc_auth_bytes[0] = pwexpiry + 1;
 	}
+
+	misc_auth_bytes[0] = pwexpiry + 1;
     }
 
     if (as->parms[5].items) {
@@ -672,15 +668,14 @@ SetFields(struct cmd_syndesc *as, void *arock)
     if (as->parms[6].items) {
 	int nfailures;
 
+        nfailures = strtol(as->parms[6].items->data, &end, 10);
 
-	if (util_isint(as->parms[6].items->data)
-	    && ((nfailures = atoi(as->parms[6].items->data)) < 255)) {
-	    misc_auth_bytes[2] = nfailures + 1;
-	} else {
+	if (*end != '\0' || nfailures < 0 || nfailures > 254) {
 	    fprintf(stderr, "Failure limit must be in [0..254].\n");
 	    fprintf(stderr, "Zero represents unlimited login attempts.\n");
 	    return KABADCMD;
 	}
+	misc_auth_bytes[2] = nfailures + 1;
     }
 
     if (as->parms[7].items) {
@@ -724,7 +719,7 @@ SetFields(struct cmd_syndesc *as, void *arock)
 
     if (was_spare || flags || expiration || lifetime || (maxAssociates >= 0))
 	code =
-	    ubik_Call(KAM_SetFields, conn, 0, name, instance, flags,
+	    ubik_KAM_SetFields(conn, 0, name, instance, flags,
 		      expiration, lifetime, maxAssociates, was_spare,
 		      /* spare */ 0);
     else {
@@ -764,7 +759,7 @@ StringToKey(struct cmd_syndesc *as, void *arock)
     ka_PrintBytes((char *)&key, sizeof(key));
     printf("'.\n");
 
-    des_string_to_key(as->parms[0].items->data, ktc_to_cblockptr(&key));
+    DES_string_to_key(as->parms[0].items->data, ktc_to_cblockptr(&key));
 
     printf("Converting %s with the DES string to key yields key='",
 	   as->parms[0].items->data);
@@ -821,11 +816,8 @@ SetPassword(struct cmd_syndesc *as, void *arock)
     if (as->parms[3].items)
 	sscanf(as->parms[3].items->data, "%d", &kvno);
 
-#if defined(AFS_S390_LINUX20_ENV) && !defined(AFS_S390X_LINUX20_ENV)
-    code = ubik_Call(KAM_SetPassword, conn, 0, name, instance, kvno, 0, key);
-#else
-    code = ubik_Call(KAM_SetPassword, conn, 0, name, instance, kvno, key);
-#endif
+    code = ubik_KAM_SetPassword(conn, 0, name, instance, kvno,
+				*ktc_to_EncryptionKey(&key));
     if (code)
 	afs_com_err(whoami, code, "so can't set password for %s.%s", name,
 		instance);
@@ -1053,7 +1045,8 @@ GetPassword(struct cmd_syndesc *as, void *arock)
 	if (code)
 	    goto abort;
     }
-    code = ubik_Call(KAM_GetPassword, lpbkConn, 0, name, &key);
+    code = ubik_KAM_GetPassword(lpbkConn, 0, name,
+				ktc_to_EncryptionKey(&key));
     /* Lets close down the ubik_Client connection now */
     ubik_ClientDestroy(lpbkConn);
     if (code)
@@ -1070,7 +1063,7 @@ GetRandomKey(struct cmd_syndesc *as, void *arock)
     int code;
     struct ktc_encryptionKey key;
 
-    code = ubik_Call(KAM_GetRandomKey, conn, 0, &key);
+    code = ubik_KAM_GetRandomKey(conn, 0, ktc_to_EncryptionKey(&key));
     if (code)
 	afs_com_err(whoami, code, "so can't get random key");
     else {
@@ -1100,7 +1093,7 @@ Statistics(struct cmd_syndesc *as, void *arock)
     char bob[KA_TIMESTR_LEN];
 
     code =
-	ubik_Call(KAM_GetStats, conn, 0, KAMAJORVERSION, &admins, &statics,
+	ubik_KAM_GetStats(conn, 0, KAMAJORVERSION, &admins, &statics,
 		  &dynamics);
     if (code) {
 	printf("call to GetStats failed: %s\n", ka_ErrorString(code));
@@ -1114,7 +1107,7 @@ Statistics(struct cmd_syndesc *as, void *arock)
     printf("Hash table utilization = %f%%\n",
 	   (double)dynamics.hashTableUtilization / 100.0);
     ka_timestr(dynamics.start_time, bob, KA_TIMESTR_LEN);
-    printf("From host %lx started at %s:\n", 
+    printf("From host %lx started at %s:\n",
 	   afs_printable_uint32_lu(dynamics.host), bob);
 
 #define print_stat(name) if (dynamics.name.requests) printf ("  of %d requests for %s, %d were aborted.\n", dynamics.name.requests, # name, dynamics.name.aborts)
@@ -1175,10 +1168,10 @@ DebugInfo(struct cmd_syndesc *as, void *arock)
 	    }
 	    return code;
 	}
-	code = ubik_Call(KAM_Debug, iConn, 0, KAMAJORVERSION, 0, &info);
+	code = ubik_KAM_Debug(iConn, 0, KAMAJORVERSION, 0, &info);
 	ubik_ClientDestroy(iConn);
     } else
-	code = ubik_Call(KAM_Debug, conn, 0, KAMAJORVERSION, 0, &info);
+	code = ubik_KAM_Debug(conn, 0, KAMAJORVERSION, 0, &info);
 
     if (code) {
 	afs_com_err(whoami, code, "call to Debug failed");
@@ -1205,7 +1198,7 @@ DebugInfo(struct cmd_syndesc *as, void *arock)
 	     timeOffset, now - start);
     }
     ka_timestr(info.startTime, bob, KA_TIMESTR_LEN);
-    printf("From host %lx started %sat %s:\n", 
+    printf("From host %lx started %sat %s:\n",
 	   afs_printable_uint32_lu(info.host),
 	   (info.noAuth ? "w/o authorization " : ""), bob);
     ka_timestr(info.lastTrans, bob, KA_TIMESTR_LEN);
@@ -1220,7 +1213,7 @@ DebugInfo(struct cmd_syndesc *as, void *arock)
     printf("Next autoCPW at %s or in %d updates.\n", bob,
 	   info.updatesRemaining);
     if (info.cheader_lock || info.keycache_lock)
-	printf("locks: cheader %08lx, keycache %08lx\n", 
+	printf("locks: cheader %08lx, keycache %08lx\n",
 		afs_printable_uint32_lu(info.cheader_lock),
 	        afs_printable_uint32_lu(info.keycache_lock));
     printf("Last authentication for %s, last admin user was %s\n",
@@ -1263,7 +1256,7 @@ MyAfterProc(struct cmd_syndesc *as, void *arock)
 	return 0;
 
     /* Determine if we need to destory the ubik connection.
-     * Closing it avoids resends of packets. 
+     * Closing it avoids resends of packets.
      */
     if (conn) {
 	ubik_ClientDestroy(conn);
@@ -1277,7 +1270,7 @@ int init = 0, noauth;
 char name[MAXKTCNAMELEN];
 char instance[MAXKTCNAMELEN];
 char newCell[MAXKTCREALMLEN];
-afs_int32 serverList[MAXSERVERS];
+afs_uint32 serverList[MAXSERVERS];
 
 int
 NoAuth(struct cmd_syndesc *as, void *arock)
@@ -1290,7 +1283,8 @@ static int
 MyBeforeProc(struct cmd_syndesc *as, void *arock)
 {
     struct ktc_encryptionKey key;
-    struct ktc_principal auth_server, auth_token, client;
+    struct ktc_principal auth_server, client;
+    struct ktc_token auth_token;
     char realm[MAXKTCREALMLEN];
 
     struct ktc_token token, *pToken;
@@ -1411,7 +1405,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 		    sprintf(msg, "Administrator's (%s) Password: ", name);
 		else
 		    sprintf(msg, "Password for %s: ", name);
-		code = read_pw_string(passwd, sizeof(passwd), msg, 0);
+		code = UI_UTIL_read_pw_string(passwd, sizeof(passwd), msg, 0);
 		if (code)
 		    code = KAREADPW;
 		else if (strlen(passwd) == 0)
@@ -1426,7 +1420,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 		ka_GetAdminToken(name, instance, cell, &key, KA_SIXHOURS,
 				 &token, 0 /* !new */ );
 	    if (code == KABADREQUEST) {
-		des_string_to_key(passwd, ktc_to_cblockptr(&key));
+		DES_string_to_key(passwd, ktc_to_cblockptr(&key));
 		code =
 		    ka_GetAdminToken(name, instance, cell, &key, KA_SIXHOURS,
 				     &token, 0 /* !new */ );
@@ -1517,7 +1511,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 
 		    strcpy(msg, p + 1);
 		    strcat(msg, ": ");
-		    code = read_pw_string(password, sizeof(password), msg, 1);
+		    code = UI_UTIL_read_pw_string(password, sizeof(password), msg, 1);
 		    if (code)
 			code = KAREADPW;
 		    else if (strlen(password) == 0)
@@ -1642,7 +1636,7 @@ ListTickets(struct cmd_syndesc *as, void *arock)
 }
 
 static void
-add_std_args(register struct cmd_syndesc *ts)
+add_std_args(struct cmd_syndesc *ts)
 {
     cmd_Seek(ts, 12);
     /* 12 */ cmd_AddParm(ts, "-admin_username", CMD_SINGLE, CMD_OPTIONAL,
@@ -1659,8 +1653,8 @@ add_std_args(register struct cmd_syndesc *ts)
 afs_int32
 ka_AdminInteractive(int cmd_argc, char *cmd_argv[])
 {
-    register int code;
-    register struct cmd_syndesc *ts;
+    int code;
+    struct cmd_syndesc *ts;
 
     char line[BUFSIZ];
     afs_int32 argc;
@@ -1678,7 +1672,7 @@ ka_AdminInteractive(int cmd_argc, char *cmd_argv[])
     ts = cmd_CreateSyntax("noauthentication", NoAuth, NULL,
 			  "connect to AuthServer w/o using token");
 
-    ts = cmd_CreateSyntax("list", ListUsers, NULL, 
+    ts = cmd_CreateSyntax("list", ListUsers, NULL,
 			  "list all users in database");
     cmd_AddParm(ts, "-long", CMD_FLAG, CMD_OPTIONAL,
 		"show detailed info about each user");
