@@ -7,10 +7,11 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-
 #include <afsconfig.h>
 #include <afs/param.h>
+#include <afs/stds.h>
 
+#include <roken.h>
 
 /*
  *                      (3) Define a structure, idused, instead of an
@@ -30,21 +31,13 @@
  *                          conditions.
  */
 
-#include <afs/stds.h>
-#include <sys/types.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
-#include <io.h>
 #else
-#include <netdb.h>
-#include <netinet/in.h>
 #include <sys/file.h>
 #endif
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
-#include <fcntl.h>
+
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
 #include <ubik.h>
@@ -217,7 +210,7 @@ readUbikHeader(struct misc_data *misc)
     }
 
     uheader.magic = ntohl(uheader.magic);
-    uheader.size = ntohs(uheader.size);
+    uheader.size = ntohl(uheader.size);
     uheader.version.epoch = ntohl(uheader.version.epoch);
     uheader.version.counter = ntohl(uheader.version.counter);
 
@@ -267,13 +260,6 @@ PrintEntryError(struct misc_data *misc, afs_int32 ea, struct prentry *e, int ind
     return 0;
 }
 
-int
-PrintContError(struct misc_data *misc, afs_int32 ea, struct contentry *c, int indent)
-{
-    pr_PrintContEntry(stderr, /*net order */ 0, ea, c, indent);
-    return 0;
-}
-
 afs_int32
 WalkHashTable(afs_int32 hashtable[],	/* hash table to walk */
 	      int hashType,		/* hash function to use */
@@ -399,7 +385,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
     afs_int32 head;
     int bit;
     afs_int32 code;
-    struct contentry c;		/* continuation entry */
+    struct prentry c;		/* continuation entry */
     afs_int32 na;		/* next thread */
     int ni;
     afs_int32 eid = 0;
@@ -507,7 +493,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		return PRDBBAD;
 	    if (na != sghead) {
 		fprintf(stderr, "last block: \n");
-		if (PrintContError(misc, na, &c, 4))
+		if (PrintEntryError(misc, na, &c, 4))
 		    return PRDBBAD;
 	    }
 	    return 0;
@@ -521,7 +507,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	    fprintf(stderr, "Continuation entry reused\n");
 	    if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintContError(misc, na, &c, 4))
+	    if (PrintEntryError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    break;
@@ -531,7 +517,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	    fprintf(stderr, "Continuation id mismatch\n");
 	    if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintContError(misc, na, &c, 4))
+	    if (PrintEntryError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    continue;
@@ -543,19 +529,23 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	    if (id == PRBADID)
 		continue;
 	    else if (id) {
-		int id_s;
+		int eid_s, id_s;
 		sgcount++;
 		/* in case the ids are large, convert to pure sign. */
 		if (id > 0)
 		    id_s = 1;
 		else
 		    id_s = -1;
+		if (eid > 0)
+		    eid_s = 1;
+		else
+		    eid_s = -1;
 		if (id_s > 0) {
 		    fprintf(stderr,
 			    "User can't be member of supergroup list\n");
 		    if (PrintEntryError(misc, ea, e, 2))
 			return PRDBBAD;
-		    if (PrintContError(misc, na, &c, 4))
+		    if (PrintEntryError(misc, na, &c, 4))
 			return PRDBBAD;
 		    noErrors = 0;
 		}
@@ -582,7 +572,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		return PRDBBAD;
 	    if (na != head) {
 		fprintf(stderr, "last block: \n");
-		if (PrintContError(misc, na, &c, 4))
+		if (PrintEntryError(misc, na, &c, 4))
 		    return PRDBBAD;
 	    }
 	    return 0;
@@ -598,7 +588,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		fprintf(stderr, "walking free list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintContError(misc, na, &c, 4))
+	    if (PrintEntryError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    break;
@@ -610,7 +600,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 		fprintf(stderr, "walking free list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintContError(misc, na, &c, 4))
+	    if (PrintEntryError(misc, na, &c, 4))
 		return PRDBBAD;
 	    noErrors = 0;
 	    continue;
@@ -640,7 +630,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 				"User can't be member of user in membership list\n");
 			if (PrintEntryError(misc, ea, e, 2))
 			    return PRDBBAD;
-			if (PrintContError(misc, na, &c, 4))
+			if (PrintEntryError(misc, na, &c, 4))
 			    return PRDBBAD;
 			noErrors = 0;
 		    }
@@ -650,7 +640,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
 				"Bad user/group dicotomy in membership list\n");
 			if (PrintEntryError(misc, ea, e, 2))
 			    return PRDBBAD;
-			if (PrintContError(misc, na, &c, 4))
+			if (PrintEntryError(misc, na, &c, 4))
 			    return PRDBBAD;
 			noErrors = 0;
 		    }
@@ -712,7 +702,7 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 {
     afs_int32 head;
     afs_int32 code;
-    struct prentry te;		/* next entry in owner chain */
+    struct prentry c;		/* continuation entry */
     afs_int32 na;		/* next thread */
     int ni;
     afs_int32 eid = 0;
@@ -725,7 +715,7 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 	head = ntohl(cheader.orphan);
 
     length = 0;
-    for (na = head; na; na = ntohl(te.nextOwned)) {
+    for (na = head; na; na = ntohl(c.nextOwned)) {
 	code = ConvertDiskAddress(na, &ni);
 	if (code) {
 	    fprintf(stderr, "Bad owned list ptr %d", na);
@@ -735,33 +725,23 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 		return PRDBBAD;
 	    if (na != head) {
 		fprintf(stderr, "last block: \n");
-		if (PrintEntryError(misc, na, &te, 4))
+		if (PrintEntryError(misc, na, &c, 4))
 		    return PRDBBAD;
 	    }
 	    return 0;
 	}
-	code = pr_Read(na, (char *)&te, sizeof(te));
+	code = pr_Read(na, (char *)&c, sizeof(c));
 	if (code)
 	    return code;
 	length++;
 
-	if ((ntohl(te.flags) & PRTYPE) == PRCONT) {
-	    fprintf(stderr, "Continuation entry found on owner chain\n");
-	    if (e == 0)
-		fprintf(stderr, "walking orphan list");
-	    else if (PrintEntryError(misc, ea, e, 2))
-		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &te, 4))
-		return PRDBBAD;
-	    break;
-	}
 	if (map[ni] & MAP_OWNED) {
 	    fprintf(stderr, "Entry on multiple owner chains\n");
 	    if (e == 0)
 		fprintf(stderr, "walking orphan list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &te, 4))
+	    if (PrintEntryError(misc, na, &c, 4))
 		return PRDBBAD;
 	    break;
 	}
@@ -773,16 +753,16 @@ WalkOwnedChain(char map[],		/* one byte per db entry */
 		fprintf(stderr, "walking orphan list");
 	    else if (PrintEntryError(misc, ea, e, 2))
 		return PRDBBAD;
-	    if (PrintEntryError(misc, na, &te, 4))
+	    if (PrintEntryError(misc, na, &c, 4))
 		return PRDBBAD;
 	    continue;
 	}
 	if (e) {
-	    if (ntohl(te.owner) != eid) {
+	    if (ntohl(c.owner) != eid) {
 		fprintf(stderr, "Owner id mismatch\n");
 		goto abort;
 	    }
-	} else /* orphan */ if (te.owner) {
+	} else /* orphan */ if (c.owner) {
 	    fprintf(stderr, "Orphan group owner not zero\n");
 	    goto abort;
 	}
@@ -1213,7 +1193,7 @@ DumpRecreate(char map[], struct misc_data *misc)
 		}
 		na = ntohl(e.next);
 		while (na) {
-		    struct contentry c;
+		    struct prentry c;
 		    code = pr_Read(na, (char *)&c, sizeof(c));
 		    if (code)
 			return code;
@@ -1503,11 +1483,12 @@ inccount(struct idused **idmapp, int id)
 	idmapp = &idmap->idnext;
     }
     if (!idmap) {
-	idmap = calloc(1, sizeof *idmap);
+	idmap = (struct idused *)malloc(sizeof *idmap);
 	if (!idmap) {
 	    perror("idmap");
 	    exit(1);
 	}
+	memset(idmap, 0, sizeof idmap);
 	idmap->idstart = id & ~(IDCOUNT - 1);
 	idmap->idnext = *idmapp;
 	*idmapp = idmap;
