@@ -35,17 +35,16 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <stdlib.h>
-#include <stdio.h>
 #include <ctype.h>
-#include <string.h>
+
 #include "rpc_scan.h"
 #include "rpc_parse.h"
 #include "rpc_util.h"
 
 list *proc_defined[MAX_PACKAGES], *special_defined, *typedef_defined,
-    *uniondef_defined;
+    *uniondef_defined, *complex_defined;
 char *SplitStart = NULL;
 char *SplitEnd = NULL;
 char *MasterPrefix = NULL;
@@ -248,6 +247,7 @@ def_struct(definition * defp)
     declaration dec;
     decl_list *decls;
     decl_list **tailp;
+    int special = 0;
 
     defp->def_kind = DEF_STRUCT;
 
@@ -257,6 +257,11 @@ def_struct(definition * defp)
     tailp = &defp->def.st.decls;
     do {
 	get_declaration(&dec, DEF_STRUCT);
+	/* If a structure contains an array, then we're going
+	 * to need to be clever about freeing it */
+	if (dec.rel == REL_ARRAY) {
+	   special = 1;
+	}
 	decls = ALLOC(decl_list);
 	decls->decl = dec;
 	*tailp = decls;
@@ -266,6 +271,9 @@ def_struct(definition * defp)
     } while (tok.kind != TOK_RBRACE);
     get_token(&tok);
     *tailp = NULL;
+
+    if (special)
+	STOREVAL(&complex_defined, defp);
 }
 
 static void
@@ -458,9 +466,6 @@ get_declaration(declaration * dec, defkind dkind)
 	}
 	dec->rel = REL_ARRAY;
 	if (peekscan(TOK_RANGLE, &tok)) {
-	    if ((dkind == DEF_INPARAM) || (dkind == DEF_INOUTPARAM)) {
-		error("input arrays must specify a max size");
-	    }
 	    dec->array_max = "~0";	/* unspecified size, use max */
 	} else {
 	    scan_num(&tok);
@@ -572,11 +577,8 @@ def_package(definition * defp)
     scan(TOK_IDENT, &tok);
     defp->def_name = tok.str;
     no_of_stat_funcs = 0;
-
-    PackageIndex++;
-    if (PackageIndex >= MAX_PACKAGES)
+    if (PackageIndex++ >= MAX_PACKAGES)
 	error("Exceeded upper limit of package statements\n");
-
     function_list_index = 0;
     PackagePrefix[PackageIndex] = tok.str;
     if (MasterPrefix == NULL)
@@ -906,8 +908,8 @@ analyze_ProcParams(definition * defp, token * tokp)
 	Proc_listp = &Proc_list->next;
 	decls = ALLOC(decl_list);
 	memset(decls, 0, sizeof(decl_list));
-	if (tokp->kind != TOK_RPAREN)
-	    decls->decl = dec;
+    if (tokp->kind != TOK_RPAREN)
+        decls->decl = dec;
 	*tailp = decls;
 	tailp = &decls->next;
     } while (tokp->kind != TOK_RPAREN);
@@ -1001,7 +1003,7 @@ hdle_param_tok(definition * defp, declaration * dec, token * tokp,
     Proc_list->component_kind = DEF_PARAM;
     Proc_list->code = alloc(250);
     Proc_list->scode = alloc(250);
-    get_declaration(dec, par_kind);
+    get_declaration(dec, DEF_PARAM);
     Proc_list->pl.param_name = dec->name;
     get1_param_type(defp, dec, &Proc_list->pl.param_type);
     print_param(dec);
@@ -1369,12 +1371,8 @@ static void
 ss_Proc_CodeGeneration(definition * defp)
 {
     int somefrees = 0;
-    extern char zflag;
 
-    if (zflag)
-	defp->can_fail = 0;
-    else
-	defp->can_fail = 1;
+    defp->can_fail = 0;
     ss_ProcName_setup(defp);
     if (!cflag) {
 	ss_ProcParams_setup(defp, &somefrees);
@@ -1434,8 +1432,7 @@ ss_ProcParams_setup(definition * defp, int *somefrees)
 		}
 		f_print(fout, "\t%s %s", plist->pl.param_type,
 			plist->pl.param_name);
-		if (pntr)
-		    *pntr = pres;
+		*pntr = pres;
 	    } else if (strchr(plist->pl.param_type, '*') == 0) {
 		f_print(fout, "\t%s %s", plist->pl.param_type,
 			plist->pl.param_name);
@@ -1497,6 +1494,7 @@ ss_ProcSpecial_setup(definition * defp, int *somefrees)
 
     for (listp = special_defined; listp != NULL; listp = listp->next) {
 	defp1 = (definition *) listp->val;
+
 	for (plist = defp->pc.plists; plist; plist = plist->next) {
 	    if (plist->component_kind == DEF_PARAM
 		&& (plist->pl.param_kind == DEF_INPARAM
@@ -1508,6 +1506,8 @@ ss_ProcSpecial_setup(definition * defp, int *somefrees)
 		    plist->pl.string_name = spec->sdef.string_name;
 		    plist->pl.param_flag |= FREETHIS_PARAM;
 		    *somefrees = 1;
+		    fprintf(fout, "\n\t%s.%s = 0;", plist->pl.param_name,
+			    spec->sdef.string_name);
 		}
 	    }
 	}
@@ -1525,13 +1525,22 @@ ss_ProcSpecial_setup(definition * defp, int *somefrees)
 		    case REL_ARRAY:
 			plist->pl.string_name = alloc(40);
 			if (brief_flag) {
+			    f_print(fout, "\n\t%s.val = 0;",
+				    plist->pl.param_name);
+			    f_print(fout, "\n\t%s.len = 0;",
+				    plist->pl.param_name);
 			    s_print(plist->pl.string_name, "val");
 			} else {
+			    f_print(fout, "\n\t%s.%s_val = 0;",
+				    plist->pl.param_name, defp1->def_name);
+			    f_print(fout, "\n\t%s.%s_len = 0;",
+				    plist->pl.param_name, defp1->def_name);
 			    s_print(plist->pl.string_name, "%s_val",
 				    defp1->def_name);
 			}
 			break;
 		    case REL_POINTER:
+			f_print(fout, "\n\t%s = 0;", plist->pl.param_name);
 			plist->pl.string_name = NULL;
 			break;
 		    default:
@@ -1541,12 +1550,17 @@ ss_ProcSpecial_setup(definition * defp, int *somefrees)
 	    }
 	}
     }
-
-    for (plist = defp->pc.plists; plist; plist = plist->next) {
-	if (plist->component_kind == DEF_PARAM) {
-	    fprintf(fout, "\n\tmemset(&%s, 0, sizeof(%s));",
-		    plist->pl.param_name,
-		    plist->pl.param_name);
+    for (listp = complex_defined; listp != NULL; listp = listp->next) {
+	defp1 = (definition *) listp->val;
+	for (plist = defp->pc.plists; plist; plist = plist->next) {
+	    if (plist->component_kind == DEF_PARAM) {
+		if (streq(defp1->def_name, structname(plist->pl.param_type))) {
+		    plist->pl.param_flag |= FREETHIS_PARAM;
+		    *somefrees = 1;
+		    fprintf(fout, "\n\tmemset(&%s, 0, sizeof(%s));",
+				 plist->pl.param_name, defp1->def_name);
+		}
+	    }
 	}
     }
 
@@ -1612,8 +1626,6 @@ ss_ProcCallRealProc_setup(definition * defp)
     f_print(fout, ");\n");
     if (zflag) {
 	f_print(fout, "\tif (z_result)\n\t\treturn z_result;\n");
-    } else {
-	f_print(fout, "\tif (z_result)\n\t\tgoto fail;\n");
     }
 }
 
@@ -1668,9 +1680,8 @@ ss_ProcTail_setup(definition * defp, int somefrees)
 	f_print(fout, "\tz_xdrs->x_op = XDR_FREE;\n");
     for (plist = defp->pc.plists; plist; plist = plist->next) {
 	if (plist->component_kind == DEF_PARAM
-	    && (plist->pl.param_flag & FREETHIS_PARAM)) {
+		&& (plist->pl.param_flag & FREETHIS_PARAM))
 	    f_print(fout, "\tif (!%s) goto fail1;\n", plist->scode);
-	}
     }
     for (listp = typedef_defined; listp != NULL; listp = listp->next) {
 	defp1 = (definition *) listp->val;
