@@ -18,17 +18,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-
-#include "uss_acl.h"
-#include "uss_common.h"
-#include "uss_fs.h"
-#include <rx/xdr.h>
-#include <sys/socket.h>
-#include <sys/types.h>
-#include <netdb.h>
-#include <errno.h>
-
-#include <string.h>
+#include <roken.h>
 
 #undef VIRTUE
 #undef VICE
@@ -36,6 +26,11 @@
 #include <afs/prs_fs.h>
 #include <afs/com_err.h>
 #include <afs/afs_consts.h>
+#include <rx/xdr.h>
+
+#include "uss_acl.h"
+#include "uss_common.h"
+#include "uss_fs.h"
 
 #define MAXNAME 100
 
@@ -343,10 +338,9 @@ static char *
 SkipLine(char *a_str)
 {				/*SkipLine */
 
-    while (*a_str != '\0' && *a_str != '\n')
-	a_str++;
-    if (*a_str == '\n')
+    while (*a_str != '\n')
 	a_str++;
+    a_str++;
     return (a_str);
 
 }				/*SkipLine */
@@ -409,8 +403,8 @@ static struct Acl *
 ParseAcl(char *a_str)
 {				/*ParseAcl */
 
-    int nplus = 0, nminus = 0, i, trights = 0;
-    char tname[MAXNAME + 1] = "";
+    int nplus, nminus, i, trights;
+    char tname[MAXNAME];
     struct AclEntry *first, *last, *tl;
     struct Acl *ta;
 
@@ -426,7 +420,7 @@ ParseAcl(char *a_str)
     /*
      * Allocate and initialize the first entry.
      */
-    ta = calloc(sizeof(*ta), 1);
+    ta = (struct Acl *)malloc(sizeof(struct Acl));
     ta->nplus = nplus;
     ta->nminus = nminus;
 
@@ -438,7 +432,7 @@ ParseAcl(char *a_str)
     for (i = 0; i < nplus; i++) {
 	sscanf(a_str, "%100s %d", tname, &trights);
 	a_str = SkipLine(a_str);
-	tl = calloc(sizeof(*tl), 1);
+	tl = (struct AclEntry *)malloc(sizeof(struct AclEntry));
 	if (!first)
 	    first = tl;
 	strcpy(tl->name, tname);
@@ -458,7 +452,7 @@ ParseAcl(char *a_str)
     for (i = 0; i < nminus; i++) {
 	sscanf(a_str, "%100s %d", tname, &trights);
 	a_str = SkipLine(a_str);
-	tl = calloc(sizeof(*tl), 1);
+	tl = (struct AclEntry *)malloc(sizeof(struct AclEntry));
 	if (!first)
 	    first = tl;
 	strcpy(tl->name, tname);
