@@ -1,7 +1,7 @@
 /*
- * Copyright 2006-2008, Sine Nomine Associates and others.
+ * Copyright 2006-2010, Sine Nomine Associates and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -18,29 +18,15 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <dirent.h>
-#include <sys/stat.h>
-#include <time.h>
-#include <errno.h>
 #ifdef AFS_NT40_ENV
-#include <io.h>
 #include <WINNT/afsevent.h>
-#else
-#include <sys/param.h>
-#include <sys/file.h>
-#ifndef ITIMER_REAL
-#include <sys/time.h>
-#endif /* ITIMER_REAL */
 #endif
+
 #include <rx/xdr.h>
 #include <afs/afsint.h>
-#include <afs/assert.h>
-
-#include <fcntl.h>
+#include <afs/afs_assert.h>
 
 #ifndef AFS_NT40_ENV
 #include <afs/osi_inode.h>
@@ -57,9 +43,13 @@
 #include "ihandle.h"
 #include "vnode.h"
 #include "volume.h"
+#include "volume_inline.h"
 #include "partition.h"
 #include "daemon_com.h"
+#include "daemon_com_inline.h"
 #include "fssync.h"
+#include "fssync_inline.h"
+#include "vg_cache.h"
 #ifdef AFS_NT40_ENV
 #include <pthread.h>
 #endif
@@ -74,20 +64,22 @@ struct volop_state {
     char partName[16];
 };
 
-struct state {
+struct fssync_state {
     afs_int32 reason;
     struct volop_state * vop;
 };
 
-static int common_prolog(struct cmd_syndesc *, struct state *);
-static int common_volop_prolog(struct cmd_syndesc *, struct state *);
+#ifndef AFS_DEMAND_ATTACH_FS
+/* remember argv/argc for later, if we need to re-exec */
+static char **fssd_argv;
+static int fssd_argc;
+#endif
 
-static int do_volop(struct state *, afs_int32 command, SYNC_response * res);
+static int common_prolog(struct cmd_syndesc *, struct fssync_state *);
+static int common_volop_prolog(struct cmd_syndesc *, struct fssync_state *);
 
-static char * response_code_to_string(afs_int32);
-static char * command_code_to_string(afs_int32);
-static char * reason_code_to_string(afs_int32);
-static char * program_type_to_string(afs_int32);
+static int do_volop(struct fssync_state *, afs_int32 command,
+		    SYNC_response * res);
 
 static int VolOnline(struct cmd_syndesc * as, void * rock);
 static int VolOffline(struct cmd_syndesc * as, void * rock);
@@ -105,6 +97,12 @@ static int VolOpQuery(struct cmd_syndesc * as, void * rock);
 static int StatsQuery(struct cmd_syndesc * as, void * rock);
 static int VnQuery(struct cmd_syndesc * as, void * rock);
 
+static int VGCQuery(struct cmd_syndesc * as, void * rock);
+static int VGCAdd(struct cmd_syndesc * as, void * rock);
+static int VGCDel(struct cmd_syndesc * as, void * rock);
+static int VGCScan(struct cmd_syndesc * as, void * rock);
+static int VGCScanAll(struct cmd_syndesc * as, void * rock);
+
 static void print_vol_stats_general(VolPkgStats * stats);
 static void print_vol_stats_viceP(struct DiskPartitionStats64 * stats);
 static void print_vol_stats_hash(struct VolumeHashChainStats * stats);
@@ -154,7 +152,11 @@ main(int argc, char **argv)
 	exit(2);
     }
 
-    
+#ifndef AFS_DEMAND_ATTACH_FS
+    fssd_argv = argv;
+    fssd_argc = argc;
+#endif
+
     ts = cmd_CreateSyntax("online", VolOnline, NULL, "bring a volume online (FSYNC_VOL_ON opcode)");
     VOLOP_PARMS_DECL(ts);
 
@@ -205,7 +207,7 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-volumeid", CMD_SINGLE, 0, "volume id");
     cmd_AddParm(ts, "-vnodeid", CMD_SINGLE, 0, "vnode id");
     cmd_AddParm(ts, "-unique", CMD_SINGLE, 0, "uniquifier");
-    cmd_AddParm(ts, "-partition", CMD_SINGLE, 0, "paritition name");
+    cmd_AddParm(ts, "-partition", CMD_SINGLE, 0, "partition name");
     COMMON_PARMS_DECL(ts);
 
     ts = cmd_CreateSyntax("stats", StatsQuery, NULL, "see 'stats help' for more information");
@@ -215,14 +217,89 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-arg2", CMD_SINGLE, CMD_OPTIONAL, "arg2");
     COMMON_PARMS_DECL(ts);
 
+    ts = cmd_CreateSyntax("vgcquery", VGCQuery, 0, "query volume group cache (FSYNC_VG_QUERY opcode)");
+    cmd_Seek(ts, CUSTOM_PARMS_OFFSET);
+    cmd_AddParm(ts, "-partition", CMD_SINGLE, 0, "partition name");
+    cmd_AddParm(ts, "-volumeid", CMD_SINGLE, 0, "volume id");
+    COMMON_PARMS_DECL(ts);
+    cmd_CreateAlias(ts, "vgcqry");
+
+    ts = cmd_CreateSyntax("vgcadd", VGCAdd, 0, "add entry to volume group cache (FSYNC_VG_ADD opcode)");
+    cmd_Seek(ts, CUSTOM_PARMS_OFFSET);
+    cmd_AddParm(ts, "-partition", CMD_SINGLE, 0, "partition name");
+    cmd_AddParm(ts, "-parent", CMD_SINGLE, 0, "parent volume id");
+    cmd_AddParm(ts, "-child", CMD_SINGLE, 0, "child volume id");
+    COMMON_PARMS_DECL(ts);
+
+    ts = cmd_CreateSyntax("vgcdel", VGCDel, 0, "delete entry from volume group cache (FSYNC_VG_DEL opcode)");
+    cmd_Seek(ts, CUSTOM_PARMS_OFFSET);
+    cmd_AddParm(ts, "-partition", CMD_SINGLE, 0, "partition name");
+    cmd_AddParm(ts, "-parent", CMD_SINGLE, 0, "parent volume id");
+    cmd_AddParm(ts, "-child", CMD_SINGLE, 0, "child volume id");
+    COMMON_PARMS_DECL(ts);
+
+    ts = cmd_CreateSyntax("vgcscan", VGCScan, 0,
+			  "start volume group cache re-scan"
+			  " (FSYNC_VG_SCAN opcode)");
+    cmd_Seek(ts, CUSTOM_PARMS_OFFSET);
+    cmd_AddParm(ts, "-partition", CMD_SINGLE, 0, "partition name");
+    COMMON_PARMS_DECL(ts);
+
+    ts = cmd_CreateSyntax("vgcscanall", VGCScanAll, 0,
+			  "start whole-server volume group cache re-scan"
+			  " (FSYNC_VG_SCAN_ALL opcode)");
+    COMMON_PARMS_DECL(ts);
+
     err = cmd_Dispatch(argc, argv);
     exit(err);
 }
 
-static int
-common_prolog(struct cmd_syndesc * as, struct state * state)
+#ifdef AFS_DEMAND_ATTACH_FS
+# define dafs_prolog()
+#else
+/* Try to detect if the fileserver is DAFS, and if so, re-exec as the
+ * DAFS-enabled fssync-debug (dafssync_debug). If we fail to detect or
+ * exec, just try to proceed anyway as if the server is not DAFS */
+static void
+dafs_prolog(void)
 {
-    register struct cmd_item *ti;
+    SYNC_response res;
+    SYNC_PROTO_BUF_DECL(res_buf);
+    afs_int32 code;
+    char *dfssd;
+
+    res.payload.len = SYNC_PROTO_MAX_LEN;
+    res.payload.buf = res_buf;
+
+    /* LISTVOLUMES is a no-op; we just want to get the response header flags
+     * to see if the server reports itself as DAFS or not */
+    code = FSYNC_VolOp(0, NULL, FSYNC_VOL_LISTVOLUMES, FSYNC_WHATEVER, &res);
+    if (code) {
+	/* probably failed to contact the fileserver; later code will provide
+	 * some warning/error indication */
+	return;
+    }
+
+    if (!(res.hdr.flags & SYNC_FLAG_DAFS_EXTENSIONS)) {
+	/* fileserver is not DAFS, so we don't need to do anything */
+	return;
+    }
+
+    dfssd = afs_exec_alt(fssd_argc, fssd_argv, "da", NULL);
+
+    fprintf(stderr, "\n*** server asserted demand attach extensions, but we failed\n"
+                    "*** to exec a DAFS-enabled fssync-debug '%s' (errno=%d);\n"
+                    "*** attempting to proceed without it.\n\n", dfssd, errno);
+
+    free(dfssd);
+}
+#endif /* !AFS_DEMAND_ATTACH_FS */
+
+static int
+common_prolog(struct cmd_syndesc * as, struct fssync_state * state)
+{
+    struct cmd_item *ti;
+    VolumePackageOptions opts;
 
 #ifdef AFS_NT40_ENV
     if (afs_winsockInit() < 0) {
@@ -230,13 +307,22 @@ common_prolog(struct cmd_syndesc * as, struct state * state)
     }
 #endif
 
-    VInitVolumePackage(debugUtility, 1, 1,
-		       DONT_CONNECT_FS, 0);
+    VOptDefaults(debugUtility, &opts);
+    if (VInitVolumePackage2(debugUtility, &opts)) {
+	/* VInitVolumePackage2 can fail on e.g. partition attachment errors,
+	 * but we don't really care, since all we're doing is trying to use
+	 * FSSYNC */
+	fprintf(stderr, "errors encountered initializing volume package, but "
+	                "trying to continue anyway\n");
+    }
     DInit(1);
 
     if ((ti = as->parms[COMMON_PARMS_OFFSET].items)) {	/* -reason */
 	state->reason = atoi(ti->data);
+    } else {
+	state->reason = FSYNC_WHATEVER;
     }
+
     if ((ti = as->parms[COMMON_PARMS_OFFSET+1].items)) {	/* -programtype */
 	if (!strcmp(ti->data, "fileServer")) {
 	    programType = fileServer;
@@ -246,6 +332,10 @@ common_prolog(struct cmd_syndesc * as, struct state * state)
 	    programType = salvager;
 	} else if (!strcmp(ti->data, "salvageServer")) {
 	    programType = salvageServer;
+	} else if (!strcmp(ti->data, "volumeServer")) {
+	    programType = volumeServer;
+	} else if (!strcmp(ti->data, "volumeSalvager")) {
+	    programType = volumeSalvager;
 	} else {
 	    programType = (ProgramType) atoi(ti->data);
 	}
@@ -257,12 +347,12 @@ common_prolog(struct cmd_syndesc * as, struct state * state)
 }
 
 static int
-common_volop_prolog(struct cmd_syndesc * as, struct state * state)
+common_volop_prolog(struct cmd_syndesc * as, struct fssync_state * state)
 {
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
 
     state->vop = (struct volop_state *) calloc(1, sizeof(struct volop_state));
-    assert(state->vop != NULL);
+    osi_Assert(state->vop != NULL);
 
     if ((ti = as->parms[COMMON_VOLOP_PARMS_OFFSET].items)) {	/* -volumeid */
 	state->vop->volume = atoi(ti->data);
@@ -280,7 +370,30 @@ common_volop_prolog(struct cmd_syndesc * as, struct state * state)
 }
 
 static int
-do_volop(struct state * state, afs_int32 command, SYNC_response * res)
+debug_response(afs_int32 code, SYNC_response * res)
+{
+    switch (code) {
+    case SYNC_OK:
+    case SYNC_DENIED:
+	break;
+    default:
+	fprintf(stderr, "warning: response code indicates possible protocol error.\n");
+    }
+
+    fprintf(stderr, "FSSYNC service returned %d (%s)\n", code, SYNC_res2string(code));
+
+    if (res) {
+	fprintf(stderr, "protocol header response code was %d (%s)\n",
+	        res->hdr.response, SYNC_res2string(res->hdr.response));
+	fprintf(stderr, "protocol reason code was %d (%s)\n",
+	        res->hdr.reason, FSYNC_reason2string(res->hdr.reason));
+    }
+
+    return 0;
+}
+
+static int
+do_volop(struct fssync_state * state, afs_int32 command, SYNC_response * res)
 {
     afs_int32 code;
     SYNC_PROTO_BUF_DECL(res_buf);
@@ -292,8 +405,8 @@ do_volop(struct state * state, afs_int32 command, SYNC_response * res)
 	res->payload.buf = res_buf;
     }
 
-    fprintf(stderr, "calling FSYNC_VolOp with command code %d (%s)\n", 
-	    command, command_code_to_string(command));
+    fprintf(stderr, "calling FSYNC_VolOp with command code %d (%s)\n",
+	    command, FSYNC_com2string(command));
 
     code = FSYNC_VolOp(state->vop->volume,
 		       state->vop->partName,
@@ -301,19 +414,7 @@ do_volop(struct state * state, afs_int32 command, SYNC_response * res)
 		       state->reason,
 		       res);
 
-    switch (code) {
-    case SYNC_OK:
-    case SYNC_DENIED:
-	break;
-    default:
-	fprintf(stderr, "possible sync protocol error. return code was %d\n", code);
-    }
-
-    fprintf(stderr, "FSYNC_VolOp returned %d (%s)\n", code, response_code_to_string(code));
-    fprintf(stderr, "protocol response code was %d (%s)\n", 
-	    res->hdr.response, response_code_to_string(res->hdr.response));
-    fprintf(stderr, "protocol reason code was %d (%s)\n", 
-	    res->hdr.reason, reason_code_to_string(res->hdr.reason));
+    debug_response(code, res);
 
     VDisconnectFS();
 
@@ -324,9 +425,7 @@ do_volop(struct state * state, afs_int32 command, SYNC_response * res)
 
 #define ENUMTOSTRING(en)  #en
 #define ENUMCASE(en) \
-    case en: \
-        return ENUMTOSTRING(en); \
-        break
+    case en: return ENUMTOSTRING(en)
 
 #define FLAGTOSTRING(fl)  #fl
 #define FLAGCASE(bitstr, fl, str, count) \
@@ -339,90 +438,10 @@ do_volop(struct state * state, afs_int32 command, SYNC_response * res)
         } \
     } while (0)
 
-static char *
-response_code_to_string(afs_int32 response)
-{
-    switch (response) {
-	ENUMCASE(SYNC_OK);
-	ENUMCASE(SYNC_DENIED);
-	ENUMCASE(SYNC_COM_ERROR);
-	ENUMCASE(SYNC_BAD_COMMAND);
-	ENUMCASE(SYNC_FAILED);
-    default:
-	return "**UNKNOWN**";
-    }
-}
-
-static char *
-command_code_to_string(afs_int32 command)
-{
-    switch (command) {
-	ENUMCASE(SYNC_COM_CHANNEL_CLOSE);
-	ENUMCASE(FSYNC_VOL_ON);
-	ENUMCASE(FSYNC_VOL_OFF);
-	ENUMCASE(FSYNC_VOL_LISTVOLUMES);
-	ENUMCASE(FSYNC_VOL_NEEDVOLUME);
-	ENUMCASE(FSYNC_VOL_MOVE);
-	ENUMCASE(FSYNC_VOL_BREAKCBKS);
-	ENUMCASE(FSYNC_VOL_DONE);
-	ENUMCASE(FSYNC_VOL_QUERY);
-	ENUMCASE(FSYNC_VOL_QUERY_HDR);
-	ENUMCASE(FSYNC_VOL_QUERY_VOP);
-	ENUMCASE(FSYNC_VOL_STATS_GENERAL);
-	ENUMCASE(FSYNC_VOL_STATS_VICEP);
-	ENUMCASE(FSYNC_VOL_STATS_HASH);
-	ENUMCASE(FSYNC_VOL_STATS_HDR);
-	ENUMCASE(FSYNC_VOL_STATS_VLRU);
-	ENUMCASE(FSYNC_VOL_ATTACH);
-	ENUMCASE(FSYNC_VOL_FORCE_ERROR);
-	ENUMCASE(FSYNC_VOL_LEAVE_OFF);
-	ENUMCASE(FSYNC_VOL_QUERY_VNODE);
-    default:
-	return "**UNKNOWN**";
-    }
-}
-
-static char *
-reason_code_to_string(afs_int32 reason)
-{
-    switch (reason) {
-	ENUMCASE(SYNC_REASON_NONE);
-	ENUMCASE(SYNC_REASON_MALFORMED_PACKET);
-	ENUMCASE(SYNC_REASON_NOMEM);
-	ENUMCASE(SYNC_REASON_ENCODING_ERROR);
-	ENUMCASE(FSYNC_WHATEVER);
-	ENUMCASE(FSYNC_SALVAGE);
-	ENUMCASE(FSYNC_MOVE);
-	ENUMCASE(FSYNC_OPERATOR);
-	ENUMCASE(FSYNC_EXCLUSIVE);
-	ENUMCASE(FSYNC_UNKNOWN_VOLID);
-	ENUMCASE(FSYNC_HDR_NOT_ATTACHED);
-	ENUMCASE(FSYNC_NO_PENDING_VOL_OP);
-	ENUMCASE(FSYNC_VOL_PKG_ERROR);
-	ENUMCASE(FSYNC_UNKNOWN_VNID);
-    default:
-	return "**UNKNOWN**";
-    }
-}
-
-static char *
-program_type_to_string(afs_int32 type)
-{
-    switch ((ProgramType)type) {
-	ENUMCASE(fileServer);
-	ENUMCASE(volumeUtility);
-	ENUMCASE(salvager);
-	ENUMCASE(salvageServer);
-	ENUMCASE(debugUtility);
-    default:
-	return "**UNKNOWN**";
-    }
-}
-
-static int 
+static int
 VolOnline(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -432,10 +451,10 @@ VolOnline(struct cmd_syndesc * as, void * rock)
     return 0;
 }
 
-static int 
+static int
 VolOffline(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -448,7 +467,7 @@ VolOffline(struct cmd_syndesc * as, void * rock)
 static int
 VolMode(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -461,7 +480,7 @@ VolMode(struct cmd_syndesc * as, void * rock)
 static int
 VolDetach(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -474,7 +493,7 @@ VolDetach(struct cmd_syndesc * as, void * rock)
 static int
 VolBreakCBKs(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -487,7 +506,7 @@ VolBreakCBKs(struct cmd_syndesc * as, void * rock)
 static int
 VolMove(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -500,7 +519,7 @@ VolMove(struct cmd_syndesc * as, void * rock)
 static int
 VolList(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -513,7 +532,7 @@ VolList(struct cmd_syndesc * as, void * rock)
 static int
 VolLeaveOff(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -526,7 +545,7 @@ VolLeaveOff(struct cmd_syndesc * as, void * rock)
 static int
 VolForceAttach(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -539,7 +558,7 @@ VolForceAttach(struct cmd_syndesc * as, void * rock)
 static int
 VolForceError(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_volop_prolog(as, &state);
@@ -574,6 +593,8 @@ vol_state_to_string(VolState state)
 	ENUMCASE(VOL_STATE_VNODE_CLOSE);
 	ENUMCASE(VOL_STATE_VNODE_RELEASE);
 	ENUMCASE(VOL_STATE_VLRU_ADD);
+	ENUMCASE(VOL_STATE_DELETED);
+	ENUMCASE(VOL_STATE_SALVAGE_REQ);
 	ENUMCASE(VOL_STATE_FREED);
     default:
 	return "**UNKNOWN**";
@@ -583,7 +604,7 @@ vol_state_to_string(VolState state)
 static char *
 vol_flags_to_string(afs_uint16 flags)
 {
-    static char str[128];
+    static char str[256];
     int count = 0;
     str[0]='\0';
 
@@ -595,6 +616,7 @@ vol_flags_to_string(afs_uint16 flags)
     FLAGCASE(flags, VOL_IS_BUSY, str, count);
     FLAGCASE(flags, VOL_ON_VLRU, str, count);
     FLAGCASE(flags, VOL_HDR_DONTSALV, str, count);
+    FLAGCASE(flags, VOL_LOCKED, str, count);
 
     return str;
 }
@@ -652,13 +674,12 @@ vn_flags_to_string(afs_uint32 flags)
 static int
 VolQuery(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
     SYNC_PROTO_BUF_DECL(res_buf);
     SYNC_response res;
     Volume v;
-#ifdef AFS_DEMAND_ATTACH_FS
-    int hi, lo;
-#endif
+
+    dafs_prolog();
 
     res.hdr.response_len = sizeof(res.hdr);
     res.payload.buf = res_buf;
@@ -688,7 +709,7 @@ VolQuery(struct cmd_syndesc * as, void * rock)
 	printf("\tneedsPutBack    = %d\n", v.needsPutBack);
 	printf("\tspecialStatus   = %d\n", v.specialStatus);
 	printf("\tupdateTime      = %u\n", v.updateTime);
-	
+
 	printf("\tvnodeIndex[vSmall] = {\n");
         printf("\t\thandle       = %p\n", v.vnodeIndex[vSmall].handle);
         printf("\t\tbitmap       = %p\n", v.vnodeIndex[vSmall].bitmap);
@@ -708,7 +729,7 @@ VolQuery(struct cmd_syndesc * as, void * rock)
 	    printf("\tattach_flags    = %s\n", vol_flags_to_string(v.attach_flags));
 	    printf("\tnWaiters        = %d\n", v.nWaiters);
 	    printf("\tchainCacheCheck = %d\n", v.chainCacheCheck);
-	    
+
 	    /* online salvage structure */
 	    printf("\tsalvage = {\n");
 	    printf("\t\tprio      = %u\n", v.salvage.prio);
@@ -716,39 +737,25 @@ VolQuery(struct cmd_syndesc * as, void * rock)
 	    printf("\t\trequested = %d\n", v.salvage.requested);
 	    printf("\t\tscheduled = %d\n", v.salvage.scheduled);
 	    printf("\t}\n");
-	    
+
 	    /* statistics structure */
 	    printf("\tstats = {\n");
 
-	    printf("\t\thash_lookups = {\n");
-	    SplitInt64(v.stats.hash_lookups,hi,lo);
-	    printf("\t\t\thi = %u\n", hi);
-	    printf("\t\t\tlo = %u\n", lo);
+	    printf("\t\thash_lookups = %"AFS_INT64_FMT"\n",
+		   v.stats.hash_lookups);
+	    printf("\t\thash_short_circuits = %"AFS_INT64_FMT"\n",
+		   v.stats.hash_short_circuits);
+	    printf("\t\thdr_loads = %"AFS_INT64_FMT"\n",
+		   v.stats.hdr_loads);
+	    printf("\t\thdr_gets = %"AFS_INT64_FMT"\n",
+		   v.stats.hdr_gets);
 	    printf("\t\t}\n");
 
-	    printf("\t\thash_short_circuits = {\n");
-	    SplitInt64(v.stats.hash_short_circuits,hi,lo);
-	    printf("\t\t\thi = %u\n", hi);
-	    printf("\t\t\tlo = %u\n", lo);
-	    printf("\t\t}\n");
-
-	    printf("\t\thdr_loads = {\n");
-	    SplitInt64(v.stats.hdr_loads,hi,lo);
-	    printf("\t\t\thi = %u\n", hi);
-	    printf("\t\t\tlo = %u\n", lo);
-	    printf("\t\t}\n");
-
-	    printf("\t\thdr_gets = {\n");
-	    SplitInt64(v.stats.hdr_gets,hi,lo);
-	    printf("\t\t\thi = %u\n", hi);
-	    printf("\t\t\tlo = %u\n", lo);
-	    printf("\t\t}\n");
-	    
 	    printf("\t\tattaches         = %u\n", v.stats.attaches);
 	    printf("\t\tsoft_detaches    = %u\n", v.stats.soft_detaches);
 	    printf("\t\tsalvages         = %u\n", v.stats.salvages);
 	    printf("\t\tvol_ops          = %u\n", v.stats.vol_ops);
-	    
+
 	    printf("\t\tlast_attach      = %u\n", v.stats.last_attach);
 	    printf("\t\tlast_get         = %u\n", v.stats.last_get);
 	    printf("\t\tlast_promote     = %u\n", v.stats.last_promote);
@@ -758,10 +765,10 @@ VolQuery(struct cmd_syndesc * as, void * rock)
 	    printf("\t\tlast_salvage_req = %u\n", v.stats.last_salvage_req);
 	    printf("\t\tlast_vol_op      = %u\n", v.stats.last_vol_op);
 	    printf("\t}\n");
-	    
+
 	    /* VLRU state */
 	    printf("\tvlru = {\n");
-	    printf("\t\tidx = %d (%s)\n", 
+	    printf("\t\tidx = %d (%s)\n",
 		   v.vlru.idx, vlru_idx_to_string(v.vlru.idx));
 	    printf("\t}\n");
 
@@ -770,9 +777,10 @@ VolQuery(struct cmd_syndesc * as, void * rock)
 	}
 #else /* !AFS_DEMAND_ATTACH_FS */
 	if (res.hdr.flags & SYNC_FLAG_DAFS_EXTENSIONS) {
-	    printf("*** server asserted demand attach extensions. fssync-debug not built to\n");
-	    printf("*** recognize those extensions. please recompile fssync-debug if you need\n");
-	    printf("*** to dump dafs extended state\n");
+	    printf("*** server asserted demand attach extensions. this fssync-debug\n"
+	           "*** is not built to recognize those extensions. please use an\n"
+	           "*** fssync-debug with demand attach if you need to dump dafs\n"
+	           "*** extended state.\n");
 	}
 #endif /* !AFS_DEMAND_ATTACH_FS */
 	printf("}\n");
@@ -784,7 +792,7 @@ VolQuery(struct cmd_syndesc * as, void * rock)
 static int
 VolHdrQuery(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
     SYNC_PROTO_BUF_DECL(res_buf);
     SYNC_response res;
     VolumeDiskData v;
@@ -807,11 +815,11 @@ VolHdrQuery(struct cmd_syndesc * as, void * rock)
 	printf("\t\tmagic   = 0x%x\n", v.stamp.magic);
 	printf("\t\tversion = %u\n", v.stamp.version);
 	printf("\t}\n");
-	
+
 	printf("\tid               = %u\n", v.id);
 	printf("\tname             = '%s'\n", v.name);
 	if (v.inUse != 0) {
-	    printf("\tinUse            = %d (%s)\n", v.inUse, program_type_to_string(v.inUse));
+	    printf("\tinUse            = %d (%s)\n", v.inUse, VPTypeToString(v.inUse));
 	} else {
 	    printf("\tinUse            = %d (no)\n", v.inUse);
 	}
@@ -859,7 +867,7 @@ VolHdrQuery(struct cmd_syndesc * as, void * rock)
 static int
 VolOpQuery(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
     SYNC_PROTO_BUF_DECL(res_buf);
     SYNC_response res;
     FSSYNC_VolOp_info vop;
@@ -887,21 +895,21 @@ VolOpQuery(struct cmd_syndesc * as, void * rock)
 	printf("\t\tproto_version  = %u\n", vop.com.proto_version);
 	printf("\t\tpkt_seq        = %u\n", vop.com.pkt_seq);
 	printf("\t\tcom_seq        = %u\n", vop.com.com_seq);
-	printf("\t\tprogramType    = %d (%s)\n", 
-	       vop.com.programType, program_type_to_string(vop.com.programType));
+	printf("\t\tprogramType    = %d (%s)\n",
+	       vop.com.programType, VPTypeToString(vop.com.programType));
 	printf("\t\tpid            = %d\n", vop.com.pid);
 	printf("\t\ttid            = %d\n", vop.com.tid);
-	printf("\t\tcommand        = %d (%s)\n", 
-	       vop.com.command, command_code_to_string(vop.com.command));
-	printf("\t\treason         = %d (%s)\n", 
-	       vop.com.reason, reason_code_to_string(vop.com.reason));
+	printf("\t\tcommand        = %d (%s)\n",
+	       vop.com.command, FSYNC_com2string(vop.com.command));
+	printf("\t\treason         = %d (%s)\n",
+	       vop.com.reason, FSYNC_reason2string(vop.com.reason));
 	printf("\t\tcommand_len    = %u\n", vop.com.command_len);
 	printf("\t\tflags          = 0x%lux\n", afs_printable_uint32_lu(vop.com.flags));
 	printf("\t}\n");
 
 	printf("\tvop = {\n");
 	printf("\t\tvolume         = %u\n", vop.vop.volume);
-	if (afs_strnlen(vop.vop.partName, sizeof(vop.vop.partName)) <
+	if (strnlen(vop.vop.partName, sizeof(vop.vop.partName)) <
 	    sizeof(vop.vop.partName)) {
 	    printf("\t\tpartName       = '%s'\n", vop.vop.partName);
 	} else {
@@ -916,12 +924,12 @@ VolOpQuery(struct cmd_syndesc * as, void * rock)
 }
 
 static int
-vn_prolog(struct cmd_syndesc * as, struct state * state)
+vn_prolog(struct cmd_syndesc * as, struct fssync_state * state)
 {
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
 
     state->vop = (struct volop_state *) calloc(1, sizeof(struct volop_state));
-    assert(state->vop != NULL);
+    osi_Assert(state->vop != NULL);
 
     if ((ti = as->parms[CUSTOM_PARMS_OFFSET].items)) {	/* -volumeid */
 	state->vop->volume = atoi(ti->data);
@@ -951,7 +959,7 @@ vn_prolog(struct cmd_syndesc * as, struct state * state)
 }
 
 static int
-do_vnqry(struct state * state, SYNC_response * res)
+do_vnqry(struct fssync_state * state, SYNC_response * res)
 {
     afs_int32 code;
     int command = FSYNC_VOL_QUERY_VNODE;
@@ -963,24 +971,12 @@ do_vnqry(struct state * state, SYNC_response * res)
     qry.spare = 0;
     strlcpy(qry.partName, state->vop->partName, sizeof(qry.partName));
 
-    fprintf(stderr, "calling FSYNC_GenericOp with command code %d (%s)\n", 
-	    command, command_code_to_string(command));
+    fprintf(stderr, "calling FSYNC_GenericOp with command code %d (%s)\n",
+	    command, FSYNC_com2string(command));
 
     code = FSYNC_GenericOp(&qry, sizeof(qry), command, FSYNC_OPERATOR, res);
 
-    switch (code) {
-    case SYNC_OK:
-    case SYNC_DENIED:
-	break;
-    default:
-	fprintf(stderr, "possible sync protocol error. return code was %d\n", code);
-    }
-
-    fprintf(stderr, "FSYNC_GenericOp returned %d (%s)\n", code, response_code_to_string(code));
-    fprintf(stderr, "protocol response code was %d (%s)\n", 
-	    res->hdr.response, response_code_to_string(res->hdr.response));
-    fprintf(stderr, "protocol reason code was %d (%s)\n", 
-	    res->hdr.reason, reason_code_to_string(res->hdr.reason));
+    debug_response(code, res);
 
     VDisconnectFS();
 
@@ -990,11 +986,13 @@ do_vnqry(struct state * state, SYNC_response * res)
 static int
 VnQuery(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
     SYNC_PROTO_BUF_DECL(res_buf);
     SYNC_response res;
     Vnode v;
 
+    dafs_prolog();
+
     res.hdr.response_len = sizeof(res.hdr);
     res.payload.buf = res_buf;
     res.payload.len = SYNC_PROTO_MAX_LEN;
@@ -1005,7 +1003,7 @@ VnQuery(struct cmd_syndesc * as, void * rock)
     do_vnqry(&state, &res);
 
     if (res.hdr.response == SYNC_OK) {
-	memcpy(&v, res.payload.buf, sizeof(Volume));
+	memcpy(&v, res.payload.buf, sizeof(Vnode));
 
 	printf("vnode = {\n");
 
@@ -1028,9 +1026,10 @@ VnQuery(struct cmd_syndesc * as, void * rock)
 
 #ifdef AFS_DEMAND_ATTACH_FS
 	if (!(res.hdr.flags & SYNC_FLAG_DAFS_EXTENSIONS)) {
-	    printf("*** fssync-debug built to expect demand attach extensions.  server asserted\n");
-	    printf("*** that it was not compiled with demand attach turned on.  please recompile\n");
-	    printf("*** fssync-debug to match your server\n");
+	    printf("*** this fssync-debug is built to expect demand attach extensions.\n"
+	           "*** server asserted that is was not compiled with demand attach.\n"
+	           "*** please use an fssync-debug without demand attach to match your\n"
+	           "*** server.\n");
 	    goto done;
 	}
 
@@ -1039,9 +1038,9 @@ VnQuery(struct cmd_syndesc * as, void * rock)
 	printf("\tvn_state        = %s\n", vn_state_to_string(v.vn_state));
 #else
 	if (res.hdr.flags & SYNC_FLAG_DAFS_EXTENSIONS) {
-	    printf("*** server asserted demand attach extensions. fssync-debug not built to\n");
-	    printf("*** recognize those extensions. please recompile fssync-debug if you need\n");
-	    printf("*** to dump dafs extended state\n");
+	    printf("*** server asserted demand attach extensions. this fssync-debug is not\n"
+	           "*** built to recognize those extensions. please use an fssync-debug\n"
+	           "*** with demand attach to match your server.\n");
 	    goto done;
 	}
 #endif /* !AFS_DEMAND_ATTACH_FS */
@@ -1089,21 +1088,11 @@ StatsQuery(struct cmd_syndesc * as, void * rock)
     afs_int32 code;
     int command;
     struct cmd_item *ti;
-    struct state state;
+    struct fssync_state state;
     SYNC_PROTO_BUF_DECL(res_buf);
     SYNC_response res;
     FSSYNC_StatsOp_hdr scom;
-    union {
-	void * ptr;
-	struct VolPkgStats * vol_stats;
-	struct VolumeHashChainStats * hash_stats;
-#ifdef AFS_DEMAND_ATTACH_FS
-	struct volume_hdr_LRU_stats * hdr_stats;
-#endif
-	struct DiskPartitionStats64 * vicep_stats;
-    } sres;
 
-    sres.ptr = res_buf;
     res.hdr.response_len = sizeof(res.hdr);
     res.payload.buf = res_buf;
     res.payload.len = SYNC_PROTO_MAX_LEN;
@@ -1166,8 +1155,8 @@ StatsQuery(struct cmd_syndesc * as, void * rock)
 
     common_prolog(as, &state);
 
-    fprintf(stderr, "calling FSYNC_askfs with command code %d (%s)\n", 
-	    command, command_code_to_string(command));
+    fprintf(stderr, "calling FSYNC_askfs with command code %d (%s)\n",
+	    command, FSYNC_com2string(command));
 
     code = FSYNC_StatsOp(&scom, command, FSYNC_WHATEVER, &res);
 
@@ -1179,29 +1168,45 @@ StatsQuery(struct cmd_syndesc * as, void * rock)
 	fprintf(stderr, "possible sync protocol error. return code was %d\n", code);
     }
 
-    fprintf(stderr, "FSYNC_VolOp returned %d (%s)\n", code, response_code_to_string(code));
-    fprintf(stderr, "protocol response code was %d (%s)\n", 
-	    res.hdr.response, response_code_to_string(res.hdr.response));
-    fprintf(stderr, "protocol reason code was %d (%s)\n", 
-	    res.hdr.reason, reason_code_to_string(res.hdr.reason));
+    fprintf(stderr, "FSYNC_VolOp returned %d (%s)\n", code, SYNC_res2string(code));
+    fprintf(stderr, "protocol response code was %d (%s)\n",
+	    res.hdr.response, SYNC_res2string(res.hdr.response));
+    fprintf(stderr, "protocol reason code was %d (%s)\n",
+	    res.hdr.reason, FSYNC_reason2string(res.hdr.reason));
 
     VDisconnectFS();
 
     if (res.hdr.response == SYNC_OK) {
 	switch (command) {
 	case FSYNC_VOL_STATS_GENERAL:
-	    print_vol_stats_general(sres.vol_stats);
-	    break;
+	    {
+		struct VolPkgStats vol_stats;
+		memcpy(&vol_stats, res_buf, sizeof(vol_stats));
+		print_vol_stats_general(&vol_stats);
+		break;
+	    }
 	case FSYNC_VOL_STATS_VICEP:
-	    print_vol_stats_viceP(sres.vicep_stats);
-	    break;
+	    {
+		struct DiskPartitionStats64 vicep_stats;
+		memcpy(&vicep_stats, res_buf, sizeof(vicep_stats));
+		print_vol_stats_viceP(&vicep_stats);
+		break;
+	    }
 	case FSYNC_VOL_STATS_HASH:
-	    print_vol_stats_hash(sres.hash_stats);
-	    break;
+	    {
+		struct VolumeHashChainStats hash_stats;
+		memcpy(&hash_stats, res_buf, sizeof(hash_stats));
+		print_vol_stats_hash(&hash_stats);
+		break;
+	    }
 #ifdef AFS_DEMAND_ATTACH_FS
 	case FSYNC_VOL_STATS_HDR:
-	    print_vol_stats_hdr(sres.hdr_stats);
-	    break;
+	    {
+		struct volume_hdr_LRU_stats hdr_stats;
+		memcpy(&hdr_stats, res_buf, sizeof(hdr_stats));
+		print_vol_stats_hdr(&hdr_stats);
+		break;
+	    }
 #endif /* AFS_DEMAND_ATTACH_FS */
 	}
     }
@@ -1215,66 +1220,34 @@ print_vol_stats_general(VolPkgStats * stats)
 #ifdef AFS_DEMAND_ATTACH_FS
     int i;
 #endif
-    afs_uint32 hi, lo;
 
     printf("VolPkgStats = {\n");
 #ifdef AFS_DEMAND_ATTACH_FS
     for (i = 0; i < VOL_STATE_COUNT; i++) {
-	printf("\tvol_state_count[%s] = %d\n", 
+	printf("\tvol_state_count[%s] = %d\n",
 	       vol_state_to_string(i),
 	       stats->state_levels[i]);
     }
 
-    SplitInt64(stats->hash_looks, hi, lo);
-    printf("\thash_looks = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->hash_reorders, hi, lo);
-    printf("\thash_reorders = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->salvages, hi, lo);
-    printf("\tsalvages = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->vol_ops, hi, lo);
-    printf("\tvol_ops = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
+    printf("\thash_looks = %"AFS_INT64_FMT"\n",
+	   stats->hash_looks);
+    printf("\thash_reorders = %"AFS_INT64_FMT"\n",
+	   stats->hash_reorders);
+    printf("\tsalvages = %"AFS_INT64_FMT"\n",
+	   stats->salvages);
+    printf("\tvol_ops = %"AFS_INT64_FMT"\n",
+	   stats->vol_ops);
 #endif
-    SplitInt64(stats->hdr_loads, hi, lo);
-    printf("\thdr_loads = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->hdr_gets, hi, lo);
-    printf("\thdr_gets = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->attaches, hi, lo);
-    printf("\tattaches = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->soft_detaches, hi, lo);
-    printf("\tsoft_detaches = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
+    printf("\thdr_loads = %"AFS_INT64_FMT"\n",
+	   stats->hdr_loads);
+    printf("\thdr_gets = %"AFS_INT64_FMT"\n",
+	   stats->hdr_gets);
+    printf("\tattaches = %"AFS_INT64_FMT"\n",
+	   stats->attaches);
+    printf("\tsoft_detaches = %"AFS_INT64_FMT"\n",
+	   stats->soft_detaches);
     printf("\thdr_cache_size = %d\n", stats->hdr_cache_size);
-	    
+
     printf("}\n");
 }
 
@@ -1295,10 +1268,6 @@ print_vol_stats_viceP(struct DiskPartitionStats64 * stats)
 static void
 print_vol_stats_hash(struct VolumeHashChainStats * stats)
 {
-#ifdef AFS_DEMAND_ATTACH_FS
-    afs_uint32 hi, lo;
-#endif
-
     printf("DiskPartitionStats = {\n");
     printf("\ttable_size = %d\n", stats->table_size);
     printf("\tchain_len = %d\n", stats->chain_len);
@@ -1307,23 +1276,12 @@ print_vol_stats_hash(struct VolumeHashChainStats * stats)
     printf("\tchain_cacheCheck = %d\n", stats->chain_cacheCheck);
     printf("\tchain_busy = %d\n", stats->chain_busy);
 
-    SplitInt64(stats->chain_looks, hi, lo);
-    printf("\tchain_looks = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->chain_gets, hi, lo);
-    printf("\tchain_gets = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
-
-    SplitInt64(stats->chain_reorders, hi, lo);
-    printf("\tchain_reorders = {\n");
-    printf("\t\thi = %u\n", hi);
-    printf("\t\tlo = %u\n", lo);
-    printf("\t}\n");
+    printf("\tchain_looks = %"AFS_INT64_FMT"\n",
+	   stats->chain_looks);
+    printf("\tchain_gets = %"AFS_INT64_FMT"\n",
+	   stats->chain_gets);
+    printf("\tchain_reorders = %"AFS_INT64_FMT"\n",
+	   stats->chain_reorders);
 #endif /* AFS_DEMAND_ATTACH_FS */
 
     printf("}\n");
@@ -1342,3 +1300,169 @@ print_vol_stats_hdr(struct volume_hdr_LRU_stats * stats)
 }
 #endif /* AFS_DEMAND_ATTACH_FS */
 
+
+/**
+ * query VGC.
+ *
+ * @notes args:
+ *    - CUSTOM_PARMS_OFFSET+0 is partition string
+ *    - CUSTOM_PARMS_OFFSET+1 is volume id
+ *
+ * @return operation status
+ *    @retval 0 success
+ */
+static int
+VGCQuery(struct cmd_syndesc * as, void * rock)
+{
+    afs_int32 code;
+    struct fssync_state state;
+    char * partName;
+    VolumeId volid;
+    FSSYNC_VGQry_response_t q_res;
+    SYNC_response res;
+    int i;
+    struct cmd_item *ti;
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+0].items)) {	/* -partition */
+	return -1;
+    }
+    partName = ti->data;
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+1].items)) {	/* -volumeid */
+	return -1;
+    }
+    volid = atoi(ti->data);
+
+    common_prolog(as, &state);
+
+    fprintf(stderr, "calling FSYNC_VCGQuery\n");
+
+    code = FSYNC_VGCQuery(partName, volid, &q_res, &res);
+
+    debug_response(code, &res);
+
+    if (code == SYNC_OK) {
+	printf("VG = {\n");
+	printf("\trw\t=\t%u\n", q_res.rw);
+	printf("\tchildren\t= (\n");
+	for (i = 0; i < VOL_VG_MAX_VOLS; i++) {
+	    if (q_res.children[i]) {
+		printf("\t\t%u\n", q_res.children[i]);
+	    }
+	}
+	printf("\t)\n");
+    }
+
+    VDisconnectFS();
+
+    return 0;
+}
+
+static int
+VGCAdd(struct cmd_syndesc * as, void * rock)
+{
+    afs_int32 code;
+    struct fssync_state state;
+    char * partName;
+    VolumeId parent, child;
+    struct cmd_item *ti;
+    SYNC_response res;
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+0].items)) {	/* -partition */
+	return -1;
+    }
+    partName = ti->data;
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+1].items)) {	/* -parent */
+	return -1;
+    }
+    parent = atoi(ti->data);
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+2].items)) {	/* -child */
+	return -1;
+    }
+    child = atoi(ti->data);
+
+    common_prolog(as, &state);
+    fprintf(stderr, "calling FSYNC_VCGAdd\n");
+    code = FSYNC_VGCAdd(partName, parent, child, state.reason, &res);
+    debug_response(code, &res);
+
+    VDisconnectFS();
+
+    return 0;
+}
+
+static int
+VGCDel(struct cmd_syndesc * as, void * rock)
+{
+    afs_int32 code;
+    struct fssync_state state;
+    char * partName;
+    VolumeId parent, child;
+    struct cmd_item *ti;
+    SYNC_response res;
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+0].items)) {	/* -partition */
+	return -1;
+    }
+    partName = ti->data;
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+1].items)) {	/* -parent */
+	return -1;
+    }
+    parent = atoi(ti->data);
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+2].items)) {	/* -child */
+	return -1;
+    }
+    child = atoi(ti->data);
+
+    common_prolog(as, &state);
+    fprintf(stderr, "calling FSYNC_VCGDel\n");
+    code = FSYNC_VGCDel(partName, parent, child, state.reason, &res);
+    debug_response(code, &res);
+
+    VDisconnectFS();
+
+    return 0;
+}
+
+static int
+VGCScan(struct cmd_syndesc * as, void * rock)
+{
+    afs_int32 code;
+    struct fssync_state state;
+    char * partName;
+    struct cmd_item *ti;
+
+    if (!(ti = as->parms[CUSTOM_PARMS_OFFSET+0].items)) {	/* -partition */
+	return -1;
+    }
+    partName = ti->data;
+
+    common_prolog(as, &state);
+    fprintf(stderr, "calling FSYNC_VCGScan\n");
+    code = FSYNC_VGCScan(partName, state.reason);
+    debug_response(code, NULL);
+
+    VDisconnectFS();
+
+    return 0;
+}
+
+static int
+VGCScanAll(struct cmd_syndesc * as, void * rock)
+{
+    afs_int32 code;
+    struct fssync_state state;
+
+    common_prolog(as, &state);
+    fprintf(stderr, "calling FSYNC_VCGScanAll\n");
+    code = FSYNC_VGCScan(NULL, state.reason);
+    debug_response(code, NULL);
+
+    VDisconnectFS();
+
+    return 0;
+}
