@@ -1,12 +1,12 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  *
- * Portions Copyright (c) 2006-2008 Sine Nomine Associates
+ * Portions Copyright (c) 2006-2010 Sine Nomine Associates
  */
 
 /*
@@ -43,35 +43,17 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-
-#include <sys/types.h>
-#include <stdio.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#include <time.h>
-#else
-#include <sys/param.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <sys/time.h>
-#include <unistd.h>
-#endif
-#include <errno.h>
-#ifdef AFS_PTHREAD_ENV
-#include <assert.h>
-#else /* AFS_PTHREAD_ENV */
-#include <afs/assert.h>
-#endif /* AFS_PTHREAD_ENV */
-#include <signal.h>
-#include <string.h>
+#include <roken.h>
 
 #include <rx/xdr.h>
 #include <afs/afsint.h>
 #include "nfs.h"
 #include <afs/errors.h>
 #include "daemon_com.h"
+#include "daemon_com_inline.h"
 #include "fssync.h"
+#include "fssync_inline.h"
+#include "salvsync.h"
 #include "lwp.h"
 #include "lock.h"
 #include <afs/afssyscalls.h>
@@ -80,6 +62,8 @@
 #include "volume.h"
 #include "volume_inline.h"
 #include "partition.h"
+#include "vg_cache.h"
+#include "common.h"
 
 #ifdef HAVE_POLL
 #include <sys/poll.h>
@@ -92,8 +76,6 @@
 
 #ifdef FSSYNC_BUILD_SERVER
 
-/*@printflike@*/ extern void Log(const char *format, ...);
-
 int (*V_BreakVolumeCallbacks) (VolumeId volume);
 
 #define MAXHANDLERS	4	/* Up to 4 clients; must be at least 2, so that
@@ -111,8 +93,8 @@ static struct offlineInfo OfflineVolumes[MAXHANDLERS][MAXOFFLINEVOLUMES];
 /**
  * fssync server socket handle.
  */
-static SYNC_server_state_t fssync_server_state = 
-    { -1,                       /* file descriptor */
+static SYNC_server_state_t fssync_server_state =
+    { OSI_NULLSOCKET,                       /* file descriptor */
       FSSYNC_ENDPOINT_DECL,     /* server endpoint */
       FSYNC_PROTO_VERSION,      /* protocol version */
       5,                        /* bind() retry limit */
@@ -120,6 +102,23 @@ static SYNC_server_state_t fssync_server_state =
       "FSSYNC",                 /* protocol name string */
     };
 
+#ifdef AFS_DEMAND_ATTACH_FS
+/**
+ * a queue of volume pointers to salvage in the background.
+ */
+struct fsync_salv_node {
+    struct rx_queue q;
+    Volume *vp;                     /**< volume to salvage */
+    unsigned char update_salv_prio; /**< whether we should update the salvage priority or not */
+};
+static struct {
+    struct rx_queue head;
+    pthread_cond_t cv;
+} fsync_salv;
+
+static void * FSYNC_salvageThread(void *);
+static void FSYNC_backgroundSalvage(Volume *vp);
+#endif /* AFS_DEMAND_ATTACH_FS */
 
 /* Forward declarations */
 static void * FSYNC_sync(void *);
@@ -155,7 +154,11 @@ static afs_int32 FSYNC_com_VolDone(FSSYNC_VolOp_command * com, SYNC_response * r
 static afs_int32 FSYNC_com_VolQuery(FSSYNC_VolOp_command * com, SYNC_response * res);
 static afs_int32 FSYNC_com_VolHdrQuery(FSSYNC_VolOp_command * com, SYNC_response * res);
 #ifdef AFS_DEMAND_ATTACH_FS
+static afs_int32 FSYNC_com_VGUpdate(osi_socket fd, SYNC_command * com, SYNC_response * res);
 static afs_int32 FSYNC_com_VolOpQuery(FSSYNC_VolOp_command * com, SYNC_response * res);
+static afs_int32 FSYNC_com_VGQuery(FSSYNC_VolOp_command * com, SYNC_response * res);
+static afs_int32 FSYNC_com_VGScan(FSSYNC_VolOp_command * com, SYNC_response * res);
+static afs_int32 FSYNC_com_VGScanAll(FSSYNC_VolOp_command * com, SYNC_response * res);
 #endif /* AFS_DEMAND_ATTACH_FS */
 
 static afs_int32 FSYNC_com_VnQry(osi_socket fd, SYNC_command * com, SYNC_response * res);
@@ -195,14 +198,20 @@ FSYNC_fsInit(void)
     Lock_Init(&FSYNC_handler_lock);
 
 #ifdef AFS_PTHREAD_ENV
-    assert(pthread_attr_init(&tattr) == 0);
-    assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
-    assert(pthread_create(&tid, &tattr, FSYNC_sync, NULL) == 0);
+    osi_Assert(pthread_attr_init(&tattr) == 0);
+    osi_Assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
+    osi_Assert(pthread_create(&tid, &tattr, FSYNC_sync, NULL) == 0);
 #else /* AFS_PTHREAD_ENV */
-    assert(LWP_CreateProcess
+    osi_Assert(LWP_CreateProcess
 	   (FSYNC_sync, USUAL_STACK_SIZE, USUAL_PRIORITY, (void *)0,
 	    "FSYNC_sync", &pid) == LWP_SUCCESS);
 #endif /* AFS_PTHREAD_ENV */
+
+#ifdef AFS_DEMAND_ATTACH_FS
+    queue_Init(&fsync_salv.head);
+    CV_INIT(&fsync_salv.cv, "fsync salv", CV_DEFAULT, 0);
+    osi_Assert(pthread_create(&tid, &tattr, FSYNC_salvageThread, NULL) == 0);
+#endif /* AFS_DEMAND_ATTACH_FS */
 }
 
 #if defined(HAVE_POLL) && defined(AFS_PTHREAD_ENV)
@@ -233,6 +242,10 @@ FSYNC_sync(void * args)
     int min_vinit = 1;
 #endif /* AFS_DEMAND_ATTACH_FS */
 
+    /* we must not be called before vol package initialization, since we use
+     * vol package mutexes and conds etc */
+    osi_Assert(VInit);
+
     SYNC_getAddr(&state->endpoint, &state->addr);
     SYNC_cleanupSock(state);
 
@@ -242,27 +255,30 @@ FSYNC_sync(void * args)
 
 #ifdef AFS_PTHREAD_ENV
     /* set our 'thread-id' so that the host hold table works */
-    MUTEX_ENTER(&rx_stats_mutex);	/* protects rxi_pthread_hinum */
-    tid = ++rxi_pthread_hinum;
-    MUTEX_EXIT(&rx_stats_mutex);
-    pthread_setspecific(rx_thread_id_key, (void *)(intptr_t)tid);
+    tid = rx_SetThreadNum();
     Log("Set thread id %d for FSYNC_sync\n", tid);
+    afs_pthread_setname_self("FSYNC_sync");
 #endif /* AFS_PTHREAD_ENV */
 
+    VOL_LOCK;
+
     while (VInit < min_vinit) {
 	/* Let somebody else run until all volumes have been preattached
 	 * (DAFS), or we have started attaching volumes (non-DAFS). This
 	 * doesn't mean that all volumes have been attached.
 	 */
 #ifdef AFS_PTHREAD_ENV
-	pthread_yield();
+	VOL_CV_WAIT(&vol_vinit_cond);
 #else /* AFS_PTHREAD_ENV */
 	LWP_DispatchProcess();
 #endif /* AFS_PTHREAD_ENV */
     }
+
+    VOL_UNLOCK;
+
     state->fd = SYNC_getSock(&state->endpoint);
     code = SYNC_bindSock(state);
-    assert(!code);
+    osi_Assert(!code);
 
 #ifdef AFS_DEMAND_ATTACH_FS
     /*
@@ -277,7 +293,10 @@ FSYNC_sync(void * args)
     }
     memcpy(thread_opts, &VThread_defaults, sizeof(VThread_defaults));
     thread_opts->disallow_salvsync = 1;
-    assert(pthread_setspecific(VThread_key, thread_opts) == 0);
+    osi_Assert(pthread_setspecific(VThread_key, thread_opts) == 0);
+
+    code = VVGCache_PkgInit();
+    osi_Assert(code == 0);
 #endif
 
     InitHandler();
@@ -291,12 +310,17 @@ FSYNC_sync(void * args)
 	    CallHandler(FSYNC_readfds, nfds, POLLIN|POLLPRI);
 #else
 	int maxfd;
+#ifdef AFS_PTHREAD_ENV
+	struct timeval s_timeout;
+#endif
 	GetHandler(&FSYNC_readfds, &maxfd);
 	/* Note: check for >= 1 below is essential since IOMGR_select
 	 * doesn't have exactly same semantics as select.
 	 */
 #ifdef AFS_PTHREAD_ENV
-	if (select(maxfd + 1, &FSYNC_readfds, NULL, NULL, NULL) >= 1)
+	s_timeout.tv_sec = SYNC_SELECT_TIMEOUT;
+	s_timeout.tv_usec = 0;
+	if (select(maxfd + 1, &FSYNC_readfds, NULL, NULL, &s_timeout) >= 1)
 #else /* AFS_PTHREAD_ENV */
 	if (IOMGR_Select(maxfd + 1, &FSYNC_readfds, NULL, NULL, NULL) >= 1)
 #endif /* AFS_PTHREAD_ENV */
@@ -306,6 +330,91 @@ FSYNC_sync(void * args)
     return NULL; /* hush now, little gcc */
 }
 
+#ifdef AFS_DEMAND_ATTACH_FS
+/**
+ * thread for salvaging volumes in the background.
+ *
+ * Since FSSYNC handlers cannot issue SALVSYNC requests in order to avoid
+ * deadlock issues, this thread exists so code in the FSSYNC handler thread
+ * can hand off volumes to be salvaged in the background.
+ *
+ * @param[in] args  unused
+ *
+ * @note DEMAND_ATTACH_FS only
+ */
+static void *
+FSYNC_salvageThread(void * args)
+{
+    Volume *vp;
+    struct fsync_salv_node *node;
+
+    VOL_LOCK;
+
+    for (;;) {
+	while (queue_IsEmpty(&fsync_salv.head)) {
+	    VOL_CV_WAIT(&fsync_salv.cv);
+	}
+
+	node = queue_First(&fsync_salv.head, fsync_salv_node);
+	queue_Remove(node);
+
+	vp = node->vp;
+	if (node->update_salv_prio) {
+	    if (VUpdateSalvagePriority_r(vp)) {
+		ViceLog(0, ("FSYNC_salvageThread: unable to raise salvage priority "
+		            "for volume %lu\n", afs_printable_uint32_lu(vp->hashid)));
+	    }
+	}
+
+	free(node);
+	node = NULL;
+
+	VCancelReservation_r(vp);
+    }
+
+    VOL_UNLOCK;
+
+    return NULL;
+}
+
+/**
+ * salvage a volume in the background.
+ *
+ * Salvages cannot be scheduled directly from the main FSYNC thread, so
+ * instead call this function to schedule a salvage asynchronously in the
+ * FSYNC_salvageThread thread.
+ *
+ * @param[in] vp  volume to pointer to salvage
+ *
+ * @pre VOL_LOCK held
+ *
+ * @note DEMAND_ATTACH_FS only
+ */
+static void
+FSYNC_backgroundSalvage(Volume *vp)
+{
+    struct fsync_salv_node *node;
+    Error ec;
+
+    VCreateReservation_r(vp);
+
+    node = malloc(sizeof(struct fsync_salv_node));
+    node->vp = vp;
+
+    /* Save this value, to know if we should VUpdateSalvagePriority_r.
+     * We need to save it here, snce VRequestSalvage_r will change it. */
+    node->update_salv_prio = vp->salvage.requested;
+
+    if (VRequestSalvage_r(&ec, vp, SALVSYNC_ERROR, 0)) {
+	ViceLog(0, ("FSYNC_backgroundSalvage: unable to request salvage for volume %lu\n",
+	            afs_printable_uint32_lu(vp->hashid)));
+    }
+
+    queue_Append(&fsync_salv.head, node);
+    CV_BROADCAST(&fsync_salv.cv);
+}
+#endif /* AFS_DEMAND_ATTACH_FS */
+
 static void
 FSYNC_newconnection(osi_socket afd)
 {
@@ -318,12 +427,12 @@ FSYNC_newconnection(osi_socket afd)
     socklen_t junk;
     junk = sizeof(other);
     fd = accept(afd, (struct sockaddr *)&other, &junk);
-    if (fd == -1) {
+    if (fd == OSI_NULLSOCKET) {
 	Log("FSYNC_newconnection:  accept failed, errno==%d\n", errno);
-	assert(1 == 2);
+	osi_Assert(1 == 2);
     } else if (!AddHandler(fd, FSYNC_com)) {
 	AcceptOff();
-	assert(AddHandler(fd, FSYNC_com));
+	osi_Assert(AddHandler(fd, FSYNC_com));
     }
 }
 
@@ -376,6 +485,16 @@ FSYNC_com(osi_socket fd)
 	goto done;
     }
 
+    ViceLog(125, ("FSYNC_com: from fd %d got command %ld (%s) reason %ld (%s) "
+                  "pt %ld (%s) pid %ld\n", (int)fd,
+                  afs_printable_int32_ld(com.hdr.command),
+                  FSYNC_com2string(com.hdr.command),
+                  afs_printable_int32_ld(com.hdr.reason),
+                  FSYNC_reason2string(com.hdr.reason),
+                  afs_printable_int32_ld(com.hdr.programType),
+                  VPTypeToString(com.hdr.programType),
+                  afs_printable_int32_ld(com.hdr.pid)));
+
     res.hdr.com_seq = com.hdr.com_seq;
 
     VOL_LOCK;
@@ -392,7 +511,12 @@ FSYNC_com(osi_socket fd)
     case FSYNC_VOL_DONE:
     case FSYNC_VOL_QUERY:
     case FSYNC_VOL_QUERY_HDR:
+#ifdef AFS_DEMAND_ATTACH_FS
     case FSYNC_VOL_QUERY_VOP:
+    case FSYNC_VG_QUERY:
+    case FSYNC_VG_SCAN:
+    case FSYNC_VG_SCAN_ALL:
+#endif
 	res.hdr.response = FSYNC_com_VolOp(fd, &com, &res);
 	break;
     case FSYNC_VOL_STATS_GENERAL:
@@ -405,12 +529,25 @@ FSYNC_com(osi_socket fd)
     case FSYNC_VOL_QUERY_VNODE:
 	res.hdr.response = FSYNC_com_VnQry(fd, &com, &res);
 	break;
+#ifdef AFS_DEMAND_ATTACH_FS
+    case FSYNC_VG_ADD:
+    case FSYNC_VG_DEL:
+	res.hdr.response = FSYNC_com_VGUpdate(fd, &com, &res);
+	break;
+#endif
     default:
 	res.hdr.response = SYNC_BAD_COMMAND;
 	break;
     }
     VOL_UNLOCK;
 
+    ViceLog(125, ("FSYNC_com: fd %d responding with code %ld (%s) reason %ld "
+                  "(%s)\n", (int)fd,
+                  afs_printable_int32_ld(res.hdr.response),
+                  SYNC_res2string(res.hdr.response),
+                  afs_printable_int32_ld(res.hdr.reason),
+                  FSYNC_reason2string(res.hdr.reason)));
+
  respond:
     SYNC_putRes(&fssync_server_state, fd, &res);
 
@@ -447,6 +584,10 @@ FSYNC_com_VolOp(osi_socket fd, SYNC_command * com, SYNC_response * res)
 	}
     }
 
+    ViceLog(125, ("FSYNC_com_VolOp: fd %d got command for vol %lu part %.16s\n",
+                  (int)fd, afs_printable_uint32_lu(vcom.vop->volume),
+                  vcom.vop->partName));
+
     switch (com->hdr.command) {
     case FSYNC_VOL_ON:
     case FSYNC_VOL_ATTACH:
@@ -482,6 +623,15 @@ FSYNC_com_VolOp(osi_socket fd, SYNC_command * com, SYNC_response * res)
     case FSYNC_VOL_QUERY_VOP:
 	code = FSYNC_com_VolOpQuery(&vcom, res);
 	break;
+    case FSYNC_VG_QUERY:
+	code = FSYNC_com_VGQuery(&vcom, res);
+	break;
+    case FSYNC_VG_SCAN:
+	code = FSYNC_com_VGScan(&vcom, res);
+	break;
+    case FSYNC_VG_SCAN_ALL:
+	code = FSYNC_com_VGScanAll(&vcom, res);
+	break;
 #endif /* AFS_DEMAND_ATTACH_FS */
     default:
 	code = SYNC_BAD_COMMAND;
@@ -537,11 +687,11 @@ FSYNC_com_VolOn(FSSYNC_VolOp_command * vcom, SYNC_response * res)
     vp = VLookupVolume_r(&error, vcom->vop->volume, NULL);
     if (vp &&
 	FSYNC_partMatch(vcom, vp, 1) &&
-	vp->pending_vol_op && 
+	vp->pending_vol_op &&
 	(vcom->hdr->programType != vp->pending_vol_op->com.programType)) {
 	/* a different program has this volume checked out. deny. */
 	Log("FSYNC_VolOn: WARNING: program type %u has attempted to manipulate "
-	    "state for volume %u using command code %u while the volume is " 
+	    "state for volume %u using command code %u while the volume is "
 	    "checked out by program type %u for command code %u.\n",
 	    vcom->hdr->programType,
 	    vcom->vop->volume,
@@ -562,6 +712,10 @@ FSYNC_com_VolOn(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 	/* nothing much to do if we're leaving the volume offline */
 #ifdef AFS_DEMAND_ATTACH_FS
 	if (vp) {
+	    VCreateReservation_r(vp);
+	    VWaitExclusiveState_r(vp);
+	}
+	if (vp && V_attachState(vp) != VOL_STATE_DELETED) {
 	    if (FSYNC_partMatch(vcom, vp, 1)) {
 		if ((V_attachState(vp) == VOL_STATE_UNATTACHED) ||
 		    (V_attachState(vp) == VOL_STATE_PREATTACHED)) {
@@ -576,9 +730,14 @@ FSYNC_com_VolOn(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 		res->hdr.reason = FSYNC_WRONG_PART;
 	    }
 	} else {
-	    code = SYNC_DENIED;
+	    code = SYNC_FAILED;
 	    res->hdr.reason = FSYNC_UNKNOWN_VOLID;
 	}
+
+	if (vp) {
+	    VCancelReservation_r(vp);
+	    vp = NULL;
+	}
 #endif
 	goto done;
     }
@@ -589,10 +748,14 @@ FSYNC_com_VolOn(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 				vcom->vop->partName,
 				vcom->vop->volume);
     if (vp) {
+	VCreateReservation_r(vp);
+	VWaitExclusiveState_r(vp);
 	VDeregisterVolOp_r(vp);
+	VCancelReservation_r(vp);
+	vp = NULL;
     }
 #else /* !AFS_DEMAND_ATTACH_FS */
-    tvolName[0] = '/';
+    tvolName[0] = OS_DIRSEPC;
     snprintf(&tvolName[1], sizeof(tvolName)-1, VFORMAT, afs_printable_uint32_lu(vcom->vop->volume));
     tvolName[sizeof(tvolName)-1] = '\0';
 
@@ -624,7 +787,7 @@ FSYNC_com_VolOn(FSSYNC_VolOp_command * vcom, SYNC_response * res)
  * @note this is an FSYNC RPC server stub
  *
  * @note this procedure handles the following FSSYNC command codes:
- *       - FSYNC_VOL_OFF 
+ *       - FSYNC_VOL_OFF
  *       - FSYNC_VOL_NEEDVOLUME
  *
  * @note the supplementary reason code contains additional details.
@@ -699,10 +862,19 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 	/* enforce mutual exclusion for volume ops */
 	if (vp->pending_vol_op) {
 	    if (vp->pending_vol_op->com.programType != type) {
+                if (vp->pending_vol_op->com.command == FSYNC_VOL_OFF &&
+                    vp->pending_vol_op->com.reason == FSYNC_SALVAGE) {
+
+                    Log("denying offline request for volume %lu; volume is salvaging\n",
+		        afs_printable_uint32_lu(vp->hashid));
+
+                    res->hdr.reason = FSYNC_SALVAGE;
+                    goto deny;
+                }
 		Log("volume %u already checked out\n", vp->hashid);
 		/* XXX debug */
 		Log("vp->vop = { com = { ver=%u, prog=%d, com=%d, reason=%d, len=%u, flags=0x%x }, vop = { vol=%u, part='%s' } }\n",
-		    vp->pending_vol_op->com.proto_version, 
+		    vp->pending_vol_op->com.proto_version,
 		    vp->pending_vol_op->com.programType,
 		    vp->pending_vol_op->com.command,
 		    vp->pending_vol_op->com.reason,
@@ -735,29 +907,50 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 
 	/* filter based upon requestor
 	 *
-	 * volume utilities are not allowed to check out volumes
-	 * which are in an error state
+	 * volume utilities / volserver are not allowed to check out
+	 * volumes which are in an error state
 	 *
 	 * unknown utility programs will be denied on principal
 	 */
 	switch (type) {
 	case salvageServer:
-	    /* it is possible for the salvageserver to checkout a 
+	case volumeSalvager:
+	    /* it is possible for the salvageserver to checkout a
 	     * volume for salvage before its scheduling request
 	     * has been sent to the salvageserver */
 	    if (vp->salvage.requested && !vp->salvage.scheduled) {
 		vp->salvage.scheduled = 1;
 	    }
+
+	    /* If the volume is in VOL_STATE_SALVAGE_REQ, we need to wait
+	     * for the vol to go offline before we can give it away. Also
+	     * make sure we don't come out with vp in an excl state. */
+	    while (V_attachState(vp) == VOL_STATE_SALVAGE_REQ ||
+	           VIsExclusiveState(V_attachState(vp))) {
+
+		VOL_CV_WAIT(&V_attachCV(vp));
+	    }
+
 	case debugUtility:
 	    break;
 
 	case volumeUtility:
+	case volumeServer:
+            if (VIsSalvaging(vp)) {
+                Log("denying offline request for volume %lu; volume is in salvaging state\n",
+		    afs_printable_uint32_lu(vp->hashid));
+                res->hdr.reason = FSYNC_SALVAGE;
+
+		/* the volume hasn't been checked out yet by the salvager,
+		 * but we think the volume is salvaging; schedule a
+		 * a salvage to update the salvage priority */
+		FSYNC_backgroundSalvage(vp);
+
+		goto deny;
+            }
 	    if (VIsErrorState(V_attachState(vp))) {
 		goto deny;
 	    }
-            if (vp->salvage.requested) {
-                goto deny;
-            }
 	    break;
 
 	default:
@@ -779,6 +972,9 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 	     * attaching the volume would be safe */
 	    VRegisterVolOp_r(vp, &info);
 	    vp->pending_vol_op->vol_op_state = FSSYNC_VolOpRunningUnknown;
+	    /* fall through */
+
+	case VOL_STATE_DELETED:
 	    goto done;
 	default:
 	    break;
@@ -786,9 +982,6 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 
 	/* convert to heavyweight ref */
 	nvp = VGetVolumeByVp_r(&error, vp);
-	VCancelReservation_r(rvp);
-	rvp = NULL;
-
 	if (!nvp) {
             /*
              * It's possible for VGetVolumeByVp_r to have dropped and
@@ -809,13 +1002,16 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
                  * attaching the volume would be safe */
                 VRegisterVolOp_r(vp, &info);
                 vp->pending_vol_op->vol_op_state = FSSYNC_VolOpRunningUnknown;
+		/* fall through */
+
+            case VOL_STATE_DELETED:
                 goto done;
             default:
                 break;
             }
 
-	    Log("FSYNC_com_VolOff: failed to get heavyweight reference to volume %u\n",
-		vcom->vop->volume);
+	    Log("FSYNC_com_VolOff: failed to get heavyweight reference to volume %u (state=%u, flags=0x%x)\n",
+		vcom->vop->volume, V_attachState(vp), V_attachFlags(vp));
 	    res->hdr.reason = FSYNC_VOL_PKG_ERROR;
 	    goto deny;
 	} else if (nvp != vp) {
@@ -824,6 +1020,10 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 	    vp = nvp;
 	}
 
+	/* kill off lightweight ref to ensure we can't deadlock against ourselves later... */
+	VCancelReservation_r(rvp);
+	rvp = NULL;
+
 	/* register the volume operation metadata with the volume */
 	VRegisterVolOp_r(vp, &info);
 
@@ -834,11 +1034,12 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 	if (VVolOpLeaveOnline_r(vp, &info)) {
 	    VUpdateVolume_r(&error, vp, VOL_UPDATE_WAIT);	/* At least get volume stats right */
 	    if (LogLevel) {
-		Log("FSYNC: Volume %u (%s) was left on line for an external %s request\n", 
-		    V_id(vp), V_name(vp), 
-		    vcom->hdr->reason == V_CLONE ? "clone" : 
-		    vcom->hdr->reason == V_READONLY ? "readonly" : 
-		    vcom->hdr->reason == V_DUMP ? "dump" : 
+		Log("FSYNC: Volume %u (%s) was left on line for an external %s request\n",
+		    V_id(vp), V_name(vp),
+		    vcom->hdr->reason == V_CLONE ? "clone" :
+		    vcom->hdr->reason == V_READONLY ? "readonly" :
+		    vcom->hdr->reason == V_DUMP ? "dump" :
+		    vcom->hdr->reason == FSYNC_SALVAGE ? "salvage" :
 		    "UNKNOWN");
 	    }
 #ifdef AFS_DEMAND_ATTACH_FS
@@ -860,15 +1061,18 @@ FSYNC_com_VolOff(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 	    }
 
 #ifdef AFS_DEMAND_ATTACH_FS
+	    VCreateReservation_r(vp);
             VOfflineForVolOp_r(&error, vp, "A volume utility is running.");
             if (error==0) {
-                assert(vp->nUsers==0);
-                vp->pending_vol_op->vol_op_state = FSSYNC_VolOpRunningOffline; 
+                osi_Assert(vp->nUsers==0);
+                vp->pending_vol_op->vol_op_state = FSSYNC_VolOpRunningOffline;
             }
             else {
+		VWaitExclusiveState_r(vp);
 		VDeregisterVolOp_r(vp);
                 code = SYNC_DENIED;
             }
+	    VCancelReservation_r(vp);
 #else
 	    VOffline_r(vp, "A volume utility is running.");
 #endif
@@ -1001,10 +1205,8 @@ static afs_int32
 FSYNC_com_VolDone(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 {
     afs_int32 code = SYNC_FAILED;
-#ifdef AFS_DEMAND_ATTACH_FS
     Error error;
     Volume * vp;
-#endif
 
     if (SYNC_verifyProtocolString(vcom->vop->partName, sizeof(vcom->vop->partName))) {
 	res->hdr.reason = SYNC_REASON_MALFORMED_PACKET;
@@ -1017,19 +1219,47 @@ FSYNC_com_VolDone(FSSYNC_VolOp_command * vcom, SYNC_response * res)
     if (vcom->v)
 	vcom->v->volumeID = 0;
 
-#ifdef AFS_DEMAND_ATTACH_FS
     vp = VLookupVolume_r(&error, vcom->vop->volume, NULL);
     if (vp) {
 	if (FSYNC_partMatch(vcom, vp, 1)) {
+#ifdef AFS_DEMAND_ATTACH_FS
+	    VCreateReservation_r(vp);
+	    VWaitExclusiveState_r(vp);
+
 	    if ((V_attachState(vp) == VOL_STATE_UNATTACHED) ||
-		(V_attachState(vp) == VOL_STATE_PREATTACHED)) {
-		VChangeState_r(vp, VOL_STATE_UNATTACHED);
+		(V_attachState(vp) == VOL_STATE_PREATTACHED) ||
+		VIsErrorState(V_attachState(vp))) {
+
+		/* Change state to DELETED, not UNATTACHED, so clients get
+		 * a VNOVOL error when they try to access from now on. */
+
+		VChangeState_r(vp, VOL_STATE_DELETED);
 		VDeregisterVolOp_r(vp);
+
+		/* Volume is gone; clear out old salvage stats */
+		memset(&vp->salvage, 0, sizeof(vp->salvage));
+
+		/* Someday we should free the vp, too, after about 2 hours,
+		 * possibly by putting the vp back on the VLRU. */
+
 		code = SYNC_OK;
+	    } else if (V_attachState(vp) == VOL_STATE_DELETED) {
+		VDeregisterVolOp_r(vp);
+		res->hdr.reason = FSYNC_UNKNOWN_VOLID;
+
 	    } else {
 		code = SYNC_DENIED;
 		res->hdr.reason = FSYNC_BAD_STATE;
 	    }
+
+	    VCancelReservation_r(vp);
+	    vp = NULL;
+#else /* AFS_DEMAND_ATTACH_FS */
+	    if (!vp->specialStatus) {
+		vp->specialStatus = VNOVOL;
+	    }
+	    code = SYNC_OK;
+#endif /* !AFS_DEMAND_ATTACH_FS */
 	} else {
 	    code = SYNC_OK; /* XXX is this really a good idea? */
 	    res->hdr.reason = FSYNC_WRONG_PART;
@@ -1037,7 +1267,6 @@ FSYNC_com_VolDone(FSSYNC_VolOp_command * vcom, SYNC_response * res)
     } else {
 	res->hdr.reason = FSYNC_UNKNOWN_VOLID;
     }
-#endif
 
  done:
     return code;
@@ -1083,11 +1312,39 @@ FSYNC_com_VolError(FSSYNC_VolOp_command * vcom, SYNC_response * res)
     }
 
     vp = VLookupVolume_r(&error, vcom->vop->volume, NULL);
+
+    if (!vp && vcom->hdr->reason == FSYNC_SALVAGE) {
+	/* The requested volume doesn't seem to exist. However, it is possible
+	 * that this is triggered by trying to create or clone a volume that
+	 * was prevented from succeeding by a half-created volume in the way.
+	 * (e.g. we tried to create volume X, but volume X exists except that
+	 * its .vol header was deleted for some reason) So, still try to
+	 * a salvage for that volume ID. */
+
+	Log("FSYNC_com_VolError: attempting to schedule salvage for unknown "
+	    "volume %lu part %s\n", afs_printable_uint32_lu(vcom->vop->volume),
+	    vcom->vop->partName);
+	vp = VPreAttachVolumeById_r(&error, vcom->vop->partName,
+	                            vcom->vop->volume);
+    }
+
     if (vp) {
 	if (FSYNC_partMatch(vcom, vp, 0)) {
-	    /* null out salvsync control state, as it's no longer relevant */
-	    memset(&vp->salvage, 0, sizeof(vp->salvage));
-	    VChangeState_r(vp, VOL_STATE_ERROR);
+	    VCreateReservation_r(vp);
+	    VWaitExclusiveState_r(vp);
+            VDeregisterVolOp_r(vp);
+
+            if (vcom->hdr->reason == FSYNC_SALVAGE) {
+		FSYNC_backgroundSalvage(vp);
+            } else {
+		/* null out salvsync control state, as it's no longer relevant */
+		memset(&vp->salvage, 0, sizeof(vp->salvage));
+	        VChangeState_r(vp, VOL_STATE_ERROR);
+            }
+
+	    VCancelReservation_r(vp);
+	    vp = NULL;
+
 	    code = SYNC_OK;
 	} else {
 	    res->hdr.reason = FSYNC_WRONG_PART;
@@ -1279,18 +1536,176 @@ FSYNC_com_VolOpQuery(FSSYNC_VolOp_command * vcom, SYNC_response * res)
 
     vp = VLookupVolume_r(&error, vcom->vop->volume, NULL);
 
+    if (vp) {
+	VCreateReservation_r(vp);
+	VWaitExclusiveState_r(vp);
+    }
+
     if (vp && vp->pending_vol_op) {
-	assert(sizeof(FSSYNC_VolOp_info) <= res->payload.len);
-	memcpy(res->payload.buf, vp->pending_vol_op, sizeof(FSSYNC_VolOp_info));
-	res->hdr.response_len += sizeof(FSSYNC_VolOp_info);
-    } else {
-	if (vp) {
-	    res->hdr.reason = FSYNC_NO_PENDING_VOL_OP;
+	if (!FSYNC_partMatch(vcom, vp, 1)) {
+	    res->hdr.reason = FSYNC_WRONG_PART;
+	    code = SYNC_FAILED;
 	} else {
+	    osi_Assert(sizeof(FSSYNC_VolOp_info) <= res->payload.len);
+	    memcpy(res->payload.buf, vp->pending_vol_op, sizeof(FSSYNC_VolOp_info));
+	    res->hdr.response_len += sizeof(FSSYNC_VolOp_info);
+	}
+    } else {
+	if (!vp || V_attachState(vp) == VOL_STATE_DELETED) {
 	    res->hdr.reason = FSYNC_UNKNOWN_VOLID;
+	} else if (!FSYNC_partMatch(vcom, vp, 1)) {
+	    res->hdr.reason = FSYNC_WRONG_PART;
+	} else {
+	    res->hdr.reason = FSYNC_NO_PENDING_VOL_OP;
 	}
 	code = SYNC_FAILED;
     }
+
+    if (vp) {
+	VCancelReservation_r(vp);
+    }
+    return code;
+}
+
+static afs_int32
+FSYNC_com_VGQuery(FSSYNC_VolOp_command * vcom, SYNC_response * res)
+{
+    afs_int32 code = SYNC_FAILED;
+    int rc;
+    struct DiskPartition64 * dp;
+
+    if (SYNC_verifyProtocolString(vcom->vop->partName, sizeof(vcom->vop->partName))) {
+	res->hdr.reason = SYNC_REASON_MALFORMED_PACKET;
+	goto done;
+    }
+
+    dp = VGetPartition_r(vcom->vop->partName, 0);
+    if (dp == NULL) {
+	res->hdr.reason = FSYNC_BAD_PART;
+	goto done;
+    }
+
+    osi_Assert(sizeof(FSSYNC_VGQry_response_t) <= res->payload.len);
+
+    rc = VVGCache_query_r(dp, vcom->vop->volume, res->payload.buf);
+    switch (rc) {
+    case 0:
+	res->hdr.response_len += sizeof(FSSYNC_VGQry_response_t);
+	code = SYNC_OK;
+	break;
+    case EAGAIN:
+	res->hdr.reason = FSYNC_PART_SCANNING;
+	break;
+    case ENOENT:
+	res->hdr.reason = FSYNC_UNKNOWN_VOLID;
+	break;
+    default:
+	break;
+    }
+
+ done:
+    return code;
+}
+
+static afs_int32
+FSYNC_com_VGUpdate(osi_socket fd, SYNC_command * com, SYNC_response * res)
+{
+    afs_int32 code = SYNC_FAILED;
+    struct DiskPartition64 * dp;
+    FSSYNC_VGUpdate_command_t * vgucom;
+    int rc;
+
+    if (com->recv_len != (sizeof(com->hdr) + sizeof(*vgucom))) {
+	res->hdr.reason = SYNC_REASON_MALFORMED_PACKET;
+	res->hdr.flags |= SYNC_FLAG_CHANNEL_SHUTDOWN;
+	code = SYNC_COM_ERROR;
+	goto done;
+    }
+
+    vgucom = com->payload.buf;
+
+    ViceLog(125, ("FSYNC_com_VGUpdate: fd %d got command for parent %lu child "
+                  "%lu partName %.16s\n", (int)fd,
+                  afs_printable_uint32_lu(vgucom->parent),
+                  afs_printable_uint32_lu(vgucom->child),
+                  vgucom->partName));
+
+    if (SYNC_verifyProtocolString(vgucom->partName, sizeof(vgucom->partName))) {
+	res->hdr.reason = SYNC_REASON_MALFORMED_PACKET;
+	goto done;
+    }
+
+    dp = VGetPartition_r(vgucom->partName, 0);
+    if (dp == NULL) {
+	res->hdr.reason = FSYNC_BAD_PART;
+	goto done;
+    }
+
+    switch(com->hdr.command) {
+    case FSYNC_VG_ADD:
+	rc = VVGCache_entry_add_r(dp, vgucom->parent, vgucom->child, NULL);
+	break;
+
+    case FSYNC_VG_DEL:
+	rc = VVGCache_entry_del_r(dp, vgucom->parent, vgucom->child);
+	break;
+
+    default:
+	Log("FSYNC_com_VGUpdate called improperly\n");
+	rc = -1;
+	break;
+    }
+
+    /* EINVAL means the partition VGC doesn't exist at all; not really
+     * an error */
+    if (rc == 0 || rc == EINVAL) {
+	code = SYNC_OK;
+    }
+
+    if (rc == ENOENT) {
+	res->hdr.reason = FSYNC_UNKNOWN_VOLID;
+    } else {
+	res->hdr.reason = FSYNC_WHATEVER;
+    }
+
+ done:
+    return code;
+}
+
+static afs_int32
+FSYNC_com_VGScan(FSSYNC_VolOp_command * vcom, SYNC_response * res)
+{
+    afs_int32 code = SYNC_FAILED;
+    struct DiskPartition64 * dp;
+
+    if (SYNC_verifyProtocolString(vcom->vop->partName, sizeof(vcom->vop->partName))) {
+	res->hdr.reason = SYNC_REASON_MALFORMED_PACKET;
+	goto done;
+    }
+
+    dp = VGetPartition_r(vcom->vop->partName, 0);
+    if (dp == NULL) {
+	res->hdr.reason = FSYNC_BAD_PART;
+	goto done;
+    }
+
+    if (VVGCache_scanStart_r(dp) == 0) {
+	code = SYNC_OK;
+    }
+
+ done:
+    return code;
+}
+
+static afs_int32
+FSYNC_com_VGScanAll(FSSYNC_VolOp_command * com, SYNC_response * res)
+{
+    afs_int32 code = SYNC_FAILED;
+
+    if (VVGCache_scanStart_r(NULL) == 0) {
+	code = SYNC_OK;
+    }
+
     return code;
 }
 #endif /* AFS_DEMAND_ATTACH_FS */
@@ -1310,6 +1725,14 @@ FSYNC_com_VnQry(osi_socket fd, SYNC_command * com, SYNC_response * res)
 	return SYNC_COM_ERROR;
     }
 
+    ViceLog(125, ("FSYNC_com_VnQry: fd %d got command for vol %lu vnode %lu "
+                  "uniq %lu spare %lu partName %.16s\n", (int)fd,
+                  afs_printable_uint32_lu(qry->volume),
+                  afs_printable_uint32_lu(qry->vnode),
+                  afs_printable_uint32_lu(qry->unique),
+                  afs_printable_uint32_lu(qry->spare),
+                  qry->partName));
+
 #ifdef AFS_DEMAND_ATTACH_FS
     vp = VLookupVolume_r(&error, qry->volume, NULL);
 #else /* !AFS_DEMAND_ATTACH_FS */
@@ -1363,6 +1786,13 @@ FSYNC_com_StatsOp(osi_socket fd, SYNC_command * com, SYNC_response * res)
     scom.sop = (FSSYNC_StatsOp_hdr *) com->payload.buf;
     scom.com = com;
 
+    ViceLog(125, ("FSYNC_com_StatsOp: fd %d got command for stats: "
+                  "{vlru_generation = %lu, hash_bucket = %lu, partName = "
+                  "%.16s}\n", (int)fd,
+                  afs_printable_uint32_lu(scom.sop->args.vlru_generation),
+                  afs_printable_uint32_lu(scom.sop->args.hash_bucket),
+                  scom.sop->args.partName));
+
     switch (com->hdr.command) {
     case FSYNC_VOL_STATS_GENERAL:
 	code = FSYNC_com_StatsOpGeneral(&scom, res);
@@ -1425,7 +1855,7 @@ FSYNC_com_StatsOpViceP(FSSYNC_StatsOp_command * scom, SYNC_response * res)
 	stats->minFree = dp->minFree;
 	stats->f_files = dp->f_files;
 	stats->vol_list_len = dp->vol_list.len;
-	
+
 	res->hdr.response_len += sizeof(struct DiskPartitionStats64);
     }
 
@@ -1455,7 +1885,7 @@ FSYNC_com_StatsOpHash(FSSYNC_StatsOp_command * scom, SYNC_response * res)
     AssignInt64(head->reorders, &stats->chain_reorders);
 
     res->hdr.response_len += sizeof(struct VolumeHashChainStats);
-    
+
     return code;
 }
 
@@ -1501,7 +1931,7 @@ FSYNC_com_to_info(FSSYNC_VolOp_command * vcom, FSSYNC_VolOp_info * info)
 }
 
 /**
- * check whether command packet partition name matches volume 
+ * check whether command packet partition name matches volume
  * object's partition name.
  *
  * @param[in] vcom        pointer to command packet
@@ -1518,11 +1948,11 @@ FSYNC_com_to_info(FSSYNC_VolOp_command * vcom, FSSYNC_VolOp_info * info)
  *
  * @internal
  */
-static int 
+static int
 FSYNC_partMatch(FSSYNC_VolOp_command * vcom, Volume * vp, int match_anon)
 {
     return ((match_anon && vcom->vop->partName[0] == 0) ||
-	    (strncmp(vcom->vop->partName, V_partition(vp)->name, 
+	    (strncmp(vcom->vop->partName, V_partition(vp)->name,
 		     sizeof(vcom->vop->partName)) == 0));
 }
 
@@ -1542,7 +1972,7 @@ FSYNC_Drop(osi_socket fd)
 
 	    Volume *vp;
 
-	    tvolName[0] = '/';
+	    tvolName[0] = OS_DIRSEPC;
 	    sprintf(&tvolName[1], VFORMAT, afs_printable_uint32_lu(p[i].volumeID));
 	    vp = VAttachVolumeByName_r(&error, p[i].partName, tvolName,
 				       V_VOLUPD);
@@ -1553,11 +1983,7 @@ FSYNC_Drop(osi_socket fd)
     }
     VOL_UNLOCK;
     RemoveHandler(fd);
-#ifdef AFS_NT40_ENV
-    closesocket(fd);
-#else
-    close(fd);
-#endif
+    rk_closesocket(fd);
     AcceptOn();
 }
 
@@ -1567,7 +1993,7 @@ static void
 AcceptOn(void)
 {
     if (AcceptHandler == -1) {
-	assert(AddHandler(fssync_server_state.fd, FSYNC_newconnection));
+	osi_Assert(AddHandler(fssync_server_state.fd, FSYNC_newconnection));
 	AcceptHandler = FindHandler(fssync_server_state.fd);
     }
 }
@@ -1576,7 +2002,7 @@ static void
 AcceptOff(void)
 {
     if (AcceptHandler != -1) {
-	assert(RemoveHandler(fssync_server_state.fd));
+	osi_Assert(RemoveHandler(fssync_server_state.fd));
 	AcceptHandler = -1;
     }
 }
@@ -1589,10 +2015,10 @@ static void (*HandlerProc[MAXHANDLERS]) (osi_socket);
 static void
 InitHandler(void)
 {
-    register int i;
+    int i;
     ObtainWriteLock(&FSYNC_handler_lock);
     for (i = 0; i < MAXHANDLERS; i++) {
-	HandlerFD[i] = -1;
+	HandlerFD[i] = OSI_NULLSOCKET;
 	HandlerProc[i] = 0;
     }
     ReleaseWriteLock(&FSYNC_handler_lock);
@@ -1619,7 +2045,7 @@ CallHandler(struct pollfd *fds, int nfds, int mask)
 static void
 CallHandler(fd_set * fdsetp)
 {
-    register int i;
+    int i;
     ObtainReadLock(&FSYNC_handler_lock);
     for (i = 0; i < MAXHANDLERS; i++) {
 	if (HandlerFD[i] >= 0 && FD_ISSET(HandlerFD[i], fdsetp)) {
@@ -1635,10 +2061,10 @@ CallHandler(fd_set * fdsetp)
 static int
 AddHandler(osi_socket afd, void (*aproc) (osi_socket))
 {
-    register int i;
+    int i;
     ObtainWriteLock(&FSYNC_handler_lock);
     for (i = 0; i < MAXHANDLERS; i++)
-	if (HandlerFD[i] == -1)
+	if (HandlerFD[i] == OSI_NULLSOCKET)
 	    break;
     if (i >= MAXHANDLERS) {
 	ReleaseWriteLock(&FSYNC_handler_lock);
@@ -1651,9 +2077,9 @@ AddHandler(osi_socket afd, void (*aproc) (osi_socket))
 }
 
 static int
-FindHandler(register osi_socket afd)
+FindHandler(osi_socket afd)
 {
-    register int i;
+    int i;
     ObtainReadLock(&FSYNC_handler_lock);
     for (i = 0; i < MAXHANDLERS; i++)
 	if (HandlerFD[i] == afd) {
@@ -1661,27 +2087,27 @@ FindHandler(register osi_socket afd)
 	    return i;
 	}
     ReleaseReadLock(&FSYNC_handler_lock);	/* just in case */
-    assert(1 == 2);
+    osi_Assert(1 == 2);
     return -1;			/* satisfy compiler */
 }
 
 static int
-FindHandler_r(register osi_socket afd)
+FindHandler_r(osi_socket afd)
 {
-    register int i;
+    int i;
     for (i = 0; i < MAXHANDLERS; i++)
 	if (HandlerFD[i] == afd) {
 	    return i;
 	}
-    assert(1 == 2);
+    osi_Assert(1 == 2);
     return -1;			/* satisfy compiler */
 }
 
 static int
-RemoveHandler(register osi_socket afd)
+RemoveHandler(osi_socket afd)
 {
     ObtainWriteLock(&FSYNC_handler_lock);
-    HandlerFD[FindHandler_r(afd)] = -1;
+    HandlerFD[FindHandler_r(afd)] = OSI_NULLSOCKET;
     ReleaseWriteLock(&FSYNC_handler_lock);
     return 1;
 }
@@ -1694,8 +2120,8 @@ GetHandler(struct pollfd *fds, int maxfds, int events, int *nfds)
     int fdi = 0;
     ObtainReadLock(&FSYNC_handler_lock);
     for (i = 0; i < MAXHANDLERS; i++)
-	if (HandlerFD[i] != -1) {
-	    assert(fdi<maxfds);
+	if (HandlerFD[i] != OSI_NULLSOCKET) {
+	    osi_Assert(fdi<maxfds);
 	    fds[fdi].fd = HandlerFD[i];
 	    fds[fdi].events = events;
 	    fds[fdi].revents = 0;
@@ -1708,12 +2134,12 @@ GetHandler(struct pollfd *fds, int maxfds, int events, int *nfds)
 static void
 GetHandler(fd_set * fdsetp, int *maxfdp)
 {
-    register int i;
-    register int maxfd = -1;
+    int i;
+    int maxfd = -1;
     FD_ZERO(fdsetp);
     ObtainReadLock(&FSYNC_handler_lock);	/* just in case */
     for (i = 0; i < MAXHANDLERS; i++)
-	if (HandlerFD[i] != -1) {
+	if (HandlerFD[i] != OSI_NULLSOCKET) {
 	    FD_SET(HandlerFD[i], fdsetp);
 #ifndef AFS_NT40_ENV
             /* On Windows the nfds parameter to select() is ignored */
