@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -16,20 +16,13 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <stdio.h>
-#ifdef AFS_NT40_ENV
-#include <fcntl.h>
-#include <io.h>
-#else
-#include <sys/param.h>
+#ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
-#include <sys/time.h>
-#include <unistd.h>
 #endif
-#include <string.h>
-#include <sys/stat.h>
-#include <afs/assert.h>
+
+#include <afs/afs_assert.h>
 #include <afs/afsutil.h>
 
 #include <rx/xdr.h>
@@ -48,26 +41,31 @@
 #include "partition.h"
 #include "daemon_com.h"
 #include "fssync.h"
+#include "common.h"
 
 /* forward declarations */
 static int ObliterateRegion(Volume * avp, VnodeClass aclass, StreamHandle_t * afile,
-			    afs_int32 * aoffset);
+			    afs_foff_t * aoffset);
 #if 0
 static void PurgeIndex(Volume * vp, VnodeClass class);
 static void PurgeHeader(Volume * vp);
-#endif 
+#endif
 
 static void PurgeIndex_r(Volume * vp, VnodeClass class);
 static void PurgeHeader_r(Volume * vp);
 
 /* No lock needed. Only the volserver will call this, and only one transaction
- * can have a given volume (volid/partition pair) in use at a time 
+ * can have a given volume (volid/partition pair) in use at a time
  */
 void
 VPurgeVolume(Error * ec, Volume * vp)
 {
     struct DiskPartition64 *tpartp = vp->partition;
-    char purgePath[MAXPATHLEN];
+    VolumeId volid, parent;
+    afs_int32 code;
+
+    volid = V_id(vp);
+    parent = V_parentId(vp);
 
     /* so VCheckDetach doesn't try to update the volume header and
      * dump spurious errors into the logs */
@@ -77,13 +75,17 @@ VPurgeVolume(Error * ec, Volume * vp)
      * volume header. This routine can, under some circumstances, be called
      * when two volumes with the same id exist on different partitions.
      */
-    (void)afs_snprintf(purgePath, sizeof purgePath, "%s/%s",
-		       VPartitionPath(vp->partition),
-		       VolumeExternalName(V_id(vp)));
     PurgeIndex_r(vp, vLarge);
     PurgeIndex_r(vp, vSmall);
     PurgeHeader_r(vp);
-    unlink(purgePath);
+
+    code = VDestroyVolumeDiskHeader(tpartp, volid, parent);
+    if (code) {
+	Log("VPurgeVolume: Error %ld when destroying volume %lu header\n",
+	    afs_printable_int32_ld(code),
+	    afs_printable_uint32_lu(volid));
+    }
+
     /*
      * Call the fileserver to break all call backs for that volume
      */
@@ -97,17 +99,17 @@ VPurgeVolume(Error * ec, Volume * vp)
    and otherwise doesn't touch it */
 static int
 ObliterateRegion(Volume * avp, VnodeClass aclass, StreamHandle_t * afile,
-		 afs_int32 * aoffset)
+		 afs_foff_t * aoffset)
 {
-    register struct VnodeClassInfo *vcp;
+    struct VnodeClassInfo *vcp;
     Inode inodes[MAXOBLITATONCE];
-    register afs_int32 iindex, nscanned;
-    afs_int32 offset;
+    afs_int32 iindex, nscanned;
+    afs_foff_t offset;
     char buf[SIZEOF_LARGEDISKVNODE];
     int hitEOF;
-    register int i;
-    register afs_int32 code;
-    register struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
+    int i;
+    afs_int32 code;
+    struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
 
     hitEOF = 0;
     vcp = &VnodeClassInfo[aclass];
@@ -118,7 +120,7 @@ ObliterateRegion(Volume * avp, VnodeClass aclass, StreamHandle_t * afile,
      * We remember the inodes in an array, and idec them after zeroing them in the index.
      * The reason for these contortions is to make volume deletion idempotent, even
      * if we crash in the middle of a delete operation. */
-    STREAM_SEEK(afile, offset, 0);
+    STREAM_ASEEK(afile, offset);
     while (1) {
 	if (iindex >= MAXOBLITATONCE) {
 	    break;
@@ -139,7 +141,7 @@ ObliterateRegion(Volume * avp, VnodeClass aclass, StreamHandle_t * afile,
     }
 
     /* next, obliterate the index and fflush (and fsync) it */
-    STREAM_SEEK(afile, *aoffset, 0);	/* seek back to start of vnode index region */
+    STREAM_ASEEK(afile, *aoffset);	/* seek back to start of vnode index region */
     memset(buf, 0, sizeof(buf));	/* zero out our proto-vnode */
     for (i = 0; i < nscanned; i++) {
 	if (STREAM_WRITE(buf, vcp->diskSize, 1, afile) != 1)
@@ -177,8 +179,8 @@ PurgeIndex_r(Volume * vp, VnodeClass class)
 {
     StreamHandle_t *ifile;
     struct VnodeClassInfo *vcp = &VnodeClassInfo[class];
-    afs_int32 offset;
-    register afs_int32 code;
+    afs_foff_t offset;
+    afs_int32 code;
     FdHandle_t *fdP;
 
 
