@@ -26,25 +26,16 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <afs/procmgmt.h>
+#include <roken.h>
+
+#include <stddef.h>
 
-#include <sys/types.h>
-#include <stdio.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
-#include <time.h>
-#else
-#include <sys/param.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <sys/time.h>
 #endif
-#include <errno.h>
+
 #include <afs/afs_assert.h>
-#include <signal.h>
-#include <string.h>
-
-
 #include <rx/xdr.h>
 #include <afs/afsint.h>
 #include "nfs.h"
@@ -59,11 +50,6 @@
 #include "partition.h"
 #include "common.h"
 #include <rx/rx_queue.h>
-#include <afs/procmgmt.h>
-
-#if !defined(offsetof)
-#include <stddef.h>
-#endif
 
 #ifdef USE_UNIX_SOCKETS
 #include <afs/afsutil.h>
@@ -342,11 +328,14 @@ SALVSYNC_syncThread(void * args)
 
     for (;;) {
 	int maxfd;
+	struct timeval s_timeout;
 	GetHandler(&SALVSYNC_readfds, &maxfd);
+	s_timeout.tv_sec = SYNC_SELECT_TIMEOUT;
+	s_timeout.tv_usec = 0;
 	/* Note: check for >= 1 below is essential since IOMGR_select
 	 * doesn't have exactly same semantics as select.
 	 */
-	if (select(maxfd + 1, &SALVSYNC_readfds, NULL, NULL, NULL) >= 1)
+	if (select(maxfd + 1, &SALVSYNC_readfds, NULL, NULL, &s_timeout) >= 1)
 	    CallHandler(&SALVSYNC_readfds);
     }
 
@@ -762,11 +751,7 @@ static void
 SALVSYNC_Drop(osi_socket fd)
 {
     RemoveHandler(fd);
-#ifdef AFS_NT40_ENV
-    closesocket(fd);
-#else
-    close(fd);
-#endif
+    rk_closesocket(fd);
     AcceptOn();
 }
 
