@@ -10,22 +10,10 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <sys/types.h>
 #include <ctype.h>
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
-#ifdef AFS_NT40_ENV
-#include <fcntl.h>
-#else
-#include <sys/param.h>
-#include <sys/file.h>
-#include <sys/uio.h>
-#include <netinet/in.h>
-#include <unistd.h>
-#endif
-#include <sys/stat.h>
+
 #include <afs/afs_assert.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
@@ -38,12 +26,13 @@
 #include <afs/vnode.h>
 #include <afs/volume.h>
 #include <afs/partition.h>
-#include "dump.h"
 #include <afs/daemon_com.h>
 #include <afs/fssync.h>
 #include <afs/acl.h>
 #include <afs/com_err.h>
 #include <afs/vol_prototypes.h>
+
+#include "dump.h"
 #include "volser.h"
 #include "volint.h"
 #include "dumpstuff.h"
@@ -61,7 +50,6 @@
 /*@printflike@*/ extern void Log(const char *format, ...);
 
 extern int DoLogging;
-extern int DoPreserveVolumeStats;
 
 
 /* Forward Declarations */
@@ -292,8 +280,7 @@ ReadStandardTagLen(struct iod *iodp, unsigned char tag, afs_int32 section,
     afs_int32 code, i;
     afs_uint32 off = tag >> 5;
     afs_uint32 mask = 1 << (tag & 0x1f);
-    int len;
-    unsigned char buf[8], *p;
+    unsigned char len, buf[8], *p;
 
     if (!oldtagsInited)
         initNonStandardTags();
@@ -307,9 +294,7 @@ ReadStandardTagLen(struct iod *iodp, unsigned char tag, afs_int32 section,
     }
     if (tag <= MAX_TLV_TAG) {
         len = iod_getc(iodp);
-	if (len == EOF)
-	    return VOLSERDUMPERROR;
-	else if (len < 128)
+        if (len < 128)
             *length = len;
         else {
             len &= 0x7f;
@@ -545,8 +530,8 @@ DumpByte(struct iod *iodp, char tag, byte value)
     return ((iod_Write(iodp, tbuffer, 2) == 2) ? 0 : VOLSERDUMPERROR);
 }
 
-#define putint32(p, v)  *p++ = v>>24, *p++ = v>>16, *p++ = v>>8, *p++ = v
-#define putshort(p, v) *p++ = v>>8, *p++ = v
+#define afs_putint32(p, v)  *p++ = v>>24, *p++ = v>>16, *p++ = v>>8, *p++ = v
+#define afs_putshort(p, v) *p++ = v>>8, *p++ = v
 
 static int
 DumpDouble(struct iod *iodp, char tag, afs_uint32 value1,
@@ -555,8 +540,8 @@ DumpDouble(struct iod *iodp, char tag, afs_uint32 value1,
     char tbuffer[9];
     byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
-    putint32(p, value1);
-    putint32(p, value2);
+    afs_putint32(p, value1);
+    afs_putint32(p, value2);
     return ((iod_Write(iodp, tbuffer, 9) == 9) ? 0 : VOLSERDUMPERROR);
 }
 
@@ -566,7 +551,7 @@ DumpInt32(struct iod *iodp, char tag, afs_uint32 value)
     char tbuffer[5];
     byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
-    putint32(p, value);
+    afs_putint32(p, value);
     return ((iod_Write(iodp, tbuffer, 5) == 5) ? 0 : VOLSERDUMPERROR);
 }
 
@@ -579,7 +564,7 @@ DumpArrayInt32(struct iod *iodp, char tag,
     int code = 0;
     byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
-    putshort(p, nelem);
+    afs_putshort(p, nelem);
     code = iod_Write(iodp, tbuffer, 3);
     if (code != 3)
 	return VOLSERDUMPERROR;
@@ -587,7 +572,7 @@ DumpArrayInt32(struct iod *iodp, char tag,
 	p = (unsigned char *)tbuffer;
 	v = *array++;		/*this was register */
 
-	putint32(p, v);
+	afs_putint32(p, v);
 	code = iod_Write(iodp, tbuffer, 4);
 	if (code != 4)
 	    return VOLSERDUMPERROR;
@@ -1040,21 +1025,7 @@ DumpDumpHeader(struct iod *iodp, Volume * vp,
     if (!code)
 	code = DumpString(iodp, 'n', V_name(vp));
     dumpTimes[0] = fromtime;
-    switch (V_type(vp)) {
-    case readwriteVolume:
-	dumpTimes[1] = V_updateDate(vp);	/* until last update */
-	break;
-    case readonlyVolume:
-	dumpTimes[1] = V_creationDate(vp);	/* until clone was updated */
-	break;
-    case backupVolume:
-	/* until backup was made */
-	dumpTimes[1] = V_backupDate(vp) != 0 ? V_backupDate(vp) :
-					       V_creationDate(vp);
-	break;
-    default:
-	code = EINVAL;
-    }
+    dumpTimes[1] = V_backupDate(vp);	/* Until the time the clone was made */
     if (!code)
 	code = DumpArrayInt32(iodp, 't', (afs_uint32 *) dumpTimes, 2);
     return code;
@@ -1102,7 +1073,6 @@ DumpVnode(struct iod *iodp, struct VnodeDiskObject *v, int volid,
 			       VAclDiskSize(v));
     }
     if (VNDISK_GET_INO(v)) {
-	afs_sfsize_t indexlen, disklen;
 	IH_INIT(ihP, iodp->device, iodp->parentId, VNDISK_GET_INO(v));
 	fdP = IH_OPEN(ihP);
 	if (fdP == NULL) {
@@ -1110,17 +1080,6 @@ DumpVnode(struct iod *iodp, struct VnodeDiskObject *v, int volid,
 	    IH_RELEASE(ihP);
 	    return VOLSERREAD_DUMPERROR;
 	}
-	VNDISK_GET_LEN(indexlen, v);
-	disklen = FDH_SIZE(fdP);
-	if (indexlen != disklen) {
-	    FDH_REALLYCLOSE(fdP);
-	    IH_RELEASE(ihP);
-	    Log("DumpVnode: volume %lu vnode %lu has inconsistent length "
-	        "(index %lu disk %lu); aborting dump\n",
-	        (unsigned long)volid, (unsigned long)vnodeNumber,
-	        (unsigned long)indexlen, (unsigned long)disklen);
-	    return VOLSERREAD_DUMPERROR;
-	}
 	code = DumpFile(iodp, vnodeNumber, fdP);
 	FDH_CLOSE(fdP);
 	IH_RELEASE(ihP);
@@ -1188,6 +1147,10 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_foff_t ** Bufp, int *sizep,
 	    (size <=
 	     vcp->diskSize ? 0 : size - vcp->diskSize) >> vcp->logSize;
 	if (nVnodes > 0) {
+	    if (DoLogging) {
+		Log("RestoreVolume ProcessIndex: Set up %d inodes for volume %d\n",
+		    nVnodes, V_id(vp));
+	    }
 	    Buf = malloc(nVnodes * sizeof(afs_foff_t));
 	    if (Buf == NULL) {
 		STREAM_CLOSE(afile);
@@ -1207,6 +1170,9 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_foff_t ** Bufp, int *sizep,
 		}
 		offset += vcp->diskSize;
 	    }
+	    if (DoLogging) {
+		Log("RestoreVolume ProcessIndex: found %d inodes\n", cnt);
+	    }
 	    *Bufp = Buf;
 	    *sizep = nVnodes;
 	}
@@ -1231,16 +1197,11 @@ RestoreVolume(struct rx_call *call, Volume * avp, int incremental,
     afs_foff_t *b1 = NULL, *b2 = NULL;
     int s1 = 0, s2 = 0, delo = 0, tdelo;
     int tag;
-    VolumeDiskData saved_header;
 
     iod_Init(iodp, call);
 
     vp = avp;
 
-    if (DoPreserveVolumeStats) {
-	CopyVolumeStats(&V_disk(vp), &saved_header);
-    }
-
     if (!ReadDumpHeader(iodp, &header)) {
 	Log("1 Volser: RestoreVolume: Error reading header file for dump; aborted\n");
 	return VOLSERREAD_DUMPERROR;
@@ -1267,8 +1228,6 @@ RestoreVolume(struct rx_call *call, Volume * avp, int incremental,
     vol.cloneId = cookie->clone;
     vol.parentId = cookie->parent;
 
-    V_needsSalvaged(vp) = 0;
-
     tdelo = delo;
     while (1) {
 	if (ReadVnodes(iodp, vp, 0, b1, s1, b2, s2, tdelo)) {
@@ -1309,16 +1268,7 @@ RestoreVolume(struct rx_call *call, Volume * avp, int incremental,
     }
 
   clean:
-    if (DoPreserveVolumeStats) {
-	CopyVolumeStats(&saved_header, &vol);
-    } else {
-	ClearVolumeStats(&vol);
-    }
-    if (V_needsSalvaged(vp)) {
-	/* needsSalvaged may have been set while we tried to write volume data.
-	 * prevent it from getting overwritten. */
-	vol.needsSalvaged = V_needsSalvaged(vp);
-    }
+    ClearVolumeStats(&vol);
     CopyVolumeHeader(&vol, &V_disk(vp));
     V_destroyMe(vp) = 0;
     VUpdateVolume(&vupdate, vp);
@@ -1366,6 +1316,9 @@ ReadVnodes(struct iod *iodp, Volume * vp, int incremental,
 	if (!ReadInt32(iodp, &vnode->uniquifier))
 	    return VOLSERREAD_DUMPERROR;
 
+	if (DoLogging) {
+	    Log("ReadVnodes: setup %d/%d\n", vnodeNumber, vnode->uniquifier);
+	}
 	while ((tag = iod_getc(iodp)) > D_MAX && tag != EOF) {
 	    haveStuff = 1;
             if (critical)
@@ -1437,20 +1390,20 @@ ReadVnodes(struct iod *iodp, Volume * vp, int incremental,
 		    }
 		    saw_f = 1;
 
-		    tmpH =
-			IH_CREATE_INIT(V_linkHandle(vp), V_device(vp),
+		    ino =
+			IH_CREATE(V_linkHandle(vp), V_device(vp),
 				  VPartitionPath(V_partition(vp)), nearInode,
 				  V_parentId(vp), vnodeNumber,
 				  vnode->uniquifier, vnode->dataVersion);
-		    if (!tmpH) {
+		    if (!VALID_INO(ino)) {
 			Log("1 Volser: ReadVnodes: IH_CREATE: %s - restore aborted\n",
                             afs_error_message(errno));
 			V_needsSalvaged(vp) = 1;
 			return VOLSERREAD_DUMPERROR;
 		    }
-		    ino = tmpH->ih_ino;
 		    nearInode = ino;
 		    VNDISK_SET_INO(vnode, ino);
+		    IH_INIT(tmpH, vp->device, V_parentId(vp), ino);
 		    fdP = IH_OPEN(tmpH);
 		    if (fdP == NULL) {
 			Log("1 Volser: ReadVnodes: IH_OPEN: %s - restore aborted\n",
@@ -1550,7 +1503,6 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 
 
     *status = 0;
-#ifdef AFS_64BIT_ENV
     {
 	afs_uint32 filesize_high = 0L, filesize_low = 0L;
 	if (tag == 'h') {
@@ -1565,12 +1517,6 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 	}
 	FillInt64(filesize, filesize_high, filesize_low);
     }
-#else /* !AFS_64BIT_ENV */
-    if (!ReadInt32(iodp, &filesize)) {
-	*status = 1;
-	return (0);
-    }
-#endif /* !AFS_64BIT_ENV */
     p = (unsigned char *)malloc(size);
     if (p == NULL) {
 	*status = 2;
@@ -1861,11 +1807,7 @@ SizeDumpVnode(struct iod *iodp, struct VnodeDiskObject *v, int volid,
     }
 
     if (VNDISK_GET_INO(v)) {
-	VNDISK_GET_LEN(addvar, v);
-	if (v->vn_length_hi)
-	    addvar += 9;
-	else
-	    addvar += 5;
+	FillInt64(addvar,0, (v->length + 5));
 	AddUInt64(v_size->dump_size, addvar, &v_size->dump_size);
     }
     return code;
