@@ -12,28 +12,12 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <string.h>
-#include <errno.h>
-#ifdef AFS_NT40_ENV
-#include <stdlib.h>
-#include <fcntl.h>
-#include <winsock2.h>
-#else
-#include <sys/file.h>
-#include <netinet/in.h>
-#include <unistd.h>
-#endif
-
-#include <dirent.h>
-#include <sys/stat.h>
+#include <roken.h>
 
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rxkad.h>
 #include <afs/afsint.h>
-#include <signal.h>
 #include <afs/afs_assert.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
@@ -71,7 +55,6 @@
 
 extern int DoLogging;
 extern struct afsconf_dir *tdir;
-extern int DoPreserveVolumeStats;
 
 extern void LogError(afs_int32 errcode);
 
@@ -213,7 +196,7 @@ ConvertVolume(afs_uint32 avol, char *aname, afs_int32 asize)
     if (asize < 18)
 	return -1;
     /* It's better using the Generic VFORMAT since otherwise we have to make changes to too many places... The 14 char limitation in names hits us again in AIX; print in field of 9 digits (still 10 for the rest), right justified with 0 padding */
-    (void)afs_snprintf(aname, asize, VFORMAT, (unsigned long)avol);
+    snprintf(aname, asize, VFORMAT, (unsigned long)avol);
     return 0;
 }
 
@@ -343,7 +326,7 @@ ViceCreateRoot(Volume *vp)
     DirHandle dir;
     struct acl_accessList *ACL;
     AFSFid did;
-    Inode inodeNumber, nearInode AFS_UNUSED;
+    Inode inodeNumber, AFS_UNUSED nearInode;
     struct VnodeDiskObject *vnode;
     struct VnodeClassInfo *vcp = &VnodeClassInfo[vLarge];
     IHandle_t *h;
@@ -372,10 +355,10 @@ ViceCreateRoot(Volume *vp)
     did.Vnode = (VnodeId) 1;
     did.Unique = 1;
 
-    osi_Assert(!(MakeDir(&dir, (afs_int32 *)&did, (afs_int32 *)&did)));
+    osi_Assert(!(afs_dir_MakeDir(&dir, (afs_int32 *)&did, (afs_int32 *)&did)));
     DFlush();			/* flush all modified dir buffers out */
-    DZap((afs_int32 *)&dir);			/* Remove all buffers for this dir */
-    length = Length(&dir);	/* Remember size of this directory */
+    DZap(&dir);			/* Remove all buffers for this dir */
+    length = afs_dir_Length(&dir);	/* Remember size of this directory */
 
     FidZap(&dir);		/* Done with the dir handle obtained via SetSalvageDirHandle() */
 
@@ -432,7 +415,6 @@ SAFSVolPartitionInfo(struct rx_call *acid, char *pname, struct diskPartition
     struct diskPartition64 *dp = (struct diskPartition64 *)
 	malloc(sizeof(struct diskPartition64));
 
-    memset(partition, 0, sizeof(*partition));
     code = VolPartitionInfo(acid, pname, dp);
     if (!code) {
 	strncpy(partition->name, dp->name, 32);
@@ -452,7 +434,6 @@ SAFSVolPartitionInfo64(struct rx_call *acid, char *pname, struct diskPartition64
 {
     afs_int32 code;
 
-    memset(partition, 0, sizeof(*partition));
     code = VolPartitionInfo(acid, pname, partition);
     osi_auditU(acid, VS_ParInfEvent, code, AUD_STR, pname, AUD_END);
     return code;
@@ -464,6 +445,9 @@ VolPartitionInfo(struct rx_call *acid, char *pname, struct diskPartition64
 {
     struct DiskPartition64 *dp;
 
+/*
+    if (!afsconf_SuperUser(tdir, acid, caller)) return VOLSERBAD_ACCESS;
+*/
     VResetDiskUsage();
     dp = VGetPartition(pname, 0);
     if (dp) {
@@ -744,6 +728,8 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
     }
     newId = *newNumber;
 
+    if (newType != readonlyVolume && newType != backupVolume)
+	return EINVAL;
     tt = FindTrans(atrans);
     if (!tt)
 	return ENOENT;
@@ -770,6 +756,12 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
 	purgevp = NULL;
     }
     originalvp = tt->volume;
+    if ((V_type(originalvp) == backupVolume)
+	|| (V_type(originalvp) == readonlyVolume)) {
+	Log("1 Volser: Clone: The volume to be cloned must be a read/write; aborted\n");
+	error = EROFS;
+	goto fail;
+    }
     if ((V_destroyMe(originalvp) == DESTROY_ME) || !V_inService(originalvp)) {
 	Log("1 Volser: Clone: Volume %d is offline and cannot be cloned\n",
 	    V_id(originalvp));
@@ -787,8 +779,15 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
 	    error = EINVAL;
 	    goto fail;
 	}
-	if (V_parentId(originalvp) != V_parentId(purgevp)) {
-	    Log("1 Volser: Clone: Volume %u and volume %u were not originally cloned from the same parent; aborted\n", purgeId, tt->volid);
+	if (V_type(originalvp) == readonlyVolume
+	    && V_parentId(originalvp) != V_parentId(purgevp)) {
+	    Log("1 Volser: Clone: Volume %u and volume %u were not cloned from the same parent volume; aborted\n", tt->volid, purgeId);
+	    error = EXDEV;
+	    goto fail;
+	}
+	if (V_type(originalvp) == readwriteVolume
+	    && tt->volid != V_parentId(purgevp)) {
+	    Log("1 Volser: Clone: Volume %u was not originally cloned from volume %u; aborted\n", purgeId, tt->volid);
 	    error = EXDEV;
 	    goto fail;
 	}
@@ -799,18 +798,13 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
     salv_vp = originalvp;
 #endif
 
-    if (purgeId == newId) {
-	newvp = purgevp;
-    } else {
-	newvp =
-	    VCreateVolume(&error, originalvp->partition->name, newId,
-			  V_parentId(originalvp));
-	if (error) {
-	    Log("1 Volser: Clone: Couldn't create new volume %lu for parent %lu; clone aborted\n",
-		afs_printable_uint32_lu(newId), afs_printable_uint32_lu(V_parentId(originalvp)));
-	    newvp = (Volume *) 0;
-	    goto fail;
-	}
+    newvp =
+	VCreateVolume(&error, originalvp->partition->name, newId,
+		      V_parentId(originalvp));
+    if (error) {
+	Log("1 Volser: Clone: Couldn't create new volume; clone aborted\n");
+	newvp = (Volume *) 0;
+	goto fail;
     }
     if (newType == readonlyVolume)
 	V_cloneId(originalvp) = newId;
@@ -908,7 +902,6 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
     afs_int32 newType;
     struct volser_trans *tt, *ttc;
     char caller[MAXKTCNAMELEN];
-    VolumeDiskData saved_header;
 
     /*not a super user */
     if (!afsconf_SuperUser(tdir, acid, caller))
@@ -938,6 +931,12 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
     TSetRxCall(tt, acid, "ReClone");
 
     originalvp = tt->volume;
+    if ((V_type(originalvp) == backupVolume)
+	|| (V_type(originalvp) == readonlyVolume)) {
+	Log("1 Volser: Clone: The volume to be cloned must be a read/write; aborted\n");
+	error = EROFS;
+	goto fail;
+    }
     if ((V_destroyMe(originalvp) == DESTROY_ME) || !V_inService(originalvp)) {
 	Log("1 Volser: Clone: Volume %d is offline and cannot be cloned\n",
 	    V_id(originalvp));
@@ -959,14 +958,22 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
 	error = EXDEV;
 	goto fail;
     }
-    if (V_parentId(originalvp) != V_parentId(clonevp)) {
-	Log("1 Volser: Clone: Volume %u was not originally cloned from volume %u; aborted\n", cloneId, tt->volid);
+    if (V_type(clonevp) != readonlyVolume && V_type(clonevp) != backupVolume) {
+	Log("1 Volser: Clone: The \"recloned\" volume must be a read only volume; aborted\n");
+	error = EINVAL;
+	goto fail;
+    }
+    if (V_type(originalvp) == readonlyVolume
+	&& V_parentId(originalvp) != V_parentId(clonevp)) {
+	Log("1 Volser: Clone: Volume %u and volume %u were not cloned from the same parent volume; aborted\n", tt->volid, cloneId);
 	error = EXDEV;
 	goto fail;
     }
-
-    if (DoPreserveVolumeStats) {
-	CopyVolumeStats(&V_disk(clonevp), &saved_header);
+    if (V_type(originalvp) == readwriteVolume
+	&& tt->volid != V_parentId(clonevp)) {
+	Log("1 Volser: Clone: Volume %u was not originally cloned from volume %u; aborted\n", cloneId, tt->volid);
+	error = EXDEV;
+	goto fail;
     }
 
     error = 0;
@@ -991,20 +998,15 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
     }
     /* don't do strcpy onto diskstuff.name, it's still OK from 1st clone */
 
-    /* update the creationDate, since this represents the last cloning date
-     * for ROs. But do not update copyDate; let it stay so we can identify
-     * when the clone was first created. */
-    V_creationDate(clonevp) = time(0);
-    if (DoPreserveVolumeStats) {
-	CopyVolumeStats(&saved_header, &V_disk(clonevp));
-    } else {
-	ClearVolumeStats(&V_disk(clonevp));
-    }
+    /* pretend recloned volume is a totally new instance */
+    V_copyDate(clonevp) = time(0);
+    V_creationDate(clonevp) = V_copyDate(clonevp);
+    ClearVolumeStats(&V_disk(clonevp));
     V_destroyMe(clonevp) = 0;
     V_inService(clonevp) = 0;
     if (newType == backupVolume) {
-	V_backupDate(originalvp) = V_creationDate(clonevp);
-	V_backupDate(clonevp) = V_creationDate(clonevp);
+	V_backupDate(originalvp) = V_copyDate(clonevp);
+	V_backupDate(clonevp) = V_copyDate(clonevp);
     }
     V_inUse(clonevp) = 0;
     VUpdateVolume(&error, clonevp);
@@ -1567,11 +1569,6 @@ VolRestore(struct rx_call *acid, afs_int32 atrans, afs_int32 aflags,
 	TRELE(tt);
 	return ENOENT;
     }
-    if (DoLogging) {
-	char buffer[16];
-	Log("%s on %s is executing Restore %u\n", caller,
-	    callerAddress(acid, buffer), tt->volid);
-    }
     TSetRxCall(tt, acid, "Restore");
 
     DFlushVolume(V_parentId(tt->volume)); /* Ensure dir buffers get dropped */
@@ -1917,7 +1914,7 @@ XVolListPartitions(struct rx_call *acid, struct partEntries *pEntries)
 	pEntries->partEntries_val = (afs_int32 *) malloc(j * sizeof(int));
 	if (!pEntries->partEntries_val)
 	    return ENOMEM;
-	memcpy(pEntries->partEntries_val, partList.partId,
+	memcpy((char *)pEntries->partEntries_val, (char *)&partList,
 		j * sizeof(int));
 	pEntries->partEntries_len = j;
     } else {
@@ -1928,6 +1925,22 @@ XVolListPartitions(struct rx_call *acid, struct partEntries *pEntries)
 
 }
 
+/*extract the volume id from string vname. Its of the form " V0*<id>.vol  "*/
+afs_int32
+ExtractVolId(char vname[])
+{
+    int i;
+    char name[VOLSER_MAXVOLNAME + 1];
+
+    strcpy(name, vname);
+    i = 0;
+    while (name[i] == 'V' || name[i] == '0')
+	i++;
+
+    name[11] = '\0';		/* smash the "." */
+    return (atol(&name[i]));
+}
+
 /*return the name of the next volume header in the directory associated with dirp and dp.
 *the volume id is  returned in volid, and volume header name is returned in volname*/
 int
@@ -1938,7 +1951,7 @@ GetNextVol(DIR * dirp, char *volname, afs_uint32 * volid)
     dp = readdir(dirp);		/*read next entry in the directory */
     if (dp) {
 	if ((dp->d_name[0] == 'V') && !strcmp(&(dp->d_name[11]), VHDREXT)) {
-	    *volid = VolumeNumber(dp->d_name);
+	    *volid = ExtractVolId(dp->d_name);
 	    strcpy(volname, dp->d_name);
 	    return 0;		/*return the name of the file representing a volume */
 	} else {
@@ -2236,7 +2249,7 @@ GetVolInfo(afs_uint32 partId,
     ttc = NewTrans(volumeId, partId);
     if (!ttc) {
 	code = -3;
-	VOLINT_INFO_STORE(handle, status, VBUSY);
+	VOLINT_INFO_STORE(handle, status, VOLSERVOLBUSY);
 	VOLINT_INFO_STORE(handle, volid, volumeId);
 	goto drop;
     }
@@ -2244,15 +2257,8 @@ GetVolInfo(afs_uint32 partId,
     /* Get volume from volserver */
     if (mode == VOL_INFO_LIST_MULTIPLE)
 	tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
-    else {
-#ifdef AFS_DEMAND_ATTACH_FS
-	int mode = V_PEEK;
-#else
-	int mode = V_READONLY;   /* informs the fileserver to update the volume headers. */
-#endif
-	tv = VAttachVolumeByName_retry(&error, pname, volname, mode);
-    }
-
+    else
+	tv = VAttachVolumeByName_retry(&error, pname, volname, V_PEEK);
     if (error) {
 	Log("1 Volser: GetVolInfo: Could not attach volume %u (%s:%s) error=%d\n",
 	    volumeId, pname, volname, error);
@@ -2361,6 +2367,7 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid,
     DIR *dirp;
     afs_uint32 volid;
     int found = 0;
+    int code;
     volint_info_handle_t handle;
 
     volumeInfo->volEntries_val = (volintInfo *) malloc(sizeof(volintInfo));
@@ -2402,20 +2409,19 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid,
 	handle.volinfo_type = VOLINT_INFO_TYPE_BASE;
 	handle.volinfo_ptr.base = volumeInfo->volEntries_val;
 
-	/* The return code from GetVolInfo is ignored; there is no error from
-	 * it that results in the whole call being aborted. Any volume
-	 * attachment failures are reported in 'status' field in the
-	 * volumeInfo payload. */
-	GetVolInfo(partid,
-	           volid,
-	           pname,
-	           volname,
-	           &handle,
-	           VOL_INFO_LIST_SINGLE);
+	code = GetVolInfo(partid,
+			  volid,
+			  pname,
+			  volname,
+			  &handle,
+			  VOL_INFO_LIST_SINGLE);
     }
 
     closedir(dirp);
-    return (found) ? 0 : ENODEV;
+    if (found)
+        return code ? ENODEV: 0;
+    else
+        return ENODEV;
 }
 
 /*------------------------------------------------------------------------
@@ -2462,6 +2468,7 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
     DIR *dirp;			/*Partition directory ptr */
     afs_uint32 currVolID;	        /*Current volume ID */
     int found = 0;		/*Did we find the volume we need? */
+    int code;
     volint_info_handle_t handle;
 
     /*
@@ -2475,6 +2482,7 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
     memset(a_volumeXInfoP->volXEntries_val, 0, sizeof(volintXInfo)); /* Clear structure */
 
     a_volumeXInfoP->volXEntries_len = 1;
+    code = ENODEV;
 
     /*
      * If the partition name we've been given is bad, bogue out.
@@ -2530,16 +2538,13 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	handle.volinfo_type = VOLINT_INFO_TYPE_EXT;
 	handle.volinfo_ptr.ext = a_volumeXInfoP->volXEntries_val;
 
-	/* The return code from GetVolInfo is ignored; there is no error from
-	 * it that results in the whole call being aborted. Any volume
-	 * attachment failures are reported in 'status' field in the
-	 * volumeInfo payload. */
-	GetVolInfo(a_partID,
-	           a_volID,
-	           pname,
-	           volname,
-	           &handle,
-	           VOL_INFO_LIST_SINGLE);
+	code = GetVolInfo(a_partID,
+			  a_volID,
+			  pname,
+			  volname,
+			  &handle,
+			  VOL_INFO_LIST_SINGLE);
+
     }
 
     /*
@@ -2547,7 +2552,10 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
      * return the proper value.
      */
     closedir(dirp);
-    return (found) ? 0 : ENODEV;
+    if (found)
+        return code ? ENODEV: 0;
+    else
+        return ENODEV;
 }				/*SAFSVolXListOneVolume */
 
 /*returns all the volumes on partition partid. If flags = 1 then all the
@@ -2853,7 +2861,6 @@ VolMonitor(struct rx_call *acid, transDebugEntries *transInfo)
 	goto done;		/*no active transactions */
     for (tt = allTrans; tt; tt = nt) {	/*copy relevant info into pntr */
 	nt = tt->next;
-	memset(pntr, 0, sizeof(*pntr));
         VTRANS_OBJ_LOCK(tt);
 	pntr->tid = tt->tid;
 	pntr->time = tt->time;
